#!/bin/bash

GMMPS=`pwd`

###############################################################################
# 0: Check OS
###############################################################################

OS=`src/discoverOS.sh`
if [ $OS = "UNSUPPORTED" ]; then
    echo " "
    echo "####################################################"
    echo "GMMPS Installer: ERROR: Unsupported operating system"
    echo "####################################################"
    echo " "
    exit
elif [ $OS = "Darwin" ]; then
    libsuffix=".dylib"
else
    libsuffix=".so"
fi

###############################################################################
# 1: Check that the skycat libraries are available (and compatible)
###############################################################################

echo ""
echo "#########################################################################"
echo "GMMPS Installer: Looking for skycat libraries... this may take a while..."
echo "#########################################################################"
echo ""
sleep 1

cd src
skycatpath=`./check_skycat.sh`
tcltkpath=`./check_tcltk.sh`
cd ${GMMPS}

if [ ${skycatpath} == "unknown" ]; then
    echo ""
    echo "Skycat libraries were not found under /usr nor ${HOME}."
    echo "Please provide the path to your skycat installation"
    echo "(the directory with libskycat3.1.2${libsuffix}) below, or install skycat first!"
    echo ""
    echo -n "Skycat path [press ENTER]: "
    read skycatpath

    if [ ${skycatpath}_A == "_A" ]; then
	echo ""
	echo "ERROR: No skycat installation was found. Exit."
	exit
    fi

    newskycatpath=./check_skycat.sh ${skycatpath}
    if [ ${newskycatpath} == "unknown" ]; then
	echo "ERROR: One or more of the following libraries were not found:"
	echo "  " $skycatpath/libcat4.1.0${libsuffix}
	echo "  " $skycatpath/libastrotcl2.1.0${libsuffix}
	echo "  " $skycatpath/libtclutil2.1.0${libsuffix}
	echo "  " $skycatpath/libskycat3.1.2${libsuffix}
	echo ""
	exit
    fi
    
    # Check if the Tcl/Tk stuff is there as well
    newtcltkpath=./check_tcltk.sh ${skycatpath}
    if [ ${newtcltkpath} != "unknown" ]; then
	tcltkpath=${skycatpath}
    fi
fi

if [ ${tcltkpath} == "unknown" ]; then
    echo ""
    echo "Tcl/Tk libraries were not found under /usr nor ${HOME}."
    echo "Please provide the path to your Tcl/Tk installation"
    echo "(the directory with libBLT24${libsuffix}) below, or install skycat first!"
    echo ""
    echo -n "Tcl/Tk path [press ENTER]: "
    read tcltkpath

    if [ ${tcltkpath}_A == "_A" ]; then
	echo ""
	echo "ERROR: No skycat installation was found. Exit."
	exit
    fi

    ./check_skycat.sh ${skycatpath}
    if [ ${skycatpath} == "unknown" ]; then
	echo "ERROR: One or more of the following libraries were not found:"
	echo "  " $skycatpath/libBLT24${libsuffix}
	echo "  " $skycatpath/libtcl8.4${libsuffix}
	echo "  " $skycatpath/libtk8.4${libsuffix}
	echo ""
	exit
    fi
fi

echo ""
echo "Skycat libraries found in:"  $skycatpath
echo "tcl/Tk libraries found in:"  $tcltkpath
echo ""

# Linking the relevant libraries:
ln -sf ${skycatpath}/libastrotcl2.1.0${libsuffix} ${GMMPS}/lib.${OS}/
ln -sf ${skycatpath}/libtclutil2.1.0${libsuffix}  ${GMMPS}/lib.${OS}/
ln -sf ${skycatpath}/libcat4.1.0${libsuffix}      ${GMMPS}/lib.${OS}/
ln -sf ${tcltkpath}/libBLT24${libsuffix}          ${GMMPS}/lib.${OS}/
ln -sf ${tcltkpath}/libtcl8.4${libsuffix}         ${GMMPS}/lib.${OS}/
ln -sf ${tcltkpath}/libtk8.4${libsuffix}          ${GMMPS}/lib.${OS}/


###############################################################################
# 2: System setup
###############################################################################

cd ${GMMPS}
SETPLAT_STATUS="1"
if [ -d lib.$OS ]; then
    echo "Your system is identical to one for which pre-compiled binaries exist: $OS"
    echo "Linking bin.$OS to bin"
    echo "Linking lib.$OS to lib"
    ln -sf bin.$OS bin
    ln -sf lib.$OS lib
    SETPLAT_STATUS="0"
else
    echo "Scanning for matching platforms ..."
    ls -l | grep "bin\." > bindirlist
    exec < bindirlist
    while read bindir
    do
	libdir=`sed 's/bin/lib/g' $bindir`
	bindir/gmCat2Fits    | grep USAGE >  log 2> /dev/null
	bindir/gmConvert2Cat | grep USAGE >> log 2> /dev/null
	bindir/gmMakeMasks_2 | grep USAGE >> log 2> /dev/null
	bindir/gmmps_fov     | grep USAGE >> log 2> /dev/null
	bindir/gmmps_sel     | grep USAGE >> log 2> /dev/null
	bindir/gmOdf2Cat     | grep USAGE >> log 2> /dev/null
	nsuccess=`wc -l log | awk '{print $1}'`
	if [ $nsuccess == 6 ]; then
	    ln -sf $bindir bin
	    ln -sf $libdir lib
	    SETPLAT_STATUS="0"
	    break
	fi
    done
fi

if [ $SETPLAT_STATUS = "1" ]; then
    echo "ERROR: Could not find matching binaries for your platform, $OS."
    echo "Must compile GMMPS from source."
    exit
fi


echo " "
echo "############################################################################# "
echo "GMMPS Installer: GMMPS modules verified. Building the GMMPS startup script ..."
echo "############################################################################# "
echo " "

$GMMPS/src/build_gmmps $GMMPS $skycatpath $OS > $GMMPS/bin/gmmps

chmod a+x $GMMPS/bin/gmmps

echo
echo "########################################################################"
echo ""
echo "Finished. The GMMPS startup script is:"
echo "   $GMMPS/bin/gmmps"
echo ""
echo "Please add"
echo "   $GMMPS/bin/"
echo "to your PATH variable. Restart your shell and type 'gmmps' to run gmmps."
echo ""
echo "########################################################################"
echo
