#!/bin/bash

GMMPS=`pwd`

###############################################################################
# 0: Check OS
###############################################################################

OS=`src/discoverOS.sh`
if [ $OS = "UNSUPPORTED" ]; then
    echo " "
    echo "####################################################"
    echo "GMMPS Installer: ERROR: Unsupported operating system"
    echo "####################################################"
    echo " "
    exit
elif [ $OS = "Darwin" ]; then
    libsuffix=".dylib"
else
    libsuffix=".so"
fi


cd ${GMMPS}
echo " "
echo "You are compiling on this machine: ${OS}"
echo " "
echo "mkdir  ${GMMPS}/bin.$OS"
echo "mkdir  ${GMMPS}/lib.$OS"
echo "ln -sf bin.$OS/ bin"
echo "ln -sf lib.$OS/ lib"
test -e bin && \rm bin
test -e lib && \rm lib
test -d bin.$OS && rm -rf bin.$OS
test -d lib.$OS && rm -rf lib.$OS
mkdir -p bin.$OS lib.$OS
ln -sf bin.$OS bin
ln -sf lib.$OS lib


###############################################################################
# 1: Check that the skycat libraries are available (and compatible)
###############################################################################

echo ""
echo "#########################################################################"
echo "GMMPS Installer: Looking for skycat libraries... this may take a while..."
echo "#########################################################################"
echo ""
sleep 1

cd src
skycatpath=`./check_skycat.sh`
tcltkpath=`./check_tcltk.sh`
cd ${GMMPS}

if [ ${skycatpath} == "unknown" ]; then
    echo ""
    echo "Skycat libraries were not found under /usr nor ${HOME}."
    echo "Please provide the path to your skycat installation"
    echo "(the directory with libskycat3.1.2${libsuffix}) below, or install skycat first!"
    echo ""
    echo -n "Skycat path [press ENTER]: "
    read skycatpath

    if [ ${skycatpath}_A == "_A" ]; then
	echo ""
	echo "ERROR: No skycat installation was found. Exit."
	exit
    fi

    newskycatpath=./check_skycat.sh ${skycatpath}
    if [ ${newskycatpath} == "unknown" ]; then
	echo "ERROR: One or more of the following libraries were not found:"
	echo "  " $skycatpath/libcat4.1.0${libsuffix}
	echo "  " $skycatpath/libastrotcl2.1.0${libsuffix}
	echo "  " $skycatpath/libtclutil2.1.0${libsuffix}
	echo "  " $skycatpath/libskycat3.1.2${libsuffix}
	echo ""
	exit
    fi
    
    # Check if the Tcl/Tk stuff is there as well
    newtcltkpath=./check_tcltk.sh ${skycatpath}
    if [ ${newtcltkpath} != "unknown" ]; then
	tcltkpath=${skycatpath}
    fi
fi

if [ ${tcltkpath} == "unknown" ]; then
    echo ""
    echo "Tcl/Tk libraries were not found under /usr nor ${HOME}."
    echo "Please provide the path to your Tcl/Tk installation"
    echo "(the directory with libBLT24${libsuffix}) below, or install skycat first!"
    echo ""
    echo -n "Tcl/Tk path [press ENTER]: "
    read tcltkpath

    if [ ${tcltkpath}_A == "_A" ]; then
	echo ""
	echo "ERROR: No skycat installation was found. Exit."
	exit
    fi

    ./check_skycat.sh ${skycatpath}
    if [ ${skycatpath} == "unknown" ]; then
	echo "ERROR: One or more of the following libraries were not found:"
	echo "  " $skycatpath/libBLT24${libsuffix}
	echo "  " $skycatpath/libtcl8.4${libsuffix}
	echo "  " $skycatpath/libtk8.4${libsuffix}
	echo ""
	exit
    fi
fi

echo ""
echo "Skycat libraries found in:"  $skycatpath
echo "tcl/Tk libraries found in:"  $tcltkpath
echo ""

# Linking the relevant libraries:
ln -sf ${skycatpath}/libastrotcl2.1.0${libsuffix} ${GMMPS}/lib.${OS}/
ln -sf ${skycatpath}/libtclutil2.1.0${libsuffix}  ${GMMPS}/lib.${OS}/
ln -sf ${skycatpath}/libcat4.1.0${libsuffix}      ${GMMPS}/lib.${OS}/
ln -sf ${tcltkpath}/libBLT24${libsuffix}          ${GMMPS}/lib.${OS}/
ln -sf ${tcltkpath}/libtcl8.4${libsuffix}         ${GMMPS}/lib.${OS}/
ln -sf ${tcltkpath}/libtk8.4${libsuffix}          ${GMMPS}/lib.${OS}/


###############################################################################
# 2: Install CFITSIO
###############################################################################

echo " "
echo "################################################################### "
echo "GMMPS Installer: Installing cfitsio ... "
echo "################################################################### "
echo " "

sleep 2

cd cfitsio

make distclean
./configure | tee cfitsio.log
success=`grep "Congratulations, Makefile update was successful." cfitsio.log`
if [ "${success}_A" = "_A" ]; then
    echo " "
    echo "################################################################### "
    echo "GMMPS Installer: ERROR! cfitsio did not configure correctly ..."
    echo "################################################################### "
    echo " "
    exit
else
    echo " "
    echo "################################################################### "
    echo "GMMPS Installer: cfitsio configured fine... making"
    echo "################################################################### "
    echo " "
    sleep 2
fi

make
success=`test -f libcfitsio.a && echo success`
if [ "${success}_A" = "_A" ]; then
    echo " "
    echo "################################################################### "
    echo "GMMPS Installer: ERROR! libcfitsio.a was not created!"
    echo "################################################################### "
    echo " "
    exit
else
    echo " "
    echo "################################################################### "
    echo "GMMPS Installer: libcfitsio.a created ... installing"
    echo "################################################################### "
    echo " "
    sleep 2
fi

make install
rm cfitsio.log


###############################################################################
# 3: Setting up the system
###############################################################################

echo " "
echo "################################################################### "
echo "GMMPS Installer: Building GMMPS ..."
echo "################################################################### "
echo " "

sleep 2

cd ${GMMPS}

# Copy the skycat libraries
#echo "cp $skycatpath/libcat4.1.0${libsuffix}      ${GMMPS}/lib/"
#echo "cp $skycatpath/libastrotcl2.1.0${libsuffix} ${GMMPS}/lib/"
#echo "cp $skycatpath/libtclutil2.1.0${libsuffix}  ${GMMPS}/lib/"
echo "cp cfitsio/libcfitsio.a lib/"
echo "cp cfitsio/include/*.h  include/"

#cp $skycatpath/libcat4.1.0${libsuffix}      ${GMMPS}/lib/
#cp $skycatpath/libastrotcl2.1.0${libsuffix} ${GMMPS}/lib/
#cp $skycatpath/libtclutil2.1.0${libsuffix}  ${GMMPS}/lib/

cd ${GMMPS}/cfitsio
cp libcfitsio.a ../lib/
cp include/*.h ../include/

if [ $OS = "Darwin" ]; then
    export DYLD_LIBRARY_PATH=${GMMPS}/lib.$OS
else
    export LD_LIBRARY_PATH=${GMMPS}/lib.$OS
fi

cd ${GMMPS}/src
make clean
make

# Check if all binaries are there and executable
cd ${GMMPS}/bin/

./gmCat2Fits    | grep USAGE > log
./gmConvert2Cat | grep USAGE >> log
./gmMakeMasks_2 | grep USAGE >> log
./gmmps_fov     | grep USAGE >> log
./gmmps_sel     | grep USAGE >> log
./gmOdf2Cat     | grep USAGE >> log
nsuccess=`wc -l log | awk '{print $1}'`
if [ $nsuccess != 6 ]; then
    echo " "
    echo "######################################################################### "
    echo "GMMPS Installer: ERROR: Not all GMMPS executables were built correctly!"
    echo "                 Revise the output below for errors!"
    echo "######################################################################### "
    echo " "
    cat log
    \rm log
    exit
fi
\rm log

# Do the last installation step
cd ${GMMPS}

echo " "
echo "############################################################################## "
echo "GMMPS Installer: GMMPS modules verified. Building the GMMPS startup script ..."
echo "############################################################################## "
echo " "

${GMMPS}/src/build_gmmps ${GMMPS} $skycatpath $OS > ${GMMPS}/bin/gmmps

chmod a+x ${GMMPS}/bin/gmmps

echo
echo "########################################################################"
echo ""
echo "Finished. The GMMPS startup script is:"
echo "   ${GMMPS}/bin/gmmps"
echo ""
echo "Please add"
echo "   ${GMMPS}/bin/"
echo "to your PATH variable. Restart your shell and type 'gmmps' to run GMMPS."
echo ""
echo "########################################################################"
echo
