#!/bin/bash

# $1: optional path where libraries are checked

OS=`./discoverOS.sh`
if [ $OS = "Darwin" ]; then
    libsuffix=".dylib"
else
    libsuffix=".so"
fi

# If a search path was given
if [ $1_A != "_A" ]; then
    # Check if files exist
    t0=`test -f $1/libBLT24${libsuffix} && echo "y"`
    t1=`test -f $1/libtcl8.4${libsuffix} && echo "y"`
    t2=`test -f $1/libtk8.4${libsuffix} && echo "y"`
    if [ ${t0}_A == "y_A" ] && [ ${t1}_A == "y_A" ] && [ ${t2}_A == "y_A" ]; then
	# Check for which architecture the library was compiled
	c0=`./check_os_compatibility.sh $1/libBLT24${libsuffix}`
	c1=`./check_os_compatibility.sh $1/libtcl8.4${libsuffix}`
	c2=`./check_os_compatibility.sh $1/libtk8.4${libsuffix}`
	if [ ${c0} == 1 ] && [ ${c1} == 1 ] && [ ${c2} == 1 ]; then
	    echo $1
	    exit
	fi
    fi
    echo "unknown"
    exit
fi


# If no search path was given
for SEARCHPATH in /usr ${HOME}
do
    \find $SEARCHPATH -type f -name libBLT24${libsuffix} 2> /dev/null | grep libBLT24 > libtcltk_$$
    numlibs=`test -s libtcltk_$$ && echo yes`
    if [ "$numlibs"_A = "_A" ]; then
	# Nothing found, jump to the next search path
	continue
    fi

    # Matching file names were found. Check OS compatibility
    test -f libtcltk_good_$$ && rm libtcltk_good_$$

    exec < libtcltk_$$
    while read lib
    do
	compatible=`./check_os_compatibility.sh $lib`	
	# If OS and filetype match, then add the lib to a list of good libs
	if [ "${compatible}" == 1 ]; then
	    echo $lib >> libtcltk_good_$$
	fi
    done
    
    # Fetch the path to the first good entry and keep our fingers crossed
    # In the future, we might compile the libs ourselves once ESO has fixed the licensing issues
    
    # loop over the list of directories and find one that contains all four libraries
    exec < libtcltk_good_$$
    while read file
    do
	path=`dirname ${file}`
	# Check that the other three skycat libraries are present, too:
	t1=`test -f $path/libtcl8.4${libsuffix} && echo "y"`
	t2=`test -f $path/libtk8.4${libsuffix} && echo "y"`
	
	if [ ${t1}_A == "y_A" ] && [ ${t2}_A == "y_A" ]; then
	    # Check for which architecture the library was compiled
	    c1=`./check_os_compatibility.sh $path/libtcl8.4${libsuffix}`
	    c2=`./check_os_compatibility.sh $path/libtk8.4${libsuffix}`
	    
	    if [ ${c1} == 1 ] && [ ${c2} == 1 ]; then
		echo $path
		rm libtcltk_$$ libtcltk_good_$$
		exit
	    fi
	fi
    done
done

echo "unknown"

rm libtcltk_$$ libtcltk_good_$$
