# Copyright (C) 2014 Association of Universities for Research in Astronomy, Inc.
# Contact: mschirme@gemini.edu
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# $Id: gmmps_spoc.tcl,v 1.8 2013/04/23 15:41:14 gmmps Exp $
#
#***********************************************************************
#****     D A O   I N S T R U M E N T A T I O N   G R O U P        *****
#
# (c) <year>				(c) <year>
# National Research Council		Conseil national de recherches
# Ottawa, Canada, K1A 0R6 		Ottawa, Canada, K1A 0R6
# All rights reserved			Tous droits reserves
# 					
# NRC disclaims any warranties,		Le CNRC denie toute garantie
# expressed, implied, or statu-		enoncee, implicite ou legale,
# tory, of any kind with respect	de quelque nature que se soit,
# to the software, including		concernant le logiciel, y com-
# without limitation any war-		pris sans restriction toute
# ranty of merchantability or		garantie de valeur marchande
# fitness for a particular pur-		ou de pertinence pour un usage
# pose.  NRC shall not be liable	particulier.  Le CNRC ne
# in any event for any damages,		pourra en aucun cas etre tenu
# whether direct or indirect,		responsable de tout dommage,
# special or general, consequen-	direct ou indirect, particul-
# tial or incidental, arising		ier ou general, accessoire ou
# from the use of the software.		fortuit, resultant de l'utili-
# 					sation du logiciel.
#
#***********************************************************************
#
# FILENAME
# gmmps_spoc.tcl 
#
# PURPOSE:
# GUI for SPOC.
#
# Original created by:
# E.S.O. - VLT project/ ESO Archive
# D.Bottini    01 Apr 00 Created
#
# PROC NAME(S)
# constructor	Constructor
# debugStatement
# init -	Read files and draw gui
# make_layout	Draw gui
# spoc -	Fundtions called when SPOC button is pressed.
# calcSpectrum  Calculate spectrum length
#
#
# $Log: gmmps_spoc.tcl,v $
# Revision 1.8  2013/04/23 15:41:14  gmmps
# Fixed bug with FoV checking (removed fovCheck) and changed NS_DIST keyword to SHUFSIZE in unbinned pixels.
#
# Revision 1.7  2013/02/04 18:38:05  gmmps
# Small features and bug fixes from Dec2012 and Jan2013.
#
# Revision 1.6  2011/08/18 22:37:18  gmmps
# Bugfixes and small improvements, focus on mask design algorithm
#
# Revision 1.5  2011/04/29 18:38:57  gmmps
# Wiggle vals now remembered.
#
# Revision 1.4  2011/04/29 16:31:02  gmmps
# Quieted down GMMPS in terminal and cleaned up src folder.
#
# Revision 1.3  2011/04/29 02:26:50  gmmps
# Slit wiggle is active!
#
# Revision 1.18  2011/04/28 22:24:40  gmmps
# More F2 specLen tweaks.
#
# Revision 1.17  2011/04/28 21:19:50  gmmps
# F2 wavelength calc modifications.
#
# Revision 1.16  2011/04/27 18:14:12  gmmps
# F2 g/f warning window bug fixed.
#
# Revision 1.15  2011/04/19 20:58:03  gmmps
# Bugfixes.
#
# Revision 1.14  2011/04/18 19:13:53  gmmps
# Bugfixes.
#
# Revision 1.13  2011/04/15 20:09:23  gmmps
# Now displays a barcode rectangle when plotting GMOS gaps.
#
# Revision 1.12  2011/04/14 22:30:12  gmmps
# Max Mode bugfix.
# N&S Bandfile bugfix.
# Added cancel button to N&S dialog.
#
# Revision 1.11  2011/04/07 21:27:42  gmmps
# Fixed catalog window title bug.
#
# Revision 1.10  2011/04/07 17:05:50  gmmps
# Band definition files are now associated with OT and ODF files.
#
# Revision 1.9  2011/04/06 22:28:34  gmmps
# Continued N&S tweaks.
#
# Revision 1.8  2011/03/31 19:37:32  gmmps
# N&S properties are now remembered by the config file. Bugfixes.
#
# Revision 1.7  2011/03/28 17:17:48  gmmps
# Added DET_IMG and DET_SPEC to generated fits ODF catalogs.
#
# Revision 1.6  2011/03/28 16:55:49  gmmps
# Added "DET_IMG" and "DET_SPEC" to generated ODF files.
# Slit conflict detection butfix in gmMakeMasks_2.cc.
#
# Revision 1.5  2011/03/23 16:39:49  gmmps
# 'Plot GMOS -detname- Gaps' now consistant over multiple open windows.
# Grating/Filter/etc now hold thier values in the make masks window.
#
# Revision 1.4  2011/03/21 23:13:01  gmmps
# configuration file development continues.
#
# Revision 1.3  2011/03/17 17:23:50  gmmps
# Generalized detector settings and fixed an install script bug.
#
# Revision 1.2  2011/02/22 19:19:04  gmmps
# Minor changes.
#
# Revision 1.1  2011/01/24 20:02:15  gmmps
# Compiled for RedHat 5.5 32 and 64 bit.
#
# Revision 1.16  2007/08/22 01:51:28  callen
# some bug fixes from James Turner again fixed while I was at
# scipy, I could not reproduce these errors but they appear related to
# some modifications for FLAMINGOS-2
#
# Revision 1.15  2006/05/16 23:00:42  callen
# version Alpha of the F2 mods
#
# Revision 1.14  2006/05/06 09:29:39  callen
# more changes for FLAMINGOS-2
#
# Revision 1.13  2003/02/20 02:12:40  callen
# files: band_def_UI.tcl gmmps_spoc.tcl vmSkySearch.tcl
#
# the vmSkySearch change fixes the incompatibility with skycat 2.7.3
#
# The other two files had problems with the way I removed a commented line.
# I cleaned this up and tested (silly me, I didn't test after removing just
# a comment!).
#
# Revision 1.12  2003/02/11 21:09:19  callen
# fixed default slit width in gmmps_spoc when adding slit width column is not
# present.  Also removed two commented out pieces of code in
# both of these files (band_def_UI.tcl gmmps_spoc.tcl )
#
# Revision 1.11  2003/02/05 01:39:23  callen
# I had left an extra debug info_dialog in... which I like, but the users wouldn't....
#
# Revision 1.10  2003/01/18 06:06:59  callen
# more debug information for linux bug
#
# Revision 1.9  2003/01/18 03:45:39  callen
# finished version .20 for distribution, including drawing the hashmarks
# on the prohibited area, not the band, fixing some terms
# (px-->pix, Band Height--> Band Size)
#
# Modified Files:
# 	band_def_UI.tcl gmmps_spoc.tcl vmQueryResult.tcl
#
# Revision 1.8  2003/01/17 13:38:02  callen
# these files changed tonight for these additions:
# NODSIZE/SHUFSIZE or BANDSIZE/SHUFSIZE into FITS ODF header
# slitwidth given via LabelEntry (not slider)
# save and load *.bands files for values in bandGUI
# clear bands:
# debugging output for linux bug
#
# Modified Files:
#  	band_def_UI.tcl gmConvert2Cat.c gmMakeMasks.cc gmmps_spoc.tcl
#  	vmAstroCat.tcl vmTableList.tcl
#
# Revision 1.7  2002/12/02 11:06:19  callen
# handles microshuffle, but there seems to be a problem with the reference objects not having enough of a margin with the shuffle area of the reference objects
#
# Revision 1.6  2002/11/27 03:18:15  callen
#
# modifications at work, checked in to get them at home.
#
# Revision 1.5  2002/11/21 19:43:28  callen
# stable point, band definition file name is passed through from
# band def --> gmmps_spoc --> gmMakeMask
#
# NOTE: MOST OF THESE FILES WERE NOT INVOLVED... I must have tweaked the
# dates with one experiment I did where I backed up the files and restored them
# outside CVS!
#
# Revision 1.4  2002/11/20 03:45:20  callen
# these changes bring up a nod and shuffle interface which can start the mask design interface.
#
# note: the feature is not yet active... it's just a GUI!
#
# Revision 1.3  2002/08/20 00:21:03  callen
# improves the overwrite file message a bit for clarity
#
# Revision 1.2  2002/08/10 03:36:38  callen
# changed logic and message in gmmps_spoc.tcl for ODF file overwriting
#
# Revision 1.1.1.1  2002/07/19 00:02:09  callen
# importing gmmps as recieved from Jennifer Dunn
# gmmps is a skycat plugin and processes for creating masks
#
# Revision 1.22  2001/12/11 00:08:30  dunn
# Added spectrum length and anamorphic factor to output fits file
#
# Revision 1.21  2001/11/27 23:07:00  dunn
# Change of name.
#
# Revision 1.20  2001/11/02 17:12:40  dunn
# Moved warning to log file for gmCat2Fits and changed dialog box prompt.
#
# Revision 1.19  2001/11/01 23:18:42  dunn
# Changes to handle different pixel scales.
#
# Revision 1.18  2001/10/24 18:11:25  dunn
# Added the path to the new name when writing the new catalog file.
#
# Revision 1.17  2001/10/22 17:35:15  dunn
# Fixed keywords in cfg file.
#
# Revision 1.16  2001/10/19 23:52:53  dunn
# Make the pixelscale passed in.
#
# Revision 1.15  2001/10/11 22:06:21  dunn
# Added cleanup of files.
#
# Revision 1.14  2001/10/11 21:50:10  dunn
# Cleared debug, got rid of re-called calcSpectrum.
#
# Revision 1.13  2001/10/11 21:19:46  dunn
# Typo.
#
# Revision 1.12  2001/10/11 21:09:13  dunn
# Changed gmmps_spoc to be gmMakeMasks, and it now makes the catalog
# Minimal Catalog ODF, rather than just the dat.
# Changed gmmps_spoc2paf to gmCat2Fist, which creates the
# Minimal Fits ODF file.
# Check for existing files, still need to fix that.
#
# Revision 1.11  2001/08/22 22:55:24  dunn
# *** empty log message ***
#
# Revision 1.10  2001/08/13 21:24:19  dunn
# Changed format of speclen to popup.
#
# Revision 1.9  2001/08/03 21:08:35  dunn
# Changes to handle multiple windows and object selection.
#
# Revision 1.8  2001/08/02 21:02:53  dunn
# Printed out more debug statements.
#
# Revision 1.7  2001/08/02 20:07:58  dunn
# *** empty log message ***
#
# Revision 1.6  2001/07/17 19:40:23  dunn
# *** empty log message ***
#
# Revision 1.5  2001/04/25 20:40:07  dunn
# Added output from gmmps_FoV to frame.
#
# Revision 1.4  2001/04/25 18:26:31  dunn
# Changed prior 2 to be green square.
#
# Revision 1.3  2001/04/25 18:12:37  dunn
# Changed conversion to pixels from .205 to .072.
#
# Revision 1.2  2001/04/25 17:01:22  dunn
# Initial revisions.
#
#
#****     D A O   I N S T R U M E N T A T I O N   G R O U P        *****
#***********************************************************************
#

itcl::class cat::gmmps_spoc {
    inherit util::TopLevelWidget
    
    #  constructor
    
    constructor {catClass catname addWIDTH addTILT addPX addPY pixs bw \
		     resultz instType config_file det_img det_spec ra_img \
		     dec_img crpix1 crpix2 dispDirection \
		     {bdfile ""} {isBandShuffle 0} {shuffleAmt 0} args} {
	global vsDebugLevel
	set vsDebugLevel 1 
	set args {}
	eval itk_initialize $args
	set itk_option(-catalog) $catname
	set itk_option(-width) $addWIDTH
	set itk_option(-tilt) $addTILT
	set itk_option(-posx) $addPX
	set itk_option(-posy) $addPY
	set itk_option(-pixs) $pixs
	set itk_option(-crpix1) $crpix1
	set itk_option(-crpix2) $crpix2
	set itk_option(-resultz) $resultz
	set itk_option(-bw) $bw
	set itk_option(-bdfile) $bdfile
	set itk_option(-insttype) $instType
	set itk_option(-bandshuffle) $isBandShuffle
	set itk_option(-shuffleamt) $shuffleAmt
	
	set DET_IMG_ $det_img
	set DET_SPEC_ $det_spec
	set RA_IMG_ $ra_img
	set DEC_IMG_ $dec_img
	set config_file_ $config_file
	set catClass_ $catClass
	set dispDirection_ $dispDirection		 

	#debugStatement "SPOC INIT:Central Wave = $bw"
	#debugStatement "Pixel scale = $itk_option(-pixs) "
	
	#  Open the version file and read in the first line, which
	#  should be the version number
	#  spit out the version
	#

	set versionx 0.0?
	set home $::env(GMMPS)
	catch {set ver [open $home/config/version.dat r]}
	catch {set versionxx [gets $ver versionx] }
	catch {::close $ver }
    }

    # destructor

    destructor {
	catch {close}
    }
    
    
    #############################################################
    #  Name: debugStatement
    #
    #  Description:
    #   Only print the passed in message, if the debug flag is set.
    #   BIG WARNING:  need to make this its own class.
    #############################################################
    #############################################################
    
    protected proc debugStatement {line} {
	global vsDebugLevel
	
	if { $vsDebugLevel == 1 } {
	    puts $line
	}
    }

    
    #############################################################
    #  Name: init
    #
    #  Description:
    #############################################################
    #############################################################
    
    protected method init {} {
	
	#debugStatement "\ngmmps_spoc.tcl:init: entry .... "
	wm title $w_ "gmmps_spoc ($itk_option(-number))"
	wm iconname $w_ "gmmps_spoc ($itk_option(-number))"
	set home $::env(GMMPS)
	
	set instType $itk_option(-insttype)
	
	#debugStatement "spoc::init: instType: $instType"
	
	set filter_file    $home/config/${instType}_filters.lut
	set grate_file     $home/config/${instType}_gratings.lut
	set grateList_file $home/config/${instType}_gratingeq.dat
	
	set filter    [open $filter_file r]
	set grate     [open $grate_file r]
	set grateList [open $grateList_file r]
	
	set buf_filter ""
	set buf_grate ""
	set buf_grequest ""
	set buf_gtilt ""
	
	#  Read in the config files.
	#debugStatement "gmmps_spoc.tcl:init: reading $filter_file .."
	while {[gets $filter line] >= 0} {
	    if {[string match *##* $line] == 0} { lappend buf_filter $line}
	}
	::close $filter
	
	#debugStatement "gmmps_spoc.tcl:init: reading $grate_file ..."
	while {[gets $grate line] >= 0} {
	    if {[string match *##* $line] == 0} { lappend buf_grate $line}
	}
	::close $grate

	# Get the gRequest/gTilt table read in. Put in buf_grequest & buf_gtilt
	#
	#debugStatement "gmmps_spoc.tcl:init: reading $grateList_file ..."
	set cnt 0
	while {[gets $grateList line] >= 0} {
	    if {[string match *#* $line] == 0} { 
		#
		#  Break up the line into parts: gRequest gTilt
		incr cnt 1
		set buf_grequest [lappend buf_grequest [lindex $line 0] ]
		set tmpbuf [lindex $line 0]
		# debugStatement "init: buf_request $cnt = $tmpbuf, line=$line"
		set buf_gtilt [lappend buf_tilt [lindex $line 1] ]
	    }
	}
	::close $grateList
		
	#
	#  Set pull down list values and then draw spoc box.
	set nfilter [llength $buf_filter]
	set ngrate [llength $buf_grate]
	set ngtilt [llength $buf_gtilt]
	make_layout $itk_option(-catalog) $itk_option(-width)
     }

    
    #############################################################
    #  Name: add_widgets
    #
    #  Description:
    #    Draw the SPOC box and its widgets
    #############################################################
    #############################################################
    protected method make_layout {mycatname addWIDTH} {
	TopLevelWidget::make_short_help
		
	global ::spoc_autoexpansion
	set spoc_autoexpansion 0

	set instType $itk_option(-insttype)
	
	# Load values from configuration file, if they are present.
	set prev_inst      [$catClass_ gmmps_config_ [list "instrument" ]]
	set prev_cenwave   [$catClass_ gmmps_config_ [list "$instType-cenwave" ]]
	set prev_grating   [$catClass_ gmmps_config_ [list "$instType-grating" ]]
	set prev_filter    [$catClass_ gmmps_config_ [list "$instType-filter" ]]
	set prev_expansion [$catClass_ gmmps_config_ [list "$instType-expand_slits" ]]
	set prev_wiggle    [$catClass_ gmmps_config_ [list "$instType-length_offset_%" ]]
	set prev_minSpecDist [$catClass_ gmmps_config_ [list "$instType-minSpecDist" ]]

	# Determine previous expansion setting, if it exists.
	if {$prev_expansion != 0 && $prev_expansion != 1} {
	    set prev_expansion 0
	}

	# Have we wiggled previously
	if {$prev_wiggle == ""} {
	    set prev_wiggle 0
	}
	
	# Have we given a minimum slit separation previously
	if {$prev_minSpecDist == ""} {
	    set prev_minSpecDist 4
	}
	
	
	#############################################################
	#    The main frame containing the two setup frames
	#############################################################
	pack [frame $w_.masterTopFrame ] \
	    -side top -anchor w -padx 5 -pady 5 -ipadx 5 -ipady 5
	
	
	#############################################################
	#    Frames for instrumental setup and slit placement
	#############################################################
	pack [frame $w_.instSetup -relief groove -borderwidth 2 ] \
	    -side left -anchor w -fill both -padx 5 -pady 5 -ipadx 5 -ipady 5 \
	    -in $w_.masterTopFrame
	
	pack [frame $w_.smartSlit -relief groove -borderwidth 2 ] \
	    -side left -anchor w -fill both -padx 5 -pady 5 -ipadx 5 -ipady 5 \
	    -in $w_.masterTopFrame
	
	
	#############################################################
	#    Frame containing the catalog name(s)
	#############################################################
	pack [ frame $w_.catLabelsFrame -relief groove -borderwidth 2 ] \
	    -side top -anchor w -padx 5 -pady 5 -ipadx 5 -ipady 5 \
	    -fill x -expand 1


	#############################################################
	#    The main frame containing the gmMakeMasks output
	#    and the rest of the elements
	#############################################################
	pack [frame $w_.masterBottomFrame ] \
	    -side top -anchor w -padx 5 -pady 5 -ipadx 5 -ipady 5 \
	    -fill y -expand 1


	#############################################################
	#    Frame containing the make mask and close buttons
	#############################################################
	pack [frame $w_.buttons ] \
	    -side top -anchor e -padx 5 -pady 5 -ipadx 5 -ipady 5

	# Now fill the master frames

	#############################################################
	# 1. Setup the instrument configuration
	#############################################################
	
	# Set a different default central wavelength for F2.
	if { $itk_option(-insttype) == "F2" || $itk_option(-insttype) == "F2-AO"} {
	    set waveDisp "NA"
	    set entryState "disabled"
	} else {
	    if {$prev_cenwave != ""} {
		set waveDisp $prev_cenwave
	    } else {
		set waveDisp $itk_option(-bw)
	    }
	    set entryState "normal"
	}

	# Pack the instrumental setup widgets
	pack \
	    [label $w_.instSetup.label -text "Setup for:    $instType" \
		 -foreground #55f -anchor w] \
	    [LabelEntry $w_.instSetup.band \
		 -text "CWL \[nm\]:" \
		 -value $waveDisp -valuewidth 6 \
		 -command [code $this calculateCentralWavelength] \
		 -valuefont -adobe-courier-medium-r-*-*-*-120* \
		 -labelfont "Courier 14 bold" ] \
	    -side top -anchor w -in $w_.instSetup \
	    -padx 5 -ipadx 5 -ipady 5 -fill x
	
	#bind $w_.option.band.entry <FocusOut> [code $this calculateCentralWavelength ]
	
	# what is this??? not used anywhere? $entryState to be deleted from F2 setup?
	$w_.instSetup.band.entry configure -state $entryState
	
	# The grating data
	pack [LabelMenu $w_.instSetup.grateorder \
		  -text "Grating:" -relief groove \
		  -orient horizontal -labelfont "Courier 14 bold"] \
	    -side top -anchor w -in $w_.instSetup \
	    -padx 5 -ipadx 5 -ipady 5 -fill x
	
	for {set n 0} {$n < $ngrate} {incr n 1} {
	    set grating_name [lindex [lindex $buf_grate $n] 0]		
	    $w_.instSetup.grateorder add -label $grating_name \
		-command [code $this wave_config "$instType-grating" $grating_name ]
	}
	
	# The filter data
	pack [LabelMenu $w_.instSetup.filter \
		  -text "Filter :" -relief groove \
		  -orient horizontal -labelfont "Courier 14 bold"] \
	    -side top -anchor w -in $w_.instSetup \
	    -padx 5 -ipadx 5 -ipady 5 -fill x
	
	for {set n 0} {$n < $nfilter} {incr n 1} {
	    set filter_name [lindex [lindex $buf_filter $n] 0]			
	    $w_.instSetup.filter add -label $filter_name \
		-command [code $this wave_config "$instType-filter" $filter_name ]
	}
	
	# The defaults button
	pack [button $w_.instSetup.resetConfig \
		  -command [code $this restore_defaults ] \
		  -text "Restore Defaults" ] \
	    -side top -anchor w -in $w_.instSetup \
	    -padx 5 -ipadx 5 -expand 1 -fill x -ipady 5
	
	# Restore previous grating/filter settings, if available. 
	if {$prev_grating != ""} {
	    $w_.instSetup.grateorder configure -value $prev_grating
	}
	if {$prev_filter != ""} {
	    $w_.instSetup.filter configure -value $prev_filter
	}
	
	# Help
	add_short_help $w_.instSetup.filter \
	    {The filter for the observation}
	add_short_help $w_.instSetup.grateorder \
	    {The grating & order for the observation}
	add_short_help $w_.instSetup.band \
	    {The central wavelength of the observation}
	

	#############################################################
	# 2. Smart slits
	#############################################################

	# Slit expansion
	pack \
	    [label $w_.smartSlit.label -text "Smart slit placement" \
		 -foreground #55f -anchor w] \
	    [checkbutton $w_.smartSlit.bias \
		 -text "Auto expansion" \
		 -variable spoc_autoexpansion] \
	    -side top -anchor w -padx 5 -ipadx 5 -ipady 5
	
	if {$prev_expansion == 0} {
	    $w_.smartSlit.bias deselect
	} else {
	    $w_.smartSlit.bias select
	}

	pack \
	    [LabelEntry $w_.smartSlit.minSpecDist \
		 -text "Min. separation \[pix\]:" \
		 -value $prev_minSpecDist \
		 -valuewidth 5 -labelfont "Arial 12 bold" \
		 -valuefont -adobe-courier-medium-r-*-*-*-120*] \
	    -side top -anchor w -padx 5 -ipadx 5 -ipady 5 -fill x

	# Slit expansion and number of masks are spin boxes with labels
	# We need two extra frames for that
	pack \
	    [frame $w_.smartSlit.wiggleFrame ] \
	    [frame $w_.smartSlit.maskFrame ] \
	    -side top -anchor w -fill x -padx 5 -ipadx 5 -ipady 5
	
	pack \
	    [label $w_.smartSlit.wiggleLabel -text "Auto-finesse \[%\]:" \
		 -anchor w -font "Arial 12 bold"] \
	    [spinbox $w_.smartSlit.wiggleSpinBox -from 0 -to 50 -increment 5 \
		 -width 6 -font -adobe-courier-medium-r-*-*-*-120* \
		 -justify right] \
	    -side left -in $w_.smartSlit.wiggleFrame -expand 1 -fill x

	$w_.smartSlit.wiggleSpinBox set $prev_wiggle

	pack \
	    [label $w_.smartSlit.maskLabel -text "Number of Masks:" \
		 -anchor w -font "Arial 12 bold"] \
	    [spinbox $w_.smartSlit.maskSpinBox -from 1 -to 10 -increment 1 \
		 -width 6 -font -adobe-courier-medium-r-*-*-*-120* \
		 -justify right] \
	    -side left -in $w_.smartSlit.maskFrame -expand 1 -fill x
	
	#  Only add the slit width if requested
	if { $addWIDTH == "yes" } {
	    pack [LabelEntry $w_.smartSlit.width \
		      -text "Slit Width \[\"\]:" \
		      -value 1.0 -valuewidth 5 -labelfont "Arial 12 bold"] \
		-side top -anchor w -padx 5 -ipadx 5 -ipady 5
	}
	
	# Create binding to poll for value change. 
	bind $w_.smartSlit.bias <FocusOut> [code $this bias_change "$instType-expand_slits" ]
	if { $itk_option(-bdfile) != ""} {
	    #	if  $itk_option(-bdfile) != "" && $itk_option(-bandshuffle) == 0
	    $w_.smartSlit.bias configure -state disabled
	    $w_.smartSlit.minSpecDist configure -state disabled
	}

	# Help
	add_short_help $w_.smartSlit.width {The slit width (arcsec)}
	add_short_help $w_.smartSlit.maskSpinBox {The number of ODF files (masks) to be generated}
	add_short_help $w_.smartSlit.minSpecDist \
	    {The minimum separation of spectra for the auto-expansion mode}
	add_short_help $w_.smartSlit.wiggleSpinBox \
	    {Allow movement of slits during placement (percentage of slit length)}
	add_short_help $w_.smartSlit.bias \
	    {Expand slits into unused space for max sky coverage}
	

	#############################################################
	# 3. gmMakeMasks output
	#############################################################

	# Input catalog
	pack [frame $w_.catLabelsFrame.catalog1] -side top -anchor w
	pack [frame $w_.catLabelsFrame.catalog2] -side top -anchor w
	pack \
	    [label $w_.catLabelsFrame.catalog1.label1 \
		 -text "Input Catalog: " \
		 -font "Arial 12 bold"] \
	    [label $w_.catLabelsFrame.catalog1.label2 \
		 -text [file tail $mycatname] -fg #174 \
		 -font "Arial 12 bold"] \
	    -side left -anchor w

	# Add the N&S config file (if applicable)
	if { $itk_option(-bdfile) != "" } {
	    pack \
		[label $w_.catLabelsFrame.catalog2.bdfile1 \
		     -text "N&S config: " \
		     -font "Arial 12 bold"] \
		[label $w_.catLabelsFrame.catalog2.bdfile2 \
		     -text [file tail $itk_option(-bdfile)] -fg #174 \
		     -font "Arial 12 bold"] \
		-side left -anchor w
	}

	# The frame containing the text output from gmMakeMasks_2
	frame $w_.masterBottomFrame.gmMM
	text $w_.masterBottomFrame.gmMM.03 -width 52 -height 15 -setgrid true -wrap none \
	    -xscrollcommand [list $w_.masterBottomFrame.gmMM.01 set] \
	    -yscrollcommand [list $w_.masterBottomFrame.gmMM.02 set] \
	    -background white
	scrollbar $w_.masterBottomFrame.gmMM.01 \
	    -command [list $w_.masterBottomFrame.gmMM.03 xview] -orient horiz
	scrollbar $w_.masterBottomFrame.gmMM.02 \
	    -command [list $w_.masterBottomFrame.gmMM.03 yview] -orient vert
	pack $w_.masterBottomFrame.gmMM.01 -side bottom -fill x
	pack $w_.masterBottomFrame.gmMM.02 -side right -fill y
	pack $w_.masterBottomFrame.gmMM.03 -side left -fill both -expand true
	pack $w_.masterBottomFrame.gmMM -side left -fill both -expand true


	# Now place the two final buttons for mask making and closure
	pack \
	    [button $w_.buttons.close -foreground black \
		 -activeforeground black \
		 -text "Close" \
		 -command [code $this cancel $mycatname ]] \
	    [button $w_.buttons.spoc -bg #bfb -foreground black \
		 -activebackground #dfd -activeforeground black \
		 -text "Make Masks (ODF)" \
		 -command [code $this spoc $mycatname $addWIDTH $instType]] \
	    -side right -expand 1 -pady 2
	
	# Help
	add_short_help $w_.masterBottomFrame.catalog {Name of input catalog}	    
	add_short_help $w_.masterBottomFrame.bdfile {Band definition file for this mask}	    
	add_short_help $w_.masterBottomFrame.gmMM \
	    {Output of the Slit Positioning Optimization Code}	    
	add_short_help $w_.buttons.spoc  {Generate the mask (ODF slit catalog)}
	add_short_help $w_.buttons.close {Close this window}
    }


    #############################################################
    #  Name: calcGTilt
    #
    #  Description:
    #    Calculate Gtilt from the the GRequest
    #############################################################
    #############################################################

    protected method calcGTilt { gRequest } {
		
	#debugStatement "calcGtilt: gRequest=$gRequest"
	set max [lindex $buf_grequest 0]
	set cntr 0
	
	for {set n 1} {$n < $ngtilt } {incr n 1} {
	    set min [lindex $buf_grequest $n]
	    if { $gRequest < $min } {
		set max $min
	    } else {
		break
	    }
	}
	#debugStatement "calcGTilt: Finished, min=$min, max=$max"
	
	#
	#  Do a linear interpolation of the gTilt, given the cntr.
	#
	
	if { $min == $max } {
	    set gTilt [ lindex $buf_gtilt $n ]
	    #debugStatement "calcGtilt: gRequest too large, use largest gtilt:$gTilt"
	} else {
	    set gtiltMax [ lindex $buf_gtilt $n ]
	    set gtiltMin [ lindex $buf_gtilt [expr $n-1] ]
	    set slope [ expr ( $gtiltMin - $gtiltMax ) / ( $min - $max ) ] ;# The weird error happens here.
	    set xo [ expr ( $gtiltMax - ( $slope * $max ) ) ]
	    set gTilt [ expr ( $slope * $gRequest ) + $xo ]
	    # debugStatement "calcGtilt: calc. gTilt=$gTilt"
	}
    }


    #############################################################
    #  Name: calculateCentralWavelength
    #
    #  Description:
    #    Check combination of wavelength, filter & grating.
    #############################################################
    #############################################################

    protected method calculateCentralWavelength { {bw ""} } {

	set Filter [$w_.instSetup.filter get]
	set Grate [$w_.instSetup.grateorder get]
	
	# If central wavelength value was not provided, then fetch it from
	# the central wavelength widget
	if {$bw == ""} {
	    set bw [$w_.instSetup.band get ]   
	}
	
	set itk_option(-bw) $bw
	
	set instType $itk_option(-insttype)
	
	# Send new central wavelength to config file. 
	$catClass_ gmmps_config_ [list "$itk_option(-insttype)-cenwave" ] [list $bw ]		
	
	#debugStatement "checkComb: bandWidth=$itk_option(-bw), grate=$Grate, filter=$Filter "
	
	# only for GMOS
	if {$instType == "GMOS-N" || $instType == "GMOS-S"} {
	    if { $itk_option(-bw) < 350 || $itk_option(-bw) > 1050 } {
		error_dialog "Central wavelength outside useful range for GMOS."
	    } elseif { $Grate == "B1200" && $itk_option(-bw) > 595 } {
		error_dialog "Central wavelength too large, large parts of spectrum lost."
	    } elseif { $Grate == "R831" && $itk_option(-bw) > 860 } {
		error_dialog "Central wavelength too large, large parts of spectrum lost."
	    }
	}
    }


    #############################################################
    #  Name: checkGratingFilterCombo
    #
    #  Description:
    #    Check if the submitted grating/filter combination is 
    #    valid.
    #############################################################
    #############################################################

    protected method checkGratingFilterCombo {instType grating filter} {
	#debugStatement "checkGratingFilterCombo Entry"
	set home $::env(GMMPS)
	set pass "FALSE"
	
	if {$instType == "F2" || $instType == "F2-AO" } {
	    set inFile [open "$home/config/F2_gfcombo.dat" r]

	    # Input valid F2 grating/filter combos.
	    while {[gets $inFile line] >= 0} {
		if {[string match *#* $line] == 0} { 
		    set grating_test [lindex $line 0]
		    set filter_test  [lindex $line 1]
		    if {$grating == $grating_test && $filter == $filter_test} {
			set pass "TRUE"
			break						
		    }
		}
	    }
	    ::close $inFile
	} elseif {($instType == "GMOS-N" || $instType == "GMOS-S")} {
	    # NOTE: GMOS not yet supported currently for this functionality. 
	    set pass "TRUE"
	}

	#debugStatement "pass = $pass"
	#debugStatement "checkGratingFilterCombo Exit"
	return $pass
    }


    #########################################################################
    #  Name: spoc
    #
    #  Description:
    #   Delete file called mycatnameQM*
    #   Set SlitWidth, Masknum, Filter, Grating, Bandwith,
    #   Bias, SpecLen, BiasType based on items set in this window.
    #   Rename mycatname.dat to mycatname.dat_temp
    #   Extract the exact GMOS field of view, and drop objects that aren't in it.
    #   This is written to mycatname.dat.  Delete the mycatname.dat_temp
    #   Execute the SPOC algorithm., writes to mycatnameQ
    #   Get all fits header info out of the catalog.
    #   Save all that info to catalog.cfg
    #   Then remove catalog.cfg.
    #   For each number of masks, write the *.cat files.
    #
    #########################################################################
    #########################################################################
    
    protected method spoc {mycatname addWIDTH instType} {
	puts "\nSlit Placement Optimization Code started for catalog: $mycatname"
	
	global ::spoc_autoexpansion
	
	#debugStatement "\n\ngmmps_spoc::spoc:\n"
	#debugStatement "Intrument Type: $instType"
	#debugStatement "Catalog Name: $mycatname"
	
	set home $::env(GMMPS)
	set target_image_ image2
	
	set addPX $itk_option(-posx)
	set addPY $itk_option(-posy)
	set pixelScale $itk_option(-pixs)
	set CRPIX1 $itk_option(-crpix1)
	set CRPIX2 $itk_option(-crpix2)
	
	# Load instrument specific configuration data.
	# (Right now just maximum slit width and length)
	set iConfigFilename $home/config/${instType}.cfg
	
	set instConfig [getInstrumentConfig $iConfigFilename]
	set maximumSlitWidth [lindex $instConfig 0]
	set maximumSlitLength [lindex $instConfig 1]
	
	#  Get information from the popup
	
	set MaskNum [$w_.smartSlit.maskSpinBox get]
	set minSpecDist [$w_.smartSlit.minSpecDist get]
	set prev_minSpecDist [$w_.smartSlit.minSpecDist get]
	$catClass_ gmmps_config_ [list "$instType-minSpecDist" ] [list $prev_minSpecDist ]
	set wiggleVal [$w_.smartSlit.wiggleSpinBox get]
	$catClass_ gmmps_config_ [list "$instType-length_offset_%" ] [list $wiggleVal ]
	set wiggleVal [format "%.2f" [expr $wiggleVal / 100.]]
	set Filter [$w_.instSetup.filter get]
	set Grate  [$w_.instSetup.grateorder get]
	set Band   [$w_.instSetup.band get]
	
	if {$instType == "F2"} {
	    set itk_option(-bw) [getF2Wave $Grate $Filter]
	}
	
	# Test if we are in MCAO mode (only relevant for F2).
	set aofold ""
	catch {
	    set aofold [$target_image_ fits get AOFOLD]
	}
	
	# Save Central Wavelength
	calculateCentralWavelength
	
	# Check that the grating/filter combination is valid.
	set gfTest [checkGratingFilterCombo $instType $Grate $Filter]
	
	if { $gfTest == "TRUE" } {
	    puts "Grating/Filter test passed. This is a valid grating/filter combination."
	    
	    #  Calculate the spectrum length
	    set out 0
	    if {[catch {set out [calcSpectrum $pixelScale $instType $aofold]} msg]} {
		error_dialog $msg 
		return
	    }	    
	} else {
	    puts "WARNING: Invalid grating/filter combination. Spectrum calculation may be incorrect."
	    set answer [tk_messageBox -message "Unknown grating and filter combination.\n$Grate/$Filter combo is not known.\nSpectrum calculations may be incorrect." -parent $w_ -title "WARNING" -type okcancel ]
	    if {$answer == "cancel"} {
		return 0
	    }
	    
	    # Grating filter test failed, make a big spectrum
	    set SpecLen 2048
	    set dpix 1
	    set anaMorphic 1
	    set lmin 0
	    set lmax 2048
	    set itk_option(-bw) 1024
	}
	
	# Check if the width column is missing from the ODF.
	# If it is missing a 'global' width will be loaded from the GUI.
	set widthAll 0
	if { $addWIDTH == "yes" } {
	    set widthAll [getSlitWidthFromGui ]
	    if {$widthAll == 0 || $widthAll > $maximumSlitWidth} {
		if {$widthAll > $maximumSlitWidth} {
		    set msg "Invalid Slit Width: max width is $maximumSlitWidth"
		} else {
		    set msg "Invalid value for Slit Width"
		}
		
		error_dialog $msg $w_
		return
	    }
	}
	
	# Check if the auto slit expansion minimum separation is valid
	if { $spoc_autoexpansion == 1 && $minSpecDist < 0.} {
	    set msg "Minimum slit separation for the slit expansion mode must be zero or greater."
	    puts "ERROR: Minimum slit separation for the slit expansion mode must be zero or greater.\n"
	    error_dialog $msg $w_
	    return
	}
	
	#  Check to see if any of the Minimal ODF catalog files exist
	#  already, and if they do, request overwrite, or different name.
	set newname ""
	set done 0
	for {set n 1} {$n <= $MaskNum} {incr n 1} {
	    set testname [file rootname $mycatname]ODF$n.fits
	    if {[file exists $testname] } {
		while { $done != 1} {
		    set testnamedir  [file dirname $testname]
		    set testnametail [file tail $testname]
		    set choice [choice_dialog "Output File Exists:\n$testnametail\n\tin\n$testnamedir" {{Overwrite} {Enter New Filename} {Browse for New Filename} Cancel} {Overwrite} $w_]
		    if {$choice == "Enter New Filename"} {
			#set newname [tk_getSaveFile -parent $w_ -title "New Filename for Minimal ODF FITS table"]
			focus -force $w_
			set newname [input_dialog "Enter new filename for Minimal ODF FITS table \n(\"ODF#.fits\" will be appended)" $w_]
			focus -force $w_
			if { $newname != "" } {
			    set done 1
			}
		    } elseif { $choice == "Browse for New Filename" } {
			set newname [tk_getSaveFile -parent $w_ -title "New Filename for Minimal ODF FITS table"]
			
			if { $newname != "" } {
			    set done 1
			    set newname [file tail $newname]
			}
		    } elseif { $choice == "Cancel" } {
			return
		    } else {
			set done 1
		    }
		}
		#  Get rid of any extensions that the file has
		if {$newname != ""} {
		    puts "New filename selected: ODF will be saved as $newname."
		}		
		break
	    }
	}

	#
	#  If have a new name, then save it
	#  Now remove all *ODF.fits and *ODF.cat
	if { $newname != "" } {
	    # Tag on path to from.
	    set addOn ""
	    catch {set addOn [file dirname $mycatname]}
	    set newname $addOn/$newname
	    if { [file exists $newname] } {
		warning_dialog "Warning, overwriting files" $w_
	    }
	} else {
	    set newname [file rootname $mycatname]
	}
	catch {eval "file delete [glob -nocomplain [file rootname $newname]ODF*.cat]"}
	catch {eval "file delete [glob -nocomplain [file rootname $newname]ODF*.fits]"}

	# Set the Algorithm chosen.
	if {$spoc_autoexpansion == 1} {
	    set BiasType "M"
	} else {
	    set BiasType "N"
	}

	#  Update the screen
	update idletasks
	
	if {$dispDirection_ == "horizontal"} {
	    set maximumSlitsizeX $maximumSlitWidth
	    set maximumSlitsizeY $maximumSlitLength
	} else {
	    set maximumSlitsizeX $maximumSlitLength
	    set maximumSlitsizeY $maximumSlitWidth
	}
	
	#  Prepare the temp catalog file, which is a copy of the 
	#  catalog file, but with only specific columns in it.
	#  do this by calling function 'myprint' in vmTableList.tcl
	#
	# NOTE: This method also checks for invalid priorities and for
	# slits exceeding maximum width. myprintMessage will contain a
	# warning message for the user if slits were dropped for either
	# reason.
	set myprintMessage [$itk_option(-resultz) myprint $itk_option(-catalog) \
				$addWIDTH $widthAll $itk_option(-tilt) \
				$addPX $addPY $maximumSlitsizeX $maximumSlitsizeY]
	
	#  Extract the field of view, and drop objects that are outside. Saves to mycatname.dat.
	#  Remove mycatname.dat_temp
	# Print out to frame the results
	set out1 ""
	
	set fovfilename $home/config/${instType}_current_fov.dat

	if {[catch {
	    set out1 [exec gmmps_fov \
			  [file rootname $mycatname].dat_temp \
			  [file rootname $mycatname].dat \
			  $fovfilename \
			  $itk_option(-pixs) \
			  $itk_option(-crpix1) \
			  $itk_option(-crpix2) ]
	} msg ]} {
	    error_dialog "error getting field of view: $msg" $w_
	    return
	}

	file delete [file rootname $mycatname].dat_temp
	
	# Print fov results to GUI.
	$w_.masterBottomFrame.gmMM.03 delete 1.0 end
	$w_.masterBottomFrame.gmMM.03 insert end $out1
	# Also print any warnings about slits dropped by method myprint.
	$w_.masterBottomFrame.gmMM.03 insert end "$myprintMessage\n\n"
	
	#  Execute the SPOC algorithm.
	#  Passing in mycatname.dat(output of gmmps_fov), write to 
	#  mycatnameODF#.cat (these are MinimalODFs), and assoc. fits files.

	set output ""

	if {[catch {
	    puts "Running slit selection algorithm ..."
	    set output [exec gmMakeMasks_2 \
			    [file rootname $mycatname].dat \
			    [file rootname $newname]ODF \
			    $fovfilename 0 $SpecLen $anaMorphic $itk_option(-bw) \
			    $dpix $lmax $lmin $itk_option(-pixs) $MaskNum $BiasType 0 \
			    $dispDirection_ $DET_IMG_ $DET_SPEC_ $RA_IMG_ $DEC_IMG_ \
			    $itk_option(-crpix1) $itk_option(-crpix2) \
			    $minSpecDist $wiggleVal $itk_option(-bdfile)]
	    puts "Slit selection algorithm completed."
	} msg]} {
	    error_dialog "ERROR creating mask file(s) : $msg" $w_
	    puts "ERROR: Error creating mask files. ODF, if created, may be invalid."
	    return
	}
	
	set SpecLengthDisplay [format "%.1f" $SpecLen]

	$w_.masterBottomFrame.gmMM.03 insert end $output
	$w_.masterBottomFrame.gmMM.03 insert end "\n\nSpectrum Length: $SpecLengthDisplay"

	#debugStatement "spoc output: $output"
	#debugStatement "spoc: deleting $mycatname.dat"
	file delete [file rootname $mycatname].dat

	#
	#  Open a mycatname.log and save results of SPOC there
	#  
	set log ""
	catch { set log [open [file rootname $mycatname]ODF.log w] }
	
	puts $log "########################## FIELD OF VIEW LOG ##########################"
	puts $log "$out1"
	puts $log "########################## SPOC LOG ##########################"
	puts $log "--------------------------------------------------------------"
	puts $log "Input parameters of SPOC"
	puts $log "--------------------------------------------------------------"
	puts $log "Input catalog name: [file rootname $mycatname].dat"
	puts $log "Minimal Object Table Files: [file rootname $newname]ODF<n>.cat"
	#puts $log "Slit width (arcsec): $SlitWidth" 
	puts $log "Filter: $Filter"
	puts $log "Grating: $Grate"
	puts $log "Bandwidth: $itk_option(-bw)"
	puts $log "Anamorphic Factor: $anaMorphic"
	puts $log "Spectrum Length: $SpecLen"
	puts $log "Pixel Scale: $itk_option(-pixs)"
	puts $log "Number of masks: $MaskNum"
	#puts $log "gRequest: $gRequest" ;#~PROBLEM - these vars were causing errors, I don't know why.
	#puts $log "gTilt: $gTilt"
	puts $log "SPOC mode: $BiasType"
	puts $log "--------------------------------------------------------------"
	puts $log "OUTPUT OF SPOC"
	puts $log "--------------------------------------------------------------"
	puts $log "$output"
	puts $log "--------------------------------------------------------------"
	
	# Fetch RA, DEC, EPOCH, EQUINOX from image header
	set imageInfo [getImageInfo $target_image_]
	set rightAsc    [lindex $imageInfo 0]
	set declination [lindex $imageInfo 1]
	set epoch       [lindex $imageInfo 2]
	set equinox     [lindex $imageInfo 3]
	
	#  Get all fits header info out of the catalog.
	#  Save all that info to mycatname.cfg
	set buffer ""
	set myname ""
	set mydate ""
	catch { set myname [exec whoami ] }
	catch { set mydate [exec date -u "+%Y-%m-%dT%H:%M:%S" ] }
	catch { set buffer [exec grep #fits $mycatname] }
	set cfg [open [file rootname $newname].cfg w]
	puts $cfg $buffer
	puts $cfg "#fits FILTSPEC= '$Filter'"
	puts $cfg "#fits GRATING = '$Grate'"
	puts $cfg "#fits WAVELENG= $itk_option(-bw)"
	puts $cfg "#fits SPEC_LEN= $SpecLen"
	puts $cfg "#fits ANAMORPH= $anaMorphic"
	puts $cfg "#fits SPOCMODE= '$BiasType'"
	puts $cfg "#fits GMMPSVER= '$versionx'"
	puts $cfg "#fits MASK_PA = 0.0"
	puts $cfg "#fits PERS_ODF= '$myname'"
	puts $cfg "#fits DATE_ODF= '$mydate'"
	puts $cfg "#fits FILE_OT = '[file tail $mycatname].fits'"
	puts $cfg "#fits DET_IMG = '$DET_IMG_'"
	puts $cfg "#fits DET_SPEC= '$DET_SPEC_'"

	# Save data from image header to cfg as well.
	# Have to trim empty chars from string as they
	# might cause a stack smashing error on gcc 4.6 or newer
	if {$rightAsc != ""} {
	    puts $cfg "#fits [string trim $rightAsc]"
	}
	if {$declination != ""} {
	    puts $cfg "#fits [string trim $declination]"
	}
	if {$epoch != ""} {
	    puts $cfg "#fits [string trim $epoch]"
	}
	if {$equinox != ""} {
	    puts $cfg "#fits [string trim $equinox]"	
	}
	
	set bdfile $itk_option(-bdfile)
	if {$bdfile != ""} {		
	    puts $cfg [format "#fits BDFILE  = '%s'" [file tail $bdfile ]]
	    set shuffPx [string trim [exec grep "shufflePx:" $bdfile | cut -f2 -d: ]]
	    puts $cfg "#fits SHUFSIZE= $shuffPx   / Shuffle distance in unbinned pixels"
	}

	::close $cfg

	#  Execute gmCat2Fits, which will translate Minimal ODF catalog
	#  files (SPOC output) to Minimal ODF Fits file.
	#  Then remove the above mycatname.cfg file.
	#
	#
	#  Add output of gmCat2Fits to the log file.
	puts $log "OUTPUT OF CREATING MINIMAL ODF FITS FILE"
	puts $log "--------------------------------------------------------------"
	set out2 ""
	
	# debugStatement "About gmCat2Fits $newname, $newname ODF, masks=$MaskNum"
	# debugStatement "Synposis: gmCat2Fits \
	    #    [file rootname $newname] \
	    #    [file rootname $newname]ODF \
	    #    $MaskNum"
	if {[catch {
	    set out2 [exec gmCat2Fits [file rootname $newname] [file rootname $newname]ODF $MaskNum]	    
	} msg]} {
	    puts $log "$out2"
	}
	#debugStatement "Output of gmCat2Fits=$out2"

	puts $log "--------------------------------------------------------------"
	::close $log
	
	if {$newname != ""} {
	    catch {file delete $newname.cfg}
	} else {
	    catch {file delete [file rootname $mycatname].cfg}
	}
    }
	    

    #########################################################################
    # Retrieve F2 wavelength based on selected filter. 
    #########################################################################

    method getF2Wave {grate filter} {	
	set home $::env(GMMPS)
	set cenWave ""
	set inFile [open "$home/config/F2_gfcombo.dat" r]
	set cenWav ""

	# Scan for correct filter and input central wavelength
	while {[gets $inFile line] >= 0} {
	    set grate_test  [lindex $line 0]
	    set filter_test [lindex $line 1]
	    if {$grate_test == $grate && $filter_test == $filter} {
		set cenWav [lindex $line 3 ]
		break
	    }
	}
	
	::close $inFile
	
	if {$cenWav == ""} {
	    set cenWav 1300
	}
	
	return $cenWav
    }

    protected method getImageInfo {target_image_} {
	set fits [$target_image_ hdu fits 1 ]
	set ra ""
	set dec ""
	set epoch ""
	set equinox ""
	foreach heading [split $fits "\n" ] {
	    if {[lindex $heading 0] == "RA"} {
		set ra $heading
	    } elseif {[lindex $heading 0] == "DEC"} {
		set dec $heading
	    } elseif {[lindex $heading 0] == "EPOCH"} {
		set epoch $heading
	    } elseif {[lindex $heading 0] == "EQUINOX"} {
		set equinox $heading
	    }
	}
	
	return [list $ra $dec $epoch $equinox]
    }
    
    
    #########################################################################
    #  Name: wave_config
    #
    #  Description:
    #	Sends wavelength information to the gmmps configuration file.
    #
    #########################################################################
    #########################################################################
    
    protected method wave_config {key value} {
	$catClass_ gmmps_config_ [concat $key] [concat $value]
    }

    #########################################################################
    #  Name: getSlitWidthFromGui
    #
    #  Description:
    #	Loads slit width from the spoc GUI.
    #
    #  Returns:
    #	0 on invalid user input, a positive double otherwise.
    #
    #########################################################################
    #########################################################################
    
    protected method getSlitWidthFromGui {} {
	# If slitsize_x column is not present load slit with from the GUI.
	set widthAll [string trim [$w_.smartSlit.width get ]]
	
	if {[expr ! [string is double -strict $widthAll]] || $widthAll <= 0} {
	    # Slit width is not valid.
	    return 0
	}
	
	return $widthAll
    }

    
    #########################################################################
    #  Name: getInstrumentConfig
    #
    #  Description:
    #     Load instrument-specific configuration data. 
    #     Right now only maximum slit width is loaded, but things are set up
    #     so more config values can be easily added.
    #
    #########################################################################
    #########################################################################
    
    protected method getInstrumentConfig {instConfigFilename} {
	set configStream [open $instConfigFilename r]
	
	set maxSlitWidth 1000000
	set maxSlitLength 1000000
	
	while {[gets $configStream line] >= 0} {
	    if {[string index $line 0] == "#" || \
		    $line == ""} {
		continue
	    } elseif {[lindex $line 0] == "max_slit_width"} {
		set maxSlitWidth [lindex $line 1]
	    } elseif {[lindex $line 0] == "max_slit_length"} {
		set maxSlitLength [lindex $line 1]
	    }
	}
	::close $configStream
	
	return [list $maxSlitWidth $maxSlitLength]
    }
	    
    #############################################################
    # A nicer error dialog that does not suffer from HUGE fonts
    # This overrides a default skycat 'error_dialog' function
    #############################################################
    #############################################################
    public method error_dialog {message} {
	tk_messageBox -type ok -icon error -title Error -message $message
    }


    #########################################################################
    #  Name: calcSpectrum
    #
    #  Description:
    #     Calculated the spectrum length
    #
    #########################################################################
    #########################################################################

    protected method calcSpectrum { pixelScale instType aofold } {
	
	set home $::env(GMMPS)
	
	#debugStatement "\n\n\tgmmps_spoc::calcSpectrum"
	#debugStatement "\n\tInstrument Type: $instType"
	#debugStatement "\tPixel Scale: $pixelScale"
	
	#
	#  Get information selected from the pop-up
	set Filter [$w_.instSetup.filter get]
	set Grate [$w_.instSetup.grateorder get]
	
	if {$instType != "F2" && $instType != "F2-AO" } {
	    set itk_option(-bw)  [$w_.instSetup.band get]
	} else {
	    set itk_option(-bw) [getF2Wave $Grate $Filter]
	}
	
	#debugStatement "\tFilter Information Loaded: $Filter"
	#debugStatement "\tGrate Information Loaded: $Grate"
	#debugStatement "\tCentral Wavelength: $itk_option(-bw)"
	
	#  Check for useful band width for GMOS
	if { $instType == "GMOS-N" || $instType == "GMOS-S" } {
	    if { $itk_option(-bw) == "" } {
		warning_dialog "Central wavelength not supplied, defaulting to 600" $w_
		set itk_option(-bw) 600
	    } elseif { $itk_option(-bw) < 350 || $itk_option(-bw) > 1050 } {
		error_dialog "Central wavelength outside useful range for GMOS."
		return
	    } elseif { $Grate == "B1200" && $itk_option(-bw) > 595 } {
		error_dialog "Central wavelength too large, large parts of spectrum lost."
		return
	    } elseif { $Grate == "R831" && $itk_option(-bw) > 860 } {
		error_dialog "Central wavelength too large, large parts of spectrum lost."
		return
	    }
	}

	#  Determine specs of filter and grating selected.
	set a 0
	for {set n 0} {$n < $nfilter} {incr n 1} {
	    if {$Filter == "[lindex [lindex $buf_filter $n] 0]"} {
		set fl1 [lindex [lindex $buf_filter $n] 1]
		set fl2 [lindex [lindex $buf_filter $n] 2]
	    }
	}
	set b 0
	for {set n 0} {$n < $ngrate} {incr n 1} {
	    if {$Grate == "[lindex [lindex $buf_grate $n] 0]"} {
		set gnm [lindex [lindex $buf_grate $n] 1]
		set gl1 [lindex [lindex $buf_grate $n] 2]
		set gl2 [lindex [lindex $buf_grate $n] 3]
	    }
	}

	# The overall extent of the detector in dispersion direction
	set detSize [ getDetectorSize $instType ]
	set halfDetSize [ expr $detSize / 2. ]

	#  Calculate length of spectrum, based on grating, filter & 
	#  wavelength 
	set l1 0
	set l2 0

	set gTilt 0
	set gRequest [ expr $itk_option(-bw) * $gnm / pow(10,6)]

	#  Determine gRequest, gTilt, Anamorphic factor, linear disperson.
	if {($instType == "GMOS-N" || $instType == "GMOS-S")} {
	    #debugStatement "c\talcSpectrum: gRequest=$gRequest, gnm=$gnm"
	    calcGTilt  $gRequest 
	}

	set rad [expr 3.14159/180. ]

	if {($instType == "GMOS-N" || $instType == "GMOS-S")} {
	    set anaMorphic \
		[expr sin(($gTilt + 50.0) * $rad) / sin($gTilt * $rad)]
	    set dpix \
		[expr $anaMorphic * $itk_option(-pixs) * $itk_option(-bw) * 81.0 * sin($gTilt * $rad) / \
		     (3600.0 / $rad * $gRequest) ]
	}

	if {$instType == "F2" || $instType == "F2-AO"} {
	    set anaMorphic 0.99
	    # Load dpix from a file based on the grating/filter combination.
	    catch { set dpixFile [open $home/config/F2_gfcombo.dat r] }
	    set dpix 0.1
	    while {[gets $dpixFile line] >= 0} {
		if { [lindex $line 0] == $Grate && [lindex $line 1] == $Filter} {
		    set dpix [lindex $line 2]
		    break
		}
	    }
	    
	    # If in AO mode dpix is halved.
	    if {$instType == "F2-AO"} {
		set dpix [expr $dpix * 0.5 ]
	    }
	}

	# Now comes the really dumb part, I have to determine the spectrum
	# length, and to do that I have to choose 2 random positions on the
	# ccd, I say random because they should be close to the center, and
	# it doesn't make sense to go thru all objects ...
	# So I choose 1/2 det size + 16%, and 1/2 det size - 16%
	#
	# What? I don't understand what is going on here. 
	set smallBit [ expr $halfDetSize * 0.16 ]

	# Calc. lambdaDec at slit positions halfDetSize-smallbit and
	# halfDetSize+smallBit
	set xlamCentrala [ expr $halfDetSize - ($smallBit) / $anaMorphic ]
	set xlamCentralb [ expr $halfDetSize + ($smallBit) / $anaMorphic ]
	set dec1a [ expr $itk_option(-bw) - ($detSize - $xlamCentrala) * $dpix ]
	set dec1b [ expr $itk_option(-bw) - ($detSize - $xlamCentralb) * $dpix ]
	set dec2a [ expr $itk_option(-bw) + $xlamCentrala * $dpix ]
	set dec2b [ expr $itk_option(-bw) + $xlamCentralb * $dpix ]
	#debugStatement "\tcalcSpectrum: dpix=$dpix, anamor=$anaMorphic"

	#  Calculate Lamda1&2, which is the max/min of filter, grating, dec, 
	#  and other limits.

	# Calculate l1/lmax:
	set lmax $fl1
	set l1 $fl1
	if { $gl1 > $l1 } { 
	    set l1 $gl1
	    set lmax $gl1
	}
	if { $instType == "GMOS-N" || $instType == "GMOS-S" } {
	    if {$l1 < 300} { 
		set l1 300 
		set lmax 300
	    }
	    if {$dec1a > $l1} {
		set l1 $dec1a
	    }
	    if {$dec1b > $l1} {
		set l1 $dec1b
	    }
	}

	# Calculate l2/lmin:
	set lmin $fl2
	set l2 $fl2
	if { $gl2 < $l2 } { 
	    set l2 $gl2 
	    set lmin $gl2
	}
	if { $instType == "GMOS-N" || $instType == "GMOS-S" } {
	    if { $l2 > 1200} { 
		set l2 1200
		set lmin 1200
	    }
	    if { $dec2a < $l2 } {
		set l2 $dec2a
	    }
	    if { $dec2b < $l2 } {
		set l2 $dec2b
	    }
	}

	set diff [expr $l2-$l1]
	set SpecLen [expr ($l2 - $l1) / $dpix]
	set SpecLenDisplay [format "%.1f" $SpecLen]

	#set cenWavePos [expr ($itk_option(-bw) - $l1) / $dpix]		
	
	#debugStatement "Calculated SpecLen: $SpecLen"
	#debugStatement "diff = $diff, dpix = $dpix"
	#debugStatement "l1 = $l1, fl1 = $fl1, gl1 = $gl1, dec = $dec1a $dec1b"
	#debugStatement "l2 = $l2: fl2 = $fl2, gl2 = $gl2, dec = $dec2a $dec2b, dpix = $dpix"

	#debugStatement "lmax = $lmax, lmin = $lmin"
	#debugStatement "central wavelength position from l1 = $cenWavePos"

	#
	#  Determine if the spectrum length calculated is valid.
	#debugStatement "\tcalcSpectrum: specLen=l2-l1/dpix, $SpecLen, dpix=$dpix"
	if { $SpecLen < 1 && ($instType == "F2" || $instType == "F2-AO")} {
	    set SpecLen 1000
	    error_dialog "Error, Calculated spectrum length invalid. Using default value: $SpecLenDisplay"
	    
	} elseif {$SpecLen < 1} {
	    error_dialog "Error, Calculated spectrum length invalid: $SpecLenDisplay, Select new filter/grating" $w_
	    return -1
	}
	
	puts [format "  Spectrum Length   : %.1f pixel" $SpecLen]
	puts [format "  Central Wavelength: %.2f nm" $itk_option(-bw)]
	puts [format "  Linear Dispersion : %.4f nm/pixel" $dpix]
	puts [format "  Anamorphic Factor : %.4f" $anaMorphic]
	puts [format "  Pixel Scale       : %.4f \"/pixel" $pixelScale]
	
	if {$instType == "GMOS-N" || $instType == "GMOS-S"} {
	    puts [format "  gTilt             : %.3f" $gTilt]
	    puts [format "  gRequest          : %.3f" $gRequest]
	    puts [format "  gnm               : %.3f" $gnm]
	}
	return $SpecLen
    }
	    
    
    ####################################################################################################
    #  Name: getDetectorSize
    #
    #  Description:
    #   Extracts the spectral detector size
    ####################################################################################################
    ####################################################################################################

    protected method getDetectorSize {instType} {

	set home $::env(GMMPS)
	set CRPIX1 $itk_option(-crpix1)
	set CRPIX2 $itk_option(-crpix2)
	set pixelScale $itk_option(-pixs)
	set fovfilename $home/config/${instType}_current_fov.dat

	catch { set fovList [open $fovfilename r] }

	set dim_x {}
	set dim_y {}
	set n_dim 0

	# Read fov values into lists
	while {[gets $fovList line] >= 0} {
	    # remove all leading and trailing white space
	    # condense internal white space to single blanks
	    set line [string trim $line ]
	    set line [regsub -all "\[ \t]+" $line { }]
	    # extract the paired values when the line matches a keyword (and is not a comment)
	    if {[string match #* $line] == 0} {
		if {[string match DIM_CORNER* $line] == 1} {
		    set dim_x [lappend dim_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set dim_y [lappend dim_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_dim
		}
	    }
	}

	::close $fovList

	# Do a consistency check
	if {$n_dim < 4} {
	    error_dialog "ERROR: $fovList incomplete. Found $n_dim vertices instead of 4 for DIM_CORNER."
	    return
	}

	# Calculate average size from the 4 vertices
	set size 0
	if {$dispDirection_ == "horizontal"} {
	    set size [expr ([lindex $dim_x 3] + [lindex $dim_x 2] - [lindex $dim_x 1] - [lindex $dim_x 0]) / 2.]
	} else {
	    set size [expr ([lindex $dim_y 1] + [lindex $dim_y 2] - [lindex $dim_y 0] - [lindex $dim_y 3]) / 2.]
	}

	return $size
    }


    #########################################################################
    #  Name: restore_defaults
    #
    #  Description:
    #	Restores the default values for grating, filter, central wavelength
    #	widgets. 
    #########################################################################
    #########################################################################    
    
    protected method restore_defaults {} {
	
	set instType $itk_option(-insttype)
	
	if {$instType == "GMOS-N" || $instType == "GMOS-S"} {
	    # Restore GMOS default central wavelength
	    $w_.instSetup.band configure -value 600
	    wave_config "$instType-cenwave" 600
	}
	
	set filter_default  [lindex [lindex $buf_filter 0 ] 0 ]
	set grating_default [lindex [lindex $buf_grate 0 ] 0 ]
	set expand_default 0
	
	# Reset in widget.
	$w_.instSetup.filter configure -value $filter_default
	$w_.instSetup.grateorder configure -value $grating_default
	if {$expand_default == 0} {
	    $w_.smartSlit.bias deselect
	} else {
	    $w_.smartSlit.bias select
	}

	# Reset in config file.
	wave_config [list "$instType-filter" "$instType-grating" "$instType-expand_slits" ] \
	    [list $filter_default $grating_default $expand_default ]	
    }
	    
    #########################################################################
    #  Name: bias_change
    #
    #  Description:
    #     Check for a change in slit expansion mode, write to config file.
    #########################################################################
    #########################################################################
    
    protected method bias_change {key} {
	global ::spoc_autoexpansion
	wave_config [list $key ] [list $spoc_autoexpansion ]
    }
    
    
    #########################################################################
    #  Name: cancel
    #
    #  Description:
    #     close
    #
    #########################################################################
    #########################################################################
    
    public method cancel {mycatname } {
	# This method gets called by anything that has a cancel button
	# on it..... not sure how to distinguish it.
	#  Need to pass back the current bandwith to vmAstroCat
	#puts "Need to pass back $itk_option(-bw) here "

	#	utilReUseWidget cat::vmAstroCat .skycat1.spoc $itk_option(-catalog) $itk_option(-xsize) $itk_option(-tilt) $itk_option(-bw) $this.results
	close
    }


    #########################################################################
    #  Name: close
    #
    #  Description:
    #     Destroy window
    #
    #########################################################################
    #########################################################################
    
    public method close {} {
	destroy $w_ 
    }


    # Contains the type of file loaded in, Object Tbl, Master OT , or fitsFile 
    itk_option define -catalog catalog Catalog "" 

    # Flag for bandshuffle mode
    itk_option define -bandshuffle bandshuffle BandShuffle 0
    # Shuffle magnitude
    itk_option define -shuffleamt shuffleamt ShuffleAmt 0
    
    # Options indicating col's that exist
    itk_option define -width width Width "no"
    itk_option define -xsize xsize XSize "no"
    itk_option define -tilt tilt Tilt "no"
    itk_option define -resultz resultz Resultz ""
    itk_option define -posx posx Posx ""
    itk_option define -posy posy Posy ""
    itk_option define -pixs pixs Pixs ""
    itk_option define -crpix1 crpix1 CRPIX1 0
    itk_option define -crpix2 crpix2 CRPIX2 0
    itk_option define -bw bw Bw "600"

    protected variable catClass_
    protected variable config_file_ 
    protected variable DET_IMG_ ""
    protected variable DET_SPEC_ ""
    protected variable RA_IMG_ ""
    protected variable DEC_IMG_ ""
    protected variable dispDirection_ ""

    # QueryResult widget used to display search results
    #   protected variable results_ {}
    
    
    protected common nfilter
    protected common ngrate
    protected common ngtilt
    protected common buf_filter
    protected common buf_grate
    protected common buf_grequest
    protected common buf_gtilt
    protected common gTilt
    protected common anaMorphic
    protected common lmin
    protected common lmax
    protected common SpecLen
    protected common dpix
    protected common gRequest
    protected common versionx
}
