# Copyright (C) 2014 Association of Universities for Research in Astronomy, Inc.
# Contact: mschirme@gemini.edu
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# # This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# $Id: vmAstroCat.tcl,v 1.10 2013/08/14 15:46:39 gmmps Exp $
#
#***********************************************************************
#****     D A O   I N S T R U M E N T A T I O N   G R O U P        *****
#
# (c) <year>				(c) <year>
# National Research Council		Conseil national de recherches
# Ottawa, Canada, K1A 0R6 		Ottawa, Canada, K1A 0R6
# All rights reserved			Tous droits reserves
# 					
# NRC disclaims any warranties,		Le CNRC denie toute garantie
# expressed, implied, or statu-		enoncee, implicite ou legale,
# tory, of any kind with respect	de quelque nature que se soit,
# to the software, including		concernant le logiciel, y com-
# without limitation any war-		pris sans restriction toute
# ranty of merchantability or		garantie de valeur marchande
# fitness for a particular pur-		ou de pertinence pour un usage
# pose.  NRC shall not be liable	particulier.  Le CNRC ne
# in any event for any damages,		pourra en aucun cas etre tenu
# whether direct or indirect,		responsable de tout dommage,
# special or general, consequen-	direct ou indirect, particul-
# tial or incidental, arising		ier ou general, accessoire ou
# from the use of the software.		fortuit, resultant de l'utili-
# 					sation du logiciel.
#
#***********************************************************************
#
# FILENAME
# vmAstroCat.tcl
#
# PURPOSE:
# User interface class for viewing catalog info
#
# WARNING:
# This needs cleanup.  There are a LOT of functions that are never called.
# Also, the catalog file is both kept and used in memory AND for some weird
# reason also written to - WITHOUT being asked.  A lot of re-processing of the
# catalog when the darn thing is already in memory.  Its almost like 2 different
# people worked on it with completely different ideas.  Anyway, since we
# only have time for modifications - I have followed this haphazard way
# of dealing with the catalog file - so sorry.
#
# CLASS NAME(S)
# itk::usual vmAstroCat - Main menu class
#
# METHOD/PROC NAME(S)
#  constructor		   - constructor of the class
#  destructor		   - destructor of the class
#  init			   - called after options have been eval'ed.
#  set_logfile		   - setup logfile in .skycat dir.
#  readInFits		   - read in a fits file and create a catalog.
#  set_feedback		   - setup feeback for HTTP transfers.
#  set_instance		   - keep array of instances.
#  add_menubar		   - add menubar in object table window
#  update_catalog_menus    - update all cat. menus to show current catalog.
#  set_plot_symbols        - Dialog to set plot symbols for a catalog.
#  close		   - close this window
#  sort_dialog		   - sort list in dialog
#  set_sort_cols	   - called when user has set col's to sort by
#  select_columns	   - pop up a dialog to select table columns
#  set_show_cols	   - called when user has selected col's to show
#  reset_table		   - reset table dialogs if needed.
#  clear		   - clear the table listing
#  print		   - popup to print table listing
#  local_catalog	   - Called when Load from file is requested
#  save_as		   - save the current data to a local catalog
#  add_to		   - add the rows in the list to a catalog file
#  add_select		   - add the currently selected rows to a catalog
#  remove_selected	   - remove selected froms fro a catalog
#  enter_new_object	   - dialog to enter data for a new object to cat.
#  edit_selected_object    - dialog to allow user to edit object
#  slits		   - draws the slits for ODF files.
#  clearslits		   - clears the slits drawn from ODF files.
#  _sel			   - updates the priority col. in cat for an object
#  open_catalog		   - opens the catalog for a window
#  add_result_table	   - fills in a results_ table from a catalog
#  mps_add_dialog_buttons  - fills in the buttons in an object table window.
#  slit_add_dialog_buttons - fills in the buttons in an ODF.
#  symbol		   - Write image & symbol information from image to cat.
#  col_PRIO		   - Add the PRIOIRTY column.
#  col_ST		   - Add the SLITTYPE column.
#  col_STXSIZE		   - Add the slitsize_x column.
#  col_STYSIZE		   - Add the slitsize_y column.
#  export		   - Runs the slit algorithm stuff.
#  layout_dialog	   - Does the dialog window for a new catalog.
#  make_short_help	   - Create short help info.
#  select_catalog	   - Called when a catalog is selected from a menu
#  set_state		   - Set/reset widget states while busy
#  query_done		   - Called when background query is done.
#  search		   - Called to do search in catalog.
#  check_local_catalog     - Check catalog file.
#  set_pos_radius	   - Set position and radius
#  set_pos_width_height	   - Set position, width and height entries
#  get_col		   - Return given column val in a row.
#  new_catalog		   - Open a window for the named catalog.
#  instances		   - Return list of instance for this class.
#  get_instance		   - Return widget instance for a catalog.
#  iscat		   - Return 1 if window is for searchable catalog.

#### Then various classes to deal with images and plotting
#  plot, display_image_file, get_equinox, get_table, ...
#
# Original taken from:
# E.S.O. - VLT project/ESO Archive
# @(#) $Id: vmAstroCat.tcl,v 1.10 2013/08/14 15:46:39 gmmps Exp $
# Created originally by:
# D.Bottini, 01 Apr 00
#
# $Log: vmAstroCat.tcl,v $
# Revision 1.10  2013/08/14 15:46:39  gmmps
# Set instType and targetInst to upper case after reading INSTRUME from header
#
# Revision 1.9  2013/06/10 15:56:00  gmmps
# added workaround to make reading of PIXSCALE from image header more reliable
#
# Revision 1.8  2013/03/18 21:17:56  gmmps
# Added procedure "get_ccdsum_image" in vmAstroCat.tcl to get CCDSUM if nothing returned by the original method
#
# Revision 1.7  2013/02/12 23:59:22  gmmps
# Fix type in comment related to slittype column test
#
# Revision 1.6  2013/02/04 18:38:06  gmmps
# Small features and bug fixes from Dec2012 and Jan2013.
#
# Revision 1.5  2011/08/18 22:37:18  gmmps
# Bugfixes and small improvements, focus on mask design algorithm
#
# Revision 1.4  2011/04/29 16:31:03  gmmps
# Quieted down GMMPS in terminal and cleaned up src folder.
#
# Revision 1.3  2011/04/29 02:26:50  gmmps
# Slit wiggle is active!
#
# Revision 1.2  2011/04/25 18:27:33  gmmps
# Forked from 0.401.12 .
#
# Revision 1.29  2011/04/25 17:15:58  gmmps
# More small bugfixes/improvements (see 0.401.12 in changelog).
#
# Revision 1.28  2011/04/25 16:35:19  gmmps
# Bugfixes.
#
# Revision 1.27  2011/04/19 20:58:03  gmmps
# Bugfixes.
#
# Revision 1.26  2011/04/19 17:57:43  gmmps
# Changed the way the barcode is displayed.
#
# Revision 1.25  2011/04/18 21:36:36  gmmps
# Fixed N&S bug.
#
# Revision 1.24  2011/04/15 20:09:23  gmmps
# Now displays a barcode rectangle when plotting GMOS gaps.
#
# Revision 1.23  2011/04/14 22:30:12  gmmps
# Max Mode bugfix.
# N&S Bandfile bugfix.
# Added cancel button to N&S dialog.
#
# Revision 1.22  2011/04/08 21:38:51  gmmps
# Flamingos-2 ODF load bugfix.
#
# Revision 1.21  2011/04/08 20:13:24  gmmps
# N&S bugfixes.
#
# Revision 1.20  2011/04/07 21:27:42  gmmps
# Fixed catalog window title bug.
#
# Revision 1.19  2011/04/07 17:05:50  gmmps
# Band definition files are now associated with OT and ODF files.
#
# Revision 1.18  2011/04/06 22:28:34  gmmps
# Continued N&S tweaks.
#
# Revision 1.17  2011/03/31 19:37:32  gmmps
# N&S properties are now remembered by the config file. Bugfixes.
#
# Revision 1.16  2011/03/29 18:04:06  gmmps
# "plot bands" behavour tweak.
#
# Revision 1.15  2011/03/29 16:31:04  gmmps
# Bugfix on plot bands method.
# Moved gmmps.cfg to hidden file in user's home directory.
#
# Revision 1.14  2011/03/28 16:55:49  gmmps
# Added "DET_IMG" and "DET_SPEC" to generated ODF files.
# Slit conflict detection butfix in gmMakeMasks_2.cc.
#
# Revision 1.13  2011/03/25 23:30:21  gmmps
# *** empty log message ***
#
# Revision 1.12  2011/03/25 00:59:25  gmmps
# Changed colorscheme for OT and ODF catalog windows.
#
# Revision 1.11  2011/03/23 16:39:49  gmmps
# 'Plot GMOS -detname- Gaps' now consistant over multiple open windows.
# Grating/Filter/etc now hold thier values in the make masks window.
#
# Revision 1.10  2011/03/21 23:13:01  gmmps
# configuration file development continues.
#
# Revision 1.9  2011/03/21 14:37:44  gmmps
# More adjustments for new GMOS-N detector. Also added gmmps.cfg file
# and some associated code in vmAstroCat.tcl.
#
# Revision 1.8  2011/03/17 17:23:50  gmmps
# Generalized detector settings and fixed an install script bug.
#
# Revision 1.7  2011/03/04 18:05:46  gmmps
# Tweaked slittilt check.
#
# Revision 1.6  2011/03/04 17:34:56  gmmps
# New install procedure implemented, slittilt check implemented.
#
# Revision 1.5  2011/02/22 19:19:04  gmmps
# Minor changes.
#
# Revision 1.4  2011/01/27 16:53:23  gmmps
# F2 Waveccd adjustments.
#
# Revision 1.3  2011/01/26 22:16:00  gmmps
# Added more robustness with regard to waveccd. Shouldn't crash gmmps now.
#
# Revision 1.2  2011/01/24 20:17:57  gmmps
# Removed white line indicating "Center of Spectrum".
#
# Revision 1.1  2011/01/24 20:02:15  gmmps
# Compiled for RedHat 5.5 32 and 64 bit.
#
# Revision 1.38  2007/08/22 01:51:28  callen
# some bug fixes from James Turner again fixed while I was at
# scipy, I could not reproduce these errors but they appear related to
# some modifications for FLAMINGOS-2
#
# Revision 1.37  2007/06/26 06:51:51  callen
# patch given by Bryan Miller... should be and tests as innocuous... so I am
# patching it... though in reality I did not see the same bug in
# FC6 with the same data when plotting slits.
#
# Revision 1.36  2007/05/12 00:15:49  callen
# updates for a more robust set of build tools and specifically for
# FC6... also brought forward some FC6 fixes which were supposed to have
# been checked in.
#
# Revision 1.35  2007/05/09 02:29:35  callen
# FC6 mods
#
# Revision 1.34  2006/05/06 09:29:39  callen
# more changes for FLAMINGOS-2
#
# Revision 1.33  2006/05/04 19:38:11  callen
# changes
#
# Revision 1.32  2004/11/11 02:02:01  callen
# gelys supplied fix that shows slit tilts
#
# Revision 1.31  2003/02/27 02:22:24  callen
# fixed gmmps so that you can plot the object symbols for objects in
# the ODF when it is loaded from a FITS file.
#
# Revision 1.30  2003/02/20 03:01:26  callen
# files: band_def_UI.tcl vmAstroCat.tcl
#
# improved a faulty default and button names
#
# Revision 1.29  2003/02/06 06:31:00  callen
# added feature to grab SPEC_LEN from the #fits "preserved" headers in the .cat
# file when plotting slits... if the "Spectrum:" cat variable isn't present.
#
# Revision 1.28  2003/02/06 01:49:45  callen
# finished with the "plot bands"
#
# Revision 1.27  2003/02/05 23:20:15  callen
# added new "Convert and Load ODF from FITS" menuitem
#
# Revision 1.26  2003/02/05 01:21:03  callen
# changes to the GUI
#
# Revision 1.25  2003/01/18 08:04:05  callen
# added a line to pause, improving the linux bug, but not really solving it yet, although I've tracked it to the paste which attempts to join the new column
#
#
# S: ----------------------------------------------------------------------
#
# Revision 1.24  2003/01/18 07:24:28  callen
# removed extra info_dialog
#
# Revision 1.22  2003/01/18 06:34:27  callen
# more debug output for comparison between linux and solaris
#
# Revision 1.21  2003/01/18 06:28:41  callen
# debug output comparison linux vs solaris
#
# Revision 1.20  2003/01/18 06:09:55  callen
# debug output
#
# Revision 1.19  2003/01/18 06:06:59  callen
# more debug information for linux bug
#
# Revision 1.18  2003/01/17 13:38:02  callen
# these files changed tonight for these additions:
# NODSIZE/SHUFSIZE or BANDSIZE/SHUFSIZE into FITS ODF header
# slitwidth given via LabelEntry (not slider)
# save and load *.bands files for values in bandGUI
# clear bands:
# debugging output for linux bug
#
# Modified Files:
#  	band_def_UI.tcl gmConvert2Cat.c gmMakeMasks.cc gmmps_spoc.tcl
#  	vmAstroCat.tcl vmTableList.tcl
#
# Revision 1.17  2003/01/17 05:02:17  callen
# more of the validation and interface features from inger's testing
#
# Revision 1.16  2003/01/16 10:27:42  callen
# debugging puts for linux bug (!!! meant to work on regular list! but the next users are linux based and I run into the bug when I use my laptop/linux)  I did track it down quite a bit
#
# Revision 1.15  2003/01/06 11:31:55  callen
#
# i done from home... more bandUI
#
# Revision 1.14  2002/12/21 03:34:53  callen
# got past the roadblocks using vmTableList but, made some changes to text messages.
#
# Not yet done with band shuffling interface.  Checking things in for safety
# before vacation.
#
# Revision 1.13  2002/12/05 23:32:02  callen
# added slitlen to N&S screen, file, and makemasks: allows different slit
# and shuffle distances, and also ensures that the slitsize is accurate (since shufflePx is rounded to the next higher int, and converting that back to arcsec
# then gives a different number... now we don't convert, we use slitlen
# directly when writing the ODF.
#
# I also changed the "Band" to "N & S" to avoid confusion with frequency bands.
#
# Revision 1.12  2002/12/05 03:07:52  callen
# added another shuffle amount so it's represented in
# arcsec and pixels and converts.  Slow going as I learned a bit
# more tcl (and about the ESO LabelEntry widget)
#
# Revision 1.11  2002/11/21 19:43:28  callen
# stable point, band definition file name is passed through from
# band def --> gmmps_spoc --> gmMakeMask
#
# NOTE: MOST OF THESE FILES WERE NOT INVOLVED... I must have tweaked the
# dates with one experiment I did where I backed up the files and restored them
# outside CVS!
#
# Revision 1.10  2002/11/20 03:45:20  callen
# these changes bring up a nod and shuffle interface which can start the mask design interface.
#
# note: the feature is not yet active... it's just a GUI!
#
# Revision 1.9  2002/09/13 02:46:42  callen
# this allowed editing of the following columns
# slitpos_x
# slitpos_y
# slitsize_x
# slitsize_y
# tilt
#
# from within the vmAstroCat interface... next... saving the changes to FITS.
#
# Revision 1.8  2002/09/12 08:38:55  callen
# finished up most of the GUI side of the row editing for the input FITS table
# of objects.  need to add help and catch the flags in the _sel function.
# checking in as checkpoint, done for the night from home.
#
# Revision 1.7  2002/09/12 03:40:24  callen
# made changes to add more editing controls... these will be things like
# slitsize_x, _y, position, etc. from the task list.  This fix includes just
# slitsize_y as my test implementation.  It's working, I'm checking it in so I
# can do the other fields from home tonight (and to have a micromilestone for
# this feature)
#
# Revision 1.6  2002/08/22 01:41:01  callen
# had to remove some info_dialogs that were in for testing... for some reason
# they were not firing off in solaris but did when I tested in linux... definately
# not supposed to be there though.
#
# Revision 1.5  2002/08/20 00:49:17  callen
# added "-i" in grep spawning when looking for column names to work toward case
# insensitivity.  NOTE: other case sensitive checks seem spread out throughout the code
# and components.
#
# Revision 1.4  2002/08/13 23:14:12  callen
# thought I should add a "Clear GMOS Gaps" button, I also made the plot erase
# the previous marks (the gap graphics are taged as "gaps")
#
# Revision 1.3  2002/08/13 04:31:11  callen
# after further testing, found I was not converting properly on the GMOS
# gaps visualization... so I fixed this.  There is still a problem when loading another image
# in which case the marks are not erased (unless the user manually does it)
#
# Revision 1.2  2002/08/13 04:06:12  callen
# checking in changes to:
# vmAstroCat.tcl (to draw GMOS detector gaps)
# gmmps.sh (to allow passing an image argument to the gmmps script to load and image immediately)
#
# Revision 1.1.1.1  2002/07/19 00:02:09  callen
# importing gmmps as recieved from Jennifer Dunn
# gmmps is a skycat plugin and processes for creating masks
#
# Revision 1.23  2001/12/11 00:09:05  dunn
# Fixed checking for PIXSCALE keyword, yet again.
#
# Revision 1.22  2001/11/27 23:07:13  dunn
# Change of name
#
# Revision 1.21  2001/11/01 23:17:59  dunn
# Changes to handle different pixel scale.
#
# Revision 1.20  2001/10/22 17:38:41  dunn
# *** empty log message ***
#
# Revision 1.19  2001/10/22 17:35:46  dunn
# Fixed symbol's grep for fits keywords.
#
# Revision 1.18  2001/10/19 23:53:23  dunn
# Make pixel scale be passed in and around .
#
# Revision 1.17  2001/10/11 21:49:56  dunn
# Cleared debugLevel
#
# Revision 1.16  2001/10/11 21:04:43  dunn
# Added mag to require col list, Changed to get pix scale from file.
#
# Revision 1.15  2001/08/22 22:55:24  dunn
# *** empty log message ***
#
# Revision 1.14  2001/08/20 23:17:51  dunn
# Better messages.
#
# Revision 1.13  2001/08/20 18:52:41  dunn
# Changed it so that a user-friendly msg comes up when they try to
# open the Mask Design popup twice.
#
# Revision 1.12  2001/08/13 21:44:32  dunn
# *** empty log message ***
#
# Revision 1.11  2001/08/13 21:24:51  dunn
# Got rid of globals, and added them to the window instead.
#
# Revision 1.10  2001/08/03 21:07:32  dunn
# Changes to handle multiple windows and object selection.
#
# Revision 1.9  2001/08/02 20:07:43  dunn
# Fixes
#
# Revision 1.8  2001/07/17 19:40:11  dunn
# *** empty log message ***
#
# Revision 1.7  2001/06/26 21:45:17  dunn
# *** empty log message ***
#
# Revision 1.6  2001/05/11 18:06:59  dunn
# Added ability to change priority of user-define object.
#
# Revision 1.5  2001/05/04 22:48:58  dunn
# *** empty log message ***
#
# Revision 1.4  2001/04/25 22:25:24  dunn
# Got rid of some puts statements.
#
# Revision 1.3  2001/04/25 18:26:07  dunn
# Changed prior 2 to be green square.
#
# Revision 1.2  2001/04/25 17:01:22  dunn
# Initial revisions.
#
#
#****     D A O   I N S T R U M E N T A T I O N   G R O U P        *****
#***********************************************************************
#

itk::usual vmAstroCat {}

######################################################################
# The vmAstroCat widget class defines a top level widget for searching and
# displaying astronomical catalog data. It contains a menubar with items
# for loading, editing, and saving catalog data and a table displaying
# rows and columns of catalog data. This class does not know anything
# about images or plotting objects in images, however these features may
# be added in a derived class (see the SkySearch(n) in the skycat
# package for an example).
#
# You can also run this class as a standalone application with the
# command "astrocat". The application options are the same as the class
# options.
######################################################################

itcl::class cat::vmAstroCat {
    inherit util::TopLevelWidget

    #############################################################
    #  Name: constructor
    #
    #  Description:
    #   Constructor, what can I say.
    #############################################################
    #############################################################

    constructor {args} {
    	set home $::env(GMMPS)

	# This is where the debug level is set. 
	# (Unless there are more places where someone has decided to turn it off...)
	global acDebugLevel
	set acDebugLevel 0

	eval itk_initialize $args
	
	wm protocol $w_ WM_DELETE_WINDOW [code $this close ]
    }

    
    #############################################################
    #  Name: destructor
    #
    #  Description:
    #   destructor - delete C++ based objects so that the temp 
    #   files are deleted
    #############################################################
    #############################################################

    destructor {
	#debugStatement "\n#vmAstroCat::destructor Entry.#"  	
    	
        catch {
            if {"[$w_.cat servtype]" == "local"} {
		if {"[string range $itk_option(-catalog) 0 4]" == "/tmp/"} {
		    file delete $itk_option(-catalog)
		}
            }
        }
        catch {$w_.cat delete}
#        catch {close $rfd_}
#        catch {close $wfd_}
        catch {unset instances_($instance_idx_)}
        if {"$current_instance_" == "$w_"} {
            set current_instance_ {}
        }
        
        # Clear all marks if this is the last catalog closed.
        if {[array size instances ] == 0} {
	    set target_canvas_ .skycat1.image.imagef.canvas
	    
	    $target_canvas_ delete cbo_ma
	    $target_canvas_ delete cbo_da
	    $target_canvas_ delete cbo_bc
	    $target_canvas_ delete cbo_po
	    $target_canvas_ delete cbo_dg
	    $target_canvas_ delete cbo_alt_ma
	    $target_canvas_ delete cbo_alt_da
	    $target_canvas_ delete cbo_alt_dg
            $target_canvas_ delete slitMark
            $target_canvas_ delete shading
	    $target_canvas_ delete
        }
    }

    
    #############################################################
    #  Name: init
    #
    #  Description:
    #   This function is called ??? after options have been evaluated
    #   It will : set the logfile
    #             do the window layout: which opens the catalog.
    #             enter widget name in instance array
    #             add a short help window
    #             create an object for running interruptable batch queries
    #             position from previous window, if pos.
    #		  for local catalogs, start search automatically
    #             Then ... something I don't understand.
    #############################################################
    #############################################################

    protected method init {} {
	#debugStatement "\n#vmAstroCat::init Entry.#"

	#debugStatement "init: catType = $itk_option(-catType)"
	#debugStatement "init: catalog = $itk_option(-catalog)"

	#CBA: not working; I'm following what seems a bad use of globals to initialize an object
        # because it's already followed for catType and I think it may make a 
        # bigger mess to screw with that in this code.  So, instrument type...
        
	set instType $itk_option(-instType)
	set dispDirection [getDispDirection]

	# Set a default central wavelength for each instrument.
	if {$instType == "GMOS-N" || $instType == "GMOS-S"} {
	    set itk_option(-bw) 600
	} elseif {$instType == "F2" || $instType == "F2-AO"} {
	    set itk_option(-bw) 1300
	}

	# if running stand-alone, setup X defaults
	if {$itk_option(-standalone)} {
	    util::setXdefaults
	    cat::setXdefaults  
	}

	# set the file used to keep track of URLs (for debugging)
	set_logfile

	# do window layout.  This opens the catalog, and saves to table.
	layout_dialog

	# enter widget name in instance array for later reference
	set_instance

	# add a short help window
	make_short_help

	# position from previous window, if pos.
	if {"$current_instance_" != ""} {
	    wm geometry $w_ +[winfo x $current_instance_]+[winfo y $current_instance_]
	}
	set current_instance_ $w_

	# for local catalogs, start search automatically
	# Search will add the columns missing: x_ccd, y_ccd, MPS, ST
	set name $itk_option(-catalog)
	if {"[$w_.cat servtype]" == "local"} {
	    #  Set the title on the window to be the name listed
	    #  after the longname in the file.  And icon is the shortname
	    #debugStatement "vmAstroCat: $name ([$w_.cat longname $name])"
	    wm title $w_ "[file tail $name] ($itk_option(-number))"
	    wm iconname $w_ "[file tail [$w_.cat shortname $name]]"
	    search
	}

	set initialized_ 1

	# Configure the dialog window depending on whether this is an OT or an ODF
	set catType $itk_option(-catType)
	if { $catType == 1 || $catType == 3 } {
	    # Change window color for OT.
	    $w_ configure -background $ot_bg_
	    mps_add_dialog_buttons $instType
	} elseif { $catType == 2 || $catType == 4} {
	    # Change window color for ODF.
	    $w_ configure -background $odf_bg_	    
	    slit_add_dialog_buttons $instType
	} else {
	    error_dialog "Error: Invalid pulldown type!" 
	}
	
	# Change shorthelp colors.
	$w_.shelp configure -background $ot_bg2_ -foreground $ot_text_	
    } 


    #############################################################
    #  Name: debugStatement
    #
    #  Description:
    #   Pop up a window so that the user can edit the selected object(s)
    #############################################################
    #############################################################

    protected proc debugStatement {line} {
	global acDebugLevel
	
	if { $acDebugLevel == 1 } {
	    puts $line
	}
    }


    #############################################################
    #  Name: set_logfile
    #
    #  Description:
    #   Create the ~/.skycat dir if it does not already exists and
    #   keep a log file there.
    #############################################################
    #############################################################

    protected method set_logfile {} {
	global ::env

	# open log file used to keep track of URLs (for debugging)
	set dir $env(HOME)/.skycat

	if {! [file isdirectory $dir]} {
	    catch {mkdir $dir}
	}
	set logfile_name_ $dir/log
    }


    #############################################################
    #  Name: readInFitsFile
    #
    #  Description:
    #   Convert a fits file to a catalog file format.
    #############################################################
    #############################################################

    protected proc readInFitsFile {fitsFile args } {
	set result " " 
	set buff " "
	
	# args is tType... 4 is a ODF.fits file
	set convertProgram gmConvert2Cat

	#debugStatement "\nreadInFitsFile: Entry. fitsFile=$fitsFile"
	#puts "readInFitsFile: about to exec: $convertProgram $fitsFile"
	#debugStatement "readInFitsFile: using $convertProgram to convert type $args"

	if {[catch {set buff [exec $convertProgram $fitsFile]} msg]} {
	    error_dialog "$msg." 
	    return -1
	}

	return 0
    }

        
    #############################################################
    #  Name: set_instance
    #
    #  Description:
    #   Keep an array of instances(name,id) to help locate the 
    #   window for a catalog
    #############################################################
    #############################################################
    
    protected method set_instance {} {
	#debugStatement "\n#vmAstroCat::set_instance Entry.#"

	set name [$w_.cat longname $itk_option(-catalog)]
	set id $itk_option(-id)
	set instance_idx_ "$name,$id"
	set instances_($instance_idx_) $w_ 

	#debugStatement "#vmAstroCat::set_instance Exit.\n#"
    }


    #############################################################
    #  Name: add_menubar
    #
    #  Description:
    #    Add the menu bar in the object table window.
    #############################################################
    #############################################################

    protected method add_menubar {} {
	#debugStatement "\n#vmAstroCat::add_menubar Entry.#"
	TopLevelWidget::add_menubar
	
	set home $::env(GMMPS)
	set m [add_menubutton File "Display File menu"]
	set file_menu_ $m

	if {$itk_option(-catType) == 1 || $itk_option(-catType) == 3} {
	    $m configure -foreground $menu_text_ot_ -background $menu_bg_ot_	
	} else {
	    $m configure -foreground $menu_text_odf_ -background $menu_bg_odf_
	}
	
	if {$iscat_} {
	    # The following does not offer an OT/ODF selection, possibly unsafe? Commented out.
#	    add_menuitem $m command "Open" \
#		{Open local catalog in tab table format} \
#		-command [code cat::vmAstroCat::local_catalog \
#			      $itk_option(-id) [info class] \
#			      $itk_option(-debug) 1 $w_] \
#		-accelerator "Control-o"
#	    
#	    $m add separator

	    add_menuitem $m command "Save as..." \
		{Save listed objects to a local catalog file} \
		-command [code $this save_as] \
		-accelerator "Control-s"

	    add_menuitem $m command "Add to..." \
		{Add listed objects to a local catalog file} \
		-command [code $this add_to] \
		-accelerator "Control-a"

	    add_menuitem $m command "Add selected..." \
		{Add selected rows to a local catalog file} \
		-command [code $this add_selected] \
		-accelerator "Control-A"

	    $m add separator

	    add_menuitem $m command "Print..." \
		{Print the listing to a printer or file} \
		-command [code $this print] \
		-accelerator "Control-p"

	    add_menuitem $m command "Clear" \
		{Clear the catalog listing} \
		-command [code $this clear]

	    $m add separator
	}
	
	add_menuitem $m command "Close" \
	    {Close this window} \
	    -command [code $this close]
	
	if {$itk_option(-standalone)} {
	    add_menuitem $m command "Exit" \
		{Exit the application} \
		-command [code $this quit] \
		-accelerator "Control-q"
	}
	
	if {$iscat_} {
	    # Edit menu
	    set m [add_menubutton Edit "Display Edit menu"]
	    set edit_menu_ $m
	    
	    if {$itk_option(-catType) == 1 || $itk_option(-catType) == 3} {
		$m configure -foreground $menu_text_ot_ -background $menu_bg_ot_		
	    } else {
		$m configure -foreground $menu_text_odf_ -background $menu_bg_odf_
	    }
	    
	    add_menuitem $m command "Remove selected" \
		{Remove selected rows from the local catalog} \
		-command [code $this remove_selected] \
		-state disabled
	    
	    add_menuitem $m command "Enter new object..." \
		{Enter the data for a new object for the local catalog} \
		-command [code $this enter_new_object] \
		-state disabled
	    
	    add_menuitem $m command "Edit selected object..." \
		{Edit the data for the selected object in the local catalog} \
		-command [code $this edit_selected_object] \
		-state disabled
	}
	
	# Options menu
	set m [add_menubutton Options "Display Options menu"]
	set options_menu_ $m

	
	if {$itk_option(-catType) == 1 || $itk_option(-catType) == 3} {
	    $m configure -foreground $menu_text_ot_ -background $menu_bg_ot_ 			
	} else {
	    $m configure -foreground $menu_text_odf_ -background $menu_bg_odf_ 
	}
	
	if {$iscat_} {
	    add_menuitem $m command "Set Sort Columns..." \
		{Set options for sorting the query results} \
		-command [code $this sort_dialog] \
		-state disabled
	    
	    add_menuitem $m command "Hide/Show Columns..." \
		{Set options for displaying columns of the query results} \
		-command [code $this select_columns] \
		-state disabled
	    
	    add_menuitem $m command "Set Plot Symbols..." \
		{Set the symbol (color, size, etc.) to use to plot objects} \
		-command [code $this set_plot_symbols] \
		-state disabled
	}

	# The stuff below (switching between instrument types) requires more work...

	# Instruments menu (for OT menu only)
#	if {$itk_option(-catType) == 1 || $itk_option(-catType) == 3} {
#	    set m [add_menubutton Instrument "Choose the instrument for which to make a MOS mask"]
#	    set inst_menu_ $m
#	    if {$itk_option(-catType) == 1 || $itk_option(-catType) == 3} {
#		$m configure -foreground $menu_text_ot_ -background $menu_bg_ot_		
#	    } else {
#		$m configure -foreground $menu_text_odf_ -background $menu_bg_odf_ 
#	    }
#	    
#	    add_menuitem $m command "Select $itk_option(-instType) Detector" \
#		{Select instrument for this MOS mask} \
#		-state disabled 
#	    
#	    # Open detector config file for reading. 
#	    set detFile [open "$home/config/Detectors.dat" r ]
#	    
#	    while {[gets $detFile line] >= 0} {
#		if {$line == ""} {
##		    continue
#		} elseif {[string index $line 0 ] == "#" } {
#		    continue
#		}
#		
#		# remove all leading and trailing white space
#		# condense internal white space to single blanks
#		# split the string into a list
#		set line [string trim $line ]
#		set line [regsub -all "\[ \t]+" $line { }]
#		set line [split $line " " ]
#		
#		# Extract instrument type, detector id and pixel scale from detector property list. 
#		set inst_status [lindex $line 0 ]
#		set inst_short  [lindex $line 1 ]
#		set inst_long   [lindex $line 3 ]
#		
#		# Add the current detector to the radio buttons
#		if { $inst_status == 1 && $inst_short == $itk_option(-instType)} {
#		    $m add radiobutton \
#			-label $inst_short \
#			-state active \
#			-selectcolor black \
#			-command [code $this setInstType $inst_short]
#		}
#		# Add the other detectors to the radio buttons
#		if { $inst_status == 1 && $inst_short != $itk_option(-instType)} {
#		    $m add radiobutton \
#			-label $inst_short \
#			-selectcolor black \
#			-command [code $this setInstType $inst_short]
#		}
#	    }
#	}
    }


    #############################################################
    #  Name: setInstType
    #
    #  Description:
    #   Sets the current instrument type being used by gmmps. 
    #############################################################
    #############################################################

#    public method setInstType {instrument} {
#	set itk_option(-instType) $instrument
#
#	# Erase and redraw boundaries
#	drawBoundaries EraseAll
#	drawBoundaries
#
#	# Set text labels accordingly
#	$w_.l2 config -text $instrument
#   }

    #############################################################
    #  Name: set_detector
    #
    #  Description:
    #   Sets the current instrument detector being used by gmmps. 
    #############################################################
    #############################################################

    public method set_detector {detInfo} {
	# This function gets called BEFORE mps_add_dialog_buttons()
	# which sets these globals. drawBoundaries() relies on them
	# being set, so we have to do that here.
	global ::cbo_barcode ::cbo_maskarea ::cbo_detarea
	global ::cbo_detgaps ::cbo_pointing
	set cbo_maskarea 0
	set cbo_barcode 0
	set cbo_detarea 0
	set cbo_detgaps 0
	set cbo_pointing 0

	# Now for the relevant stuff
	set detector_ $detInfo
	set detName [lindex $detInfo 3 ]
	set instType $itk_option(-instType)
	set target_canvas_ .skycat1.image.imagef.canvas
	drawBoundaries
    }


    #############################################################
    #  Name: set_menu_states
    #
    #  Description:
    #   Enable or disable some menus
    #############################################################
    #############################################################
    
    protected method set_menu_states {} {
	#debugStatement "\n#vmAstroCat::set_menu_states Entry.#"
	
	$options_menu_ entryconfig "Set Sort Columns..." -state normal
	$options_menu_ entryconfig "Hide/Show Columns..." -state normal

	if {[$w_.cat iswcs] || [$w_.cat ispix]} {
	    set state normal
	} else {
	    set state disabled
	}
	$options_menu_ entryconfig "Set Plot Symbols..." -state $state

	# determine states for some menu items
	if {"[$w_.cat servtype]" == "local"} {
	    set state [set sstate normal]
	} else {
	    set state [set sstate disabled]
	    # allow user to edit search cols if URL contains "%cond"
	    if {[string first "%cond" [$w_.cat url]] >= 0} {
		set sstate normal
	    } 
	}
	$edit_menu_ entryconfig "Remove selected" -state $state
	$edit_menu_ entryconfig "Enter new object..." -state $state
	$edit_menu_ entryconfig "Edit selected object..." -state $state
    }
    
    
    #############################################################
    #  Name: update_catalog_menus
    #
    #  Description:
    #   Update all of the catalog menus in all instances to show 
    #   the current catalog info. 
    #############################################################
    #############################################################

    public proc update_catalog_menus {} {
	#debugStatement "\n#vmAstroCat::update_catalog_menus Entry.#"

	foreach i [array names catalog_menu_info_] {
	    if {[winfo exists [utilNamespaceTail $i]]} {
		eval $catalog_menu_info_($i)
	    }
	}

	#debugStatement "#vmAstroCat::update_catalog_menus Exit.\n#"
    }
    

    #############################################################
    #  Name: set_plot_symbols
    #
    #  Description:
    #   Pop up a dialog to set the plot symbols to use for this
    #   catalog.
    #############################################################
    #############################################################

    public method set_plot_symbols {} {
	#debugStatement "\n#vmAstroCat::set_plot_symbols Entry.#"

	set columns $itk_option(-headingx)
	#debugStatement "vmAC: set_plot_symbols: $columns"
	if {[llength $columns] == 0} {
	    info_dialog "Please make a query first so that the column names are known" $w_
	    return
	}

	utilReUseWidget cat::SymbolConfig $w_.symconf \
	    -catalog $itk_option(-catalog) \
	    -astrocat [code $w_.cat] \
	    -columns $columns \
	    -command [code $this plot]
    }


    #############################################################
    #  Name: update_search_options
    #
    #  Description:
    #    Update the search option entries after they have been edited
    #############################################################
    #############################################################

    public method update_search_options {} {
	$searchopts_ update_search_options
    }


    #############################################################
    #  Name: close
    #
    #  Description:
    #    Close this window
    #############################################################
    #############################################################

    public method close {} {
	global ::cbo_bands ::cbo_slits ::cbo_ot_slits

	catch {$this delete_objects }
	
	if {$itk_option(-catType) == 2 || $itk_option(-catType) == 4} {
	    if {$cbo_slits} {
		catch {clearslits }
	    }
	    if {$cbo_bands} {
		catch {.skycat1.image.imagef.canvas delete bandrects}
	    }
	}
	if {$itk_option(-catType) == 1 || $itk_option(-catType) == 3} {
	    if {$cbo_ot_slits} {
		catch {clearslits_OT }
	    }
	}
        destroy $w_ 
    }

    
    #############################################################
    #  Name: sort_dialog
    #
    #  Description:
    #   Pop up a dialog to sort the list
    #############################################################
    #############################################################

    public method sort_dialog {} {
	$results_ sort_dialog
    }

    
    
    #############################################################
    #  Name: set_sort_cols
    #
    #  Description:
    #   This function is called when the user has selected columns 
    #   to sort the results by.
    #   The first arg is the sort columns, the second arg is the 
    #   order (increasing, decreasing)
    #############################################################
    #############################################################
    
    public method set_sort_cols {sort_cols sort_order} {
	if {"[$w_.cat sortcols]" != "$sort_cols" \
		|| "[$w_.cat sortorder]" != "$sort_order"} {
	    $w_.cat sortcols $sort_cols
	    $w_.cat sortorder $sort_order
	    cat::CatalogInfo::save {} $w_ 0
	    $results_ config -sort_cols $sort_cols -sort_order $sort_order
	    search
	}
    }


    #############################################################
    #  Name: select_columns
    #
    #  Description:
    #   Pop up a dialog to select table columns to display
    #############################################################
    #############################################################

    public method select_columns {} {
	$results_ select_columns
    }


    #############################################################
    #  Name: set_show_cols
    #
    #  Description:
    #   Called when the user has selected columns to show
    #############################################################
    #############################################################
    
    public method set_show_cols {cols} {
	set show [$w_.cat showcols]
	if {"$show" == ""} {
	    set show [$results_ get_headings]
	}
    }


    #############################################################
    #  Name: reset_table
    #
    #  Description:
    #    Reset table dialogs if needed
    #############################################################
    #############################################################

    public method reset_table {} {
	#debugStatement "reset_table: entry...."
	$results_ reset
	$results_ set_options {MORE PREVIEW more preview} Show 0
	$w_.cat showcols {}
	$w_.cat sortcols {}
	cat::CatalogInfo::save {} $w_ 0
    }

    
    
    #############################################################
    #  Name: clear
    #
    #  Description:
    #   Clear the table listing
    #############################################################
    #############################################################
    
    public method clear {} {
	catch {$results_ clear}
    }

    
    #############################################################
    #  Name: print
    #
    #  Description:
    #   Pop up a dialog to print the table listing
    #############################################################
    #############################################################
    
    public method print {} {
	$results_ print_dialog
    }

    
    #############################################################
    #  Name: local_catalog
    #
    #  Description:
    #   Called from menu item "Load from file..." to load in a catalog.
    #   This function asks the user for the name of a local 
    #   catalog file and then opens a window for the catalog.
    #
    #   id is an optional unique id to be associated with a new
    #   catalog widget.
    #
    #   classname is the name of the vmAstroCat subclass to use to 
    #   create new catalog widgets (defaults to "vmAstroCat").
    #
    #   $debug is a flag set from the command line arg.
    #
    #   $tType is the catalog type, Object Table, Master OT.
    #
    #   $w should be the top level window of the caller, 
    #   if specified.
    #############################################################
    #############################################################

    
    public proc local_catalog {{id ""} {classname vmAstroCat} {debug 0} tType {w ""} {instType "NONE"}} {

	# Debuglevel is being set here because this is a procedure.
	global acDebugLevel
	set acDebugLevel 0

	set loadedImage [$target_image_ cget -file ]
	
	if {$loadedImage == ""} {
	    error_dialog "Please load a preimage file into Skycat before loading an object catalog."
	    return
	}

	# Set file pattern based on catalog type. 
	if { $tType == 1 } {
	    # .cat OT
	    set filePattern *.cat
	} elseif { $tType == 3 } {
	    # .fits OT	
	    set filePattern *.fits
	} elseif { $tType == 2 } {
	    # .cat ODF
	    set filePattern *ODF*.cat
	} elseif { $tType == 4 } {
	    # .fits ODF
	    set filePattern *ODF*.fits
	}
	
	# Prompt user to choose file to open.
	# the $w argument for filename_dialog forces the dialog into the foreground
	set file [filename_dialog [pwd ] $filePattern $w]
	if {"$file" != ""} {
	    if {[file isfile $file]} {
		if {[catch {$astrocat_ check $file} msg]} {
		    error_dialog $msg
		} else {
		    select_catalog $file local $id $classname $debug $tType $w $instType
		}
	    } else {
		error_dialog "There is no file named '$file'"
	    }
	}
    }
    
    
    #############################################################
    #  Name: save_as
    #
    #  Description:
    #   Save the current data to a local catalog
    #############################################################
    #############################################################
    
    public method save_as {} {
	$results_ save_as
    }


    #############################################################
    #  Name: add_to
    #
    #  Description:
    #   Add the rows in the current listing to a local catalog file
    #############################################################
    #############################################################
    
    public method add_to {} {
	$results_ add_to
    }

    
    #############################################################
    #  Name: add_selected
    #
    #  Description:
    #   Add the currently selected rows to a local catalog file
    #############################################################
    #############################################################
    
    public method add_selected {} {
	$results_ add_selected
    }

    
    #############################################################
    #  Name: remove_selected
    #
    #  Description:
    #   Remove the currently selected rows from a local catalog file
    #############################################################
    #############################################################

    public method remove_selected {} {
	$results_ remove_selected

	# update the display
	clear
	search
    }

    
    #############################################################
    #  Name: enter_new_object
    #
    #  Description:
    #   Pop up a dialog to enter the data for a new object for a local catalog
    #############################################################
    #############################################################

    public method enter_new_object {} {
	$results_ enter_new_object [code $this search]
    }

    
    #############################################################
    #  Name: edit_selected_object
    #
    #  Description:
    #   Pop up a window so that the user can edit the selected object(s)
    #############################################################
    #############################################################

    public method edit_selected_object {} {
	$results_ edit_selected_object [code $this search]
    }


    ####################################################################################################
    #  Name: drawBoundaries
    #
    #  Description:
    #   Reads in the information about field of view, possible gaps and the overall detector dimensions
    #   Toggles the corresponding boundaries depending on the status of the respective checkbuttons; 
    #   returns the spectral min and max coordinates available to determine possible spectral truncation
    ####################################################################################################
    ####################################################################################################

    public method drawBoundaries {{altDet ""} {altDetComboText ""}} {
	
	global ::cbo_barcode ::cbo_maskarea ::cbo_detarea
	global ::cbo_detgaps ::cbo_pointing

	set bg black
	set fg red
	set crosshair green
	set gapcol blue
	set cbo_tag_maskarea cbo_ma
	set cbo_tag_detarea  cbo_da
	set cbo_tag_barcode  cbo_bc
	set cbo_tag_pointing cbo_po
	set cbo_tag_detgaps  cbo_dg

	set cbo_alt_tag_maskarea cbo_alt_ma
	set cbo_alt_tag_detarea  cbo_alt_da
	set cbo_alt_tag_detgaps  cbo_alt_dg
	set alt_fg_color #bbb

	set target_canvas_ .skycat1.image.imagef.canvas
	set target_image_ image2
	set catname $itk_option(-catalog)
	set home $::env(GMMPS)
	
	# The IMAGE pixel scale, NOT the detector pixel scale
	set pixelScale $itk_option(-pixs)

	set instType $itk_option(-instType)

	# Erase everything and leave
	if {$altDet == "EraseAll"} {
	    $target_canvas_ delete cbo_ma
	    $target_canvas_ delete cbo_da
	    $target_canvas_ delete cbo_dg
	    $target_canvas_ delete cbo_alt_ma
	    $target_canvas_ delete cbo_alt_da
	    $target_canvas_ delete cbo_alt_dg
	    return
	}

	# Show no alternative detectors if not desired
	if {$altDet == "EraseAltDet"} {
	    $target_canvas_ delete cbo_alt_ma
	    $target_canvas_ delete cbo_alt_da
	    $target_canvas_ delete cbo_alt_dg
	    return
	}

	# Do we plot the current Detector, live at the telescope, or in addition some alternative?
	if {$altDet == ""} {
	    if { $pixelScale == 0.0 || $pixelScale == "NOT FOUND" } { 
		error_dialog "Error: Pixel scale undetermined!"
		puts "ERROR: Pixel scale undetermined!"
		return
	    }
	    set fovfilename $home/config/${instType}_current_fov.dat
	    catch { set fovList [open $fovfilename r] }
	} else {
	    # delete possibly drawn previous patterns, and then prepare to plot again
	    $target_canvas_ delete cbo_alt_ma
	    $target_canvas_ delete cbo_alt_da
	    $target_canvas_ delete cbo_alt_dg
	    set fovfilename $home/config/${altDet}_fov.dat
	    catch { set fovList [open $fovfilename r] }

	    # If the current inst is F2, then the "gaps" checkbutton would not be shown.
	    # But we want to display gaps for the old detectors, hence:
	    if {$instType == "F2"} {
		set cbo_detgaps 1
	    }
	}

	# Declare a couple of lists that contain the vertices for the 
	# corners of fov, gaps, barcode and detector dimensions
	set fov_x {}
	set fov_y {}
	set dim_x {}
	set dim_y {}
	set bar_x {}
	set bar_y {}
	set gap1_x {}
	set gap1_y {}
	set gap2_x {}
	set gap2_y {}

	set CRPIX1 [get_crpix_image CRPIX1]
	set CRPIX2 [get_crpix_image CRPIX2]

	# Counters for the vertices
	set n_fov 0
	set n_dim 0
	set n_bar 0
	set n_gap1 0
	set n_gap2 0

	# Read fov values into lists
	while {[gets $fovList line] >= 0} {
	    # remove all leading and trailing white space
	    # condense internal white space to single blanks
	    set line [string trim $line ]
	    set line [regsub -all "\[ \t]+" $line { }]
	    # extract the paired values when the line matches a keyword (and is not a comment)
	    if {[string match #* $line] == 0} {
		if {[string match FOV_CORNER* $line] == 1} {
		    set fov_x [lappend fov_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set fov_y [lappend fov_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_fov
		}
		if {[string match DIM_CORNER* $line] == 1} {
		    set dim_x [lappend dim_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set dim_y [lappend dim_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_dim
		}
		if {[string match BAR_CORNER* $line] == 1} {
		    set bar_x [lappend bar_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set bar_y [lappend bar_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_bar
		}
		if {[string match GAP1_CORNER* $line] == 1} {
		    set gap1_x [lappend gap1_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set gap1_y [lappend gap1_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_gap1
		}
		if {[string match GAP2_CORNER* $line] == 1} {
		    set gap2_x [lappend gap2_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set gap2_y [lappend gap2_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_gap2
		}
	    }
	}

	::close $fovList

	# Do some consistency checks with the dimensions and the fov
	if {$n_fov < 8} {
	    error_dialog "ERROR: $fovList incomplete. Found $n_fov vertices instead of 8 for FOV_CORNER."
	    return
	}
	if {$n_dim < 4} {
	    error_dialog "ERROR: $fovList incomplete. Found $n_dim vertices instead of 4 for DIM_CORNER."
	    return
	}

	# I do not put a check concerning the bar code here. It will simply not be drawn if not present.


	# SCHEME (likewise for GAP and DIM, but with 4 vertices only)
	# The 'X' in the center is represented by $CRPIX1 and CRPIX2
	# The corners X1-8/Y1-8 are in arcsec with respect to 'X'
	#
	#
	#       X4-Y4 ----- X5-Y5
	#       /              \
	#      /                \
        #   X3-Y3              X6-Y6
	#     |                  | 
	#     |         X        | 
	#     |                  | 
	#   X2-Y2              X7-Y7
	#      \                /
	#       \              /
	#       X1-Y1 ----- X8-Y8


	# Convert the coordinates so that we can plot them

	set xtmp 0
	set ytmp 0

	set xtext [ expr [lindex $dim_x 0] + 50]
	set ytext [ expr [lindex $dim_y 0] + 50]
	$target_image_ convert coords $xtext $ytext image xtext ytext canvas

	for {set i 0} {$i < $n_fov} {incr i 1} {
	    $target_image_ convert coords [lindex $fov_x $i] [lindex $fov_y $i] image xtmp ytmp canvas
	    set fov_x [lreplace $fov_x $i $i $xtmp]
	    set fov_y [lreplace $fov_y $i $i $ytmp]
	}

	for {set i 0} {$i < $n_dim} {incr i 1} {
	    $target_image_ convert coords [lindex $dim_x $i] [lindex $dim_y $i] image xtmp ytmp canvas
	    set dim_x [lreplace $dim_x $i $i $xtmp]
	    set dim_y [lreplace $dim_y $i $i $ytmp]
	}

	if {$n_bar == 4} {
	    for {set i 0} {$i < $n_bar} {incr i 1} {
		$target_image_ convert coords [lindex $bar_x $i] [lindex $bar_y $i] image xtmp ytmp canvas
		set bar_x [lreplace $bar_x $i $i $xtmp]
		set bar_y [lreplace $bar_y $i $i $ytmp]
	    }
	}

	if {$n_gap1 == 4} {
	    for {set i 0} {$i < $n_gap1} {incr i 1} {
		$target_image_ convert coords [lindex $gap1_x $i] [lindex $gap1_y $i] image xtmp ytmp canvas
		set gap1_x [lreplace $gap1_x $i $i $xtmp]
		set gap1_y [lreplace $gap1_y $i $i $ytmp]
	    }
	}

	if {$n_gap2 == 4} {
	    for {set i 0} {$i < $n_gap2} {incr i 1} {
		$target_image_ convert coords [lindex $gap2_x $i] [lindex $gap2_y $i] image xtmp ytmp canvas
		set gap2_x [lreplace $gap2_x $i $i $xtmp]
		set gap2_y [lreplace $gap2_y $i $i $ytmp]
	    }
	}

	# Plot the stuff
	# This depends on the value of the variables associated with the CheckBoxes (called cbo_something)
	set n_fov_mod [expr $n_fov - 1]
	set n_dim_mod [expr $n_dim - 1]
	set n_bar_mod [expr $n_bar - 1]
	set n_gap1_mod [expr $n_gap1 - 1]
	set n_gap2_mod [expr $n_gap2 - 1]

	# Toggle the area available for slitlets
	if {$cbo_maskarea == 1} {
	    for {set i 0} {$i < $n_fov} {incr i 1} {
		if {$i < $n_fov_mod} {
		    set ii [expr $i + 1]
		} else {
		    set ii 0
		}
		if {$altDet == ""} {
		    $target_canvas_ create line \
			[lindex $fov_x $i]  [lindex $fov_y $i] \
			[lindex $fov_x $ii] [lindex $fov_y $ii] \
			-fill $fg -width 3 -tags $cbo_tag_maskarea
		} else {
		    $target_canvas_ create line \
			[lindex $fov_x $i]  [lindex $fov_y $i] \
			[lindex $fov_x $ii] [lindex $fov_y $ii] \
			-fill $alt_fg_color -width 3 -tags $cbo_alt_tag_maskarea
		}
	    }
	} else {
	    $target_canvas_ delete cbo_ma
	    $target_canvas_ delete cbo_alt_ma
	}

	# Toggle the detector dimensions
	if {$cbo_detarea == 1} {
	    for {set i 0} {$i < $n_dim} {incr i 1} {
		if {$i < $n_dim_mod} {
		    set ii [expr $i + 1]
		} else {
		    set ii 0
		}
		if {$altDet == ""} {
		    $target_canvas_ create line \
			[lindex $dim_x $i]  [lindex $dim_y $i] \
			[lindex $dim_x $ii] [lindex $dim_y $ii] \
			-fill $crosshair -width 2 -tags $cbo_tag_detarea
		} else {
		    $target_canvas_ create line \
			[lindex $dim_x $i]  [lindex $dim_y $i] \
			[lindex $dim_x $ii] [lindex $dim_y $ii] \
			-fill $alt_fg_color -width 2 -tags $cbo_alt_tag_detarea
		}
	    }

	    # A ID label at the lower left to indicate the instrument
	    if {$altDet == ""} {
		$target_canvas_ create text $xtext $ytext -anchor w -font {Arial 16} \
		    -text $instType -tags $cbo_tag_detarea -fill $crosshair
	    } else {
		$target_canvas_ create text $xtext $ytext -anchor w -font {Arial 16} \
		    -text $altDetComboText -tags $cbo_alt_tag_detarea -fill $alt_fg_color
	    }
	} else {
	    $target_canvas_ delete cbo_da
	    $target_canvas_ delete cbo_alt_da
	}

	# Toggle the barcode (if any)
	if {$n_bar == 4} {
	    if {$cbo_barcode == 1} {
		set bar_left   [lindex $bar_x 0]
		set bar_top    [lindex $bar_y 1]
		set bar_right  [lindex $bar_x 2]
		set bar_bottom [lindex $bar_y 3]
		$target_canvas_ create rect $bar_left $bar_top $bar_right $bar_bottom \
		    -outline black -fill #BBB -width 1 -tags $cbo_tag_barcode
	    } else {
		$target_canvas_ delete cbo_bc
	    }
	}

	# Toggle the 1st gap (if any)
	if {$n_gap1 == 4} {
	    if {$cbo_detgaps == 1} {
		if {$altDet == ""} {
		    $target_canvas_ create rect \
			[lindex $gap1_x 0] [lindex $gap1_y 1] \
			[lindex $gap1_x 2] [lindex $gap1_y 3] \
			-outline $gapcol -fill $gapcol -stipple gray12 -width 1 -tags $cbo_tag_detgaps
		} else {
		    $target_canvas_ create rect \
			[lindex $gap1_x 0] [lindex $gap1_y 1] \
			[lindex $gap1_x 2] [lindex $gap1_y 3] \
			-outline $alt_fg_color -fill $alt_fg_color -stipple gray12 -width 1 -tags $cbo_alt_tag_detgaps
		}
	    } else {
		$target_canvas_ delete cbo_dg
		$target_canvas_ delete cbo_alt_dg
	    }
	}

	# Toggle the 2nd gap (if any)
	if {$n_gap2 == 4} {
	    if {$cbo_detgaps == 1} {
		if {$altDet == ""} {
		    $target_canvas_ create rect \
			[lindex $gap2_x 0] [lindex $gap2_y 1] \
			[lindex $gap2_x 2] [lindex $gap2_y 3] \
			-outline $gapcol -fill $gapcol -stipple gray12 -width 1 -tags $cbo_tag_detgaps
		} else {
		    $target_canvas_ create rect \
			[lindex $gap2_x 0] [lindex $gap2_y 1] \
			[lindex $gap2_x 2] [lindex $gap2_y 3] \
			-outline $alt_fg_color -fill $alt_fg_color -stipple gray12 -width 1 -tags $cbo_alt_tag_detgaps
		}
	    } else {
		$target_canvas_ delete cbo_dg
		$target_canvas_ delete cbo_alt_dg
	    }
	}

	# Elements we need for the center mark cross hair
	set cw 20
	set circle1_ulx [expr $CRPIX1-$cw]
	set circle1_uly [expr $CRPIX2+$cw]
	set circle1_lrx [expr $CRPIX1+$cw]
	set circle1_lry [expr $CRPIX2-$cw]

	set line1_x1 $CRPIX1
	set line1_x2 $CRPIX1
	set line1_y1 [expr $CRPIX2 + 1 * $cw]
	set line1_y2 [expr $CRPIX2 + 2 * $cw]
	set line2_x1 $CRPIX1
	set line2_x2 $CRPIX1
	set line2_y1 [expr $CRPIX2 - 1 * $cw]
	set line2_y2 [expr $CRPIX2 - 2 * $cw]
	set line3_x1 [expr $CRPIX1 - 1 * $cw]
	set line3_x2 [expr $CRPIX1 - 2 * $cw]
	set line3_y1 $CRPIX2
	set line3_y2 $CRPIX2
	set line4_x1 [expr $CRPIX1 + 1 * $cw]
	set line4_x2 [expr $CRPIX1 + 2 * $cw]
	set line4_y1 $CRPIX2
	set line4_y2 $CRPIX2

	$target_image_ convert coords $circle1_ulx $circle1_uly image circle1_ulx circle1_uly canvas
	$target_image_ convert coords $circle1_lrx $circle1_lry image circle1_lrx circle1_lry canvas
	$target_image_ convert coords $line1_x1 $line1_y1 image line1_x1 line1_y1 canvas
	$target_image_ convert coords $line1_x2 $line1_y2 image line1_x2 line1_y2 canvas
	$target_image_ convert coords $line2_x1 $line2_y1 image line2_x1 line2_y1 canvas
	$target_image_ convert coords $line2_x2 $line2_y2 image line2_x2 line2_y2 canvas
	$target_image_ convert coords $line3_x1 $line3_y1 image line3_x1 line3_y1 canvas
	$target_image_ convert coords $line3_x2 $line3_y2 image line3_x2 line3_y2 canvas
	$target_image_ convert coords $line4_x1 $line4_y1 image line4_x1 line4_y1 canvas
	$target_image_ convert coords $line4_x2 $line4_y2 image line4_x2 line4_y2 canvas

	
	# Toggle the central crosshair
	if {$cbo_pointing == 1} {
	    $target_canvas_ create oval $circle1_ulx $circle1_uly $circle1_lrx $circle1_lry \
		-outline $crosshair -width 2 -tags $cbo_tag_pointing
	    $target_canvas_ create line $line1_x1 $line1_y1 $line1_x2 $line1_y2 \
		-fill $crosshair -width 2 -tags $cbo_tag_pointing
	    $target_canvas_ create line $line2_x1 $line2_y1 $line2_x2 $line2_y2 \
		-fill $crosshair -width 2 -tags $cbo_tag_pointing
	    $target_canvas_ create line $line3_x1 $line3_y1 $line3_x2 $line3_y2 \
		-fill $crosshair -width 2 -tags $cbo_tag_pointing
	    $target_canvas_ create line $line4_x1 $line4_y1 $line4_x2 $line4_y2 \
		-fill $crosshair -width 2 -tags $cbo_tag_pointing
	} else {
	    $target_canvas_ delete cbo_po
	}

	# Calculate the overall detector range
	set detxmin [lindex $dim_x 0]
	set detxmax [lindex $dim_x 3]
	set detymin [lindex $dim_y 0]
	set detymax [lindex $dim_y 1]
	set detrange [concat $detxmin $detxmax $detymin $detymax]

	# Reset the gap checkbutton to its correct state if we are making a mask for F2
	if {$instType == "F2" && $altDet != ""} {
	    set cbo_detgaps 0
	}

	return $detrange
    }


    #############################################################
    #  Name: check_proximity
    #
    #  Description:
    #  This function checks whether an alignment star box is 
    #  close to a gap or boundary
    #############################################################
    #############################################################

    public method check_proximity {xobj yobj id dispDirection} {

	set pixelScale $itk_option(-pixs)
	set instType $itk_option(-instType)
	set home $::env(GMMPS)
	set fovfilename $home/config/${instType}_current_fov.dat
	catch { set fovList [open $fovfilename r]}

	set fov_x {}
	set fov_y {}
	set gap1_x {}
	set gap1_y {}
	set gap2_x {}
	set gap2_y {}

	set CRPIX1 [get_crpix_image CRPIX1]
	set CRPIX2 [get_crpix_image CRPIX2]

	set error_gap ""
	set error_boundary ""
	set testvar 0.0

	# The size of our acquisition boxes is 2 arcsecs
	set acq_box_size [expr 2.0 / $pixelScale]

	# Counters for the vertices
	set n_fov 0
	set n_gap1 0
	set n_gap2 0

	# Read fov values into lists
	while {[gets $fovList line] >= 0} {
	    # remove all leading and trailing white space
	    # condense internal white space to single blanks
	    set line [string trim $line ]
	    set line [regsub -all "\[ \t]+" $line { }]
	    # extract the paired values when the line matches a keyword (and is not a comment)
	    if {[string match #* $line] == 0} {
		if {[string match FOV_CORNER* $line] == 1} {
		    set fov_x [lappend fov_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set fov_y [lappend fov_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_fov
		}
		if {[string match GAP1_CORNER* $line] == 1} {
		    set gap1_x [lappend gap1_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set gap1_y [lappend gap1_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_gap1
		}
		if {[string match GAP2_CORNER* $line] == 1} {
		    set gap2_x [lappend gap2_x [expr [lindex $line 1 ] / $pixelScale + $CRPIX1 ]]
		    set gap2_y [lappend gap2_y [expr [lindex $line 2 ] / $pixelScale + $CRPIX2 ]]
		    incr n_gap2
		}
	    }
	}

	::close $fovList
 
	# Check box distance from gaps
	if {$n_gap1 > 0} {
	    if {$dispDirection == "horizontal"} {
		set gap1 [lindex $gap1_x 0]
		set gap2 [lindex $gap1_x 3]
		set testvar $xobj
		set dist1 [expr abs($testvar - $gap1)]
		set dist2 [expr abs($testvar - $gap2)]
	    } else {
		set gap1 [lindex $gap1_y 0]
		set gap2 [lindex $gap1_y 1]
		set testvar $yobj
		set dist1 [expr abs($testvar - $gap1)]
		set dist2 [expr abs($testvar - $gap2)]
	    } 
	    # If close (star within 3 arcsec, i.e. box within 2 arcsec)
	    if {$dist1 <= [expr 1.5 * $acq_box_size] || 
		$dist2 <= [expr 1.5 * $acq_box_size]} {
		set error_gap "WARNING: Acq box $id within 2\" of gap!\n"
	    }
	    # If truncated, or within less than 0.5 arcsec from gap
	    if {$dist1 <= [expr 0.75 * $acq_box_size] || 
		$dist2 <= [expr 0.75 * $acq_box_size] ||
		($testvar >= $gap1 && $testvar <= $gap2)} {
		set error_gap "WARNING: Acq box $id too close to gap!\n"
	    }
	}
	if {$n_gap2 > 0} {
	    if {$dispDirection == "horizontal"} {
		set gap1 [lindex $gap2_x 0]
		set gap2 [lindex $gap2_x 3]
		set testvar $xobj
		set dist1 [expr abs($testvar - $gap1)]
		set dist2 [expr abs($testvar - $gap2)]
	    } else {
		set gap1 [lindex $gap2_y 0]
		set gap2 [lindex $gap2_y 1]
		set testvar $yobj
		set dist1 [expr abs($testvar - $gap1)]
		set dist2 [expr abs($testvar - $gap2)]
	    } 
	    # If close (star within 3 arcsec, i.e. box within 2 arcsec)
	    if {$dist1 <= [expr 1.5 * $acq_box_size] || 
		$dist2 <= [expr 1.5 * $acq_box_size]} {
		set error_gap "WARNING: Acq box $id within 2\" of gap!\n"
	    }
	    # If truncated, or within less than 0.5 arcsec from gap
	    if {$dist1 <= [expr 0.75 * $acq_box_size] || 
		$dist2 <= [expr 0.75 * $acq_box_size] ||
		($testvar >= $gap1 && $testvar <= $gap2)} {
		set error_gap "WARNING: Acq box $id too close to gap!\n"
	    }
	}

	# Check box distance from mask area boundary

	# Needed to evaluate smallest distance to a gap
	set mindist [expr 10.0 * $acq_box_size]
	if {$n_fov == 8} {
	    set n_fov_mod [expr $n_fov - 1]
	    for {set i 0} {$i < $n_fov} {incr i 1} {
		if {$i < $n_fov_mod} {
		    set ii [expr $i + 1]
		} else {
		    set ii 0
		}
		# Distance from acq star to line made by indices i and ii
		set x1 [lindex $fov_x $i]
		set x2 [lindex $fov_x $ii]
		set y1 [lindex $fov_y $i]
		set y2 [lindex $fov_y $ii]
		if {$x1 != $x2} {
		    set slope [expr ($y2-$y1) / ($x2-$x1)]
		    set distance [expr abs(($slope * ($x1-$xobj) - ($y1-$yobj)) / sqrt(1.+$slope*$slope))]
		} else {
		    set distance [expr abs($x1-$xobj)]
		}
		if {$distance <= [expr 2.0 * $acq_box_size] && $distance <= $mindist} {
		    set error_boundary "WARNING: Acq box $id within 3\" of boundary!\n"
		    set mindist $distance
		}
		if {$distance <= [expr 1.25 * $acq_box_size] && $distance <= $mindist} {
		    set error_boundary "WARNING: Acq box $id too close to boundary!\n"
		    set mindist $distance
		}
	    }
	}
	
	# Return the concatenated error string
	return $error_gap$error_boundary
   }


    #############################################################
    #  Name: slits
    #
    #  Description:
    #   This function draws the slits for ODF files.
    #############################################################
    #############################################################

    public method slits {{acqOnly ""}} {
	global ::cbo_slits ::cbo_shading ::cbo_acqonly
	global ::num_tot_spectra ::num_yellow ::num_red

	set target_canvas_ .skycat1.image.imagef.canvas
	set target_image_ image2
	set tags_shading shading
	set proximity_alert ""
	set proximity_alert_print ""

	# First, clear everything
	clearslits

	# If acq only is activated, toggling slits on/off doesn't do anything
	# as only acq slits are highlighted. To avoid this, we force to switch
	# off the "acq only" setting if it is on and the user toggles the slit button
	if {$acqOnly == ""} {
	    if {$cbo_slits == 1 && $cbo_acqonly == 1} {
		set cbo_acqonly 0
	    }
	}

	# Clear slits if requested and return
	if {$cbo_slits == 0 && $cbo_acqonly == 0} {
	    $w_.odfPlotCBO_shading configure -state disabled
	    return
	}
	
	# Activate the shading checkbutton
	$w_.odfPlotCBO_shading configure -state normal
	# Delete previous shading (if any)
	if {$cbo_shading == 0} {
	    $target_canvas_ delete shading
	}	    

	# Leave if the pixelscale is unknown
	set pixelScale $itk_option(-pixs)
	if { $pixelScale == 0.0 || $pixelScale == "NOT FOUND" } { 
	    error_dialog "Error: Pixel scale undetermined!"
	    puts "ERROR: Pixel scale undetermined!"
	    return
	}

	# OK, Continue plotting slits
	set bg black
	set fg2 yellow
	set tags slitMark
	set acq_color cyan
	set footprintcolor white
	set catname $itk_option(-catalog)
	set instType $itk_option(-instType)
	set dispDirection [getDispDirection]
	# Pi/180
	set rad 0.01745329252

	# Draw the boundary, and retrieve the lower and upper spectral detector cutoffs
	# The boundaries are already in canvas coordinates!
	set detrange [drawBoundaries]
	set detxmin [lindex $detrange 0]
	set detxmax [lindex $detrange 1]
	set detymin [lindex $detrange 2]
	set detymax [lindex $detrange 3]
	if {$dispDirection == "horizontal"} {
	    set detspecmin $detxmin
	    set detspecmax $detxmax
	} else {
	    set detspecmin $detymin
	    set detspecmax $detymax
	}

	#  Determine spectrum length by greping for #Spectrum in catalog file.
	#  Very cheesy way to do it. Convert that length to image length
	set spect_len ""
	catch { set spect_len [expr [exec grep #Spectrum $catname | cut -f2 -d:] / 2.0] }
	if {$spect_len == "" } {
	    catch { 
		set spect_len [exec grep "#fits SPEC_LEN=" $catname | cut -f2 -d=] 
		set spect_len [string trim $spect_len "' \t"]
		set spect_len [expr $spect_len / 2.0]
	    }
	    if {$spect_len == "" } {
		set spect_len 3100
	    }
	}

	$target_image_ convert dist $spect_len $spect_len image spect_len spect_len canvas

	#  Cycle through each item in the objects list
	#  This gets num rows from total_rows(vmTableList)
	#  and info_ connections all rows in the display.
	set nobj [$results_ total_rows]
	set num_tot_spectra $nobj
	set num_acq 0
	set num_yellow 0
	set num_red 0

	for {set n 0} {$n < $nobj} {incr n} {
            #  For each item in the object list
            #  get the row, then parse out x, y, dimx, dimy.
            # BIG WARNING: it assumes order of the columns.
            # Which is kind of OK, because we create the Master OT file.
            # Col's start at 0, Col3 = x_ccd, Col4 = y_ccd,
            # Col5 = specpos_x, Col6 = specpos_y
            # Col7 = slitpos_x, Col8 = slitpos_y
            # Col9 = slitsize_x
	    # Col10 = slitsize_y
	    # Col11 = slittilt
            # Col13 = priority
            # All col's, (except x_ccd&y_ccd, specpos_*) are in arc sec. 
            # so need to convert to pixels, /pixelScale
	    
	    set row [lindex $info_ $n]

	    # cartesian object coordinates 
	    set obj_id [lindex $row 0]
	    set xobj [lindex $row 3]
	    set yobj [lindex $row 4]
	    # cartesian slit position coordinates
	    set xccd [expr [lindex $row 3] + ([lindex $row 7]/$itk_option(-pixs)) ]
	    set yccd [expr [lindex $row 4] + ([lindex $row 8]/$itk_option(-pixs)) ]
	    # slit offsets
	    set xshift [expr [lindex $row 7 ]/$itk_option(-pixs)]
	    set yshift [expr [lindex $row 8 ]/$itk_option(-pixs)]
	    # slit dimensions
	    set dimx [expr [lindex $row 9] / 2.0 / $itk_option(-pixs)]
	    set dimy [expr [lindex $row 10] / 2.0 / $itk_option(-pixs)]
	    set prior [lindex $row 13]
	    set waveccd ""
	    catch {set waveccd [lindex $row 15]}
	    set slittilt [lindex $row 11]
	    set tmp 0
            
	    # Normal slit color (untruncated slits)
	    set fg GhostWhite

	    #  Acq. objects are cyan if not highlighted
	    set acq_color cyan

	    # Check proximity for acquisition stars
	    if {$prior == 0} {
		set num_acq [expr $num_acq + 1]
		set proximity_alert [check_proximity $xobj $yobj $obj_id $dispDirection]
		if { $proximity_alert != ""} {
		    #reduce number of good acq stars by one
		    set num_acq [expr $num_acq - 1]
		    if {$proximity_alert_print == ""} {
			set proximity_alert_print "${proximity_alert}"
		    } else {
			set proximity_alert_print "${proximity_alert_print}${proximity_alert}"
		    }
		    # if in highlighting mode, change color, depending on proximity to gap
		    if {$cbo_acqonly == "1"} {
			# In this order! The string may contain both warnings, e.g. if a box is
			# close to a boundary and too close to a gap, then the stronger argument
			# (in that case too close to a gap) determines the color
			if {[string match *within* $proximity_alert] == 1} { 
			    set acq_color #ff0
			}
			if {[string match *too* $proximity_alert] == 1} { 
			    set acq_color #f00
			}
		    }
		}
	    }

	    #debugStatement "slits: Object position before: $x, $y"
	    $target_image_ convert coords $xobj $yobj image xobj yobj canvas
	    $target_image_ convert coords $xccd $yccd image xccd yccd canvas
	    $target_image_ convert dist $dimx $dimy image dimx dimy canvas
	    if {$waveccd != ""} {
                if {$dispDirection == "horizontal"} {
                    $target_image_ convert coords $waveccd $tmp image waveccd tmp canvas
                } else {
                    $target_image_ convert coords $tmp $waveccd image tmp waveccd canvas
                }
	    }

	    set slit_xmin [expr $xccd - $dimx]
	    set slit_xmax [expr $xccd + $dimx]
	    set slit_ymin [expr $yccd - $dimy]
	    set slit_ymax [expr $yccd + $dimy]

	    if {$dispDirection == "horizontal"} {
		set spec_xmin [expr $xccd - $spect_len]
		set spec_xmax [expr $xccd + $spect_len]
		set spec_ymin [expr $yccd - $dimy]
		set spec_ymax [expr $yccd + $dimy]
	    } else {
		set spec_xmin [expr $xccd - $dimx]
		set spec_xmax [expr $xccd + $dimx]
		set spec_ymin [expr $yccd - $spect_len]
		set spec_ymax [expr $yccd + $spect_len]
	    }

	    if {$dispDirection == "horizontal"} {
		
		######################################
		# horizontal dispersion (e.g. GMOS)
		######################################
		
		# For plotting tilted slits we need to shift vertices. Introducing three helper variables:
		# tilt_dx1 (offset for tilting a 1D slit)
		# tilt_dx2 (additional offset to compensate for narrowing effect when tilting 2D slit)
		# tilt_dcwl (offset by which we have to shift the "green line"; a simple fraction of tilt_dx1)

		# Zero the offsets in case of no tilt
		set tilt_dx1  0.
		set tilt_dx2  0.
		set tilt_dcwl 0.

		set tilt_dx1 [expr $dimy * tan ($slittilt * $rad)]
		set tilt_dx2 [expr $dimx / cos ($slittilt * $rad)]
		set tilt_dcwl [expr $tilt_dx1 * $yshift / $dimy]

		# Determine if the spectrum is truncated by the detector edges
		# If 0% <= truncation <= 10%, draw it normally
		# If 10% < truncation <= 20%, draw it with a yellow outline
		# If truncation > 20%, draw it with a red outline
		set totSpecLength [expr abs($spec_xmax - $spec_xmin)]
		set truncFraction 0.
		# canvas coordinates might be "inverted", hence the size comparisons might fail
		# and thus I need to check whether the minimum value is actually the numeric minimum 
		# in canvas coordinates. if not, flip.
		set spec_xmin_cv $spec_xmin
		set spec_xmax_cv $spec_xmax
		if {$spec_xmin_cv > $spec_xmax_cv} {
		    set spec_xmin_cv $spec_xmax
		    set spec_xmax_cv $spec_xmin
		}
		set detspecmin_cv $detspecmin
		set detspecmax_cv $detspecmax
		if {$detspecmin_cv > $detspecmax_cv} {
		    set detspecmin_cv $detspecmax
		    set detspecmax_cv $detspecmin
		}
		if {$spec_xmin_cv < $detspecmin_cv} {
		    set truncFraction [expr abs($detspecmin_cv - $spec_xmin_cv) / $totSpecLength]
		}
		if {$spec_xmax_cv > $detspecmax_cv} {
		    set truncFraction [expr abs($spec_xmax_cv - $detspecmax_cv) / $totSpecLength]
		}
		# Change color due to truncation only if not an acquisition source
		if {$prior != "0"} {
		    if {$truncFraction <= 0.1} {
			set footprintcolor $fg
		    } elseif {$truncFraction > 0.1 && $truncFraction <= 0.20 && $prior != "0"} {
			set footprintcolor yellow
			set num_yellow [expr $num_yellow + 1]
		    } elseif {$truncFraction > 0.20 && $prior != "0"} {
			set footprintcolor red
			set num_red [expr $num_red + 1]
		    }
		}

		# Skip this object if not an acquisition object and only acq objects should be plotted
		if {$cbo_acqonly == 1 && $prior != "0"} {
		    continue
		}
		
		# If there is a tilt
		# (Acq boxes are never tilted)
		if { $slittilt != 0. } {

		    # If requested, draw a hashed area to the left and to the right of the slit
		    if {$cbo_shading == 1} {
			set xmin_shade1 [maxval [expr $spec_xmin - $tilt_dx1] [expr $spec_xmin + $tilt_dx1]]
			set xmax_shade1 [minval [expr $xccd - $tilt_dx1 - $tilt_dx2] [expr $xccd + $tilt_dx1 - $tilt_dx2]]
			set xmin_shade2 [maxval [expr $xccd - $tilt_dx1 + $tilt_dx2] [expr $xccd + $tilt_dx1 + $tilt_dx2]]
			set xmax_shade2 [minval [expr $spec_xmax - $tilt_dx1] [expr $spec_xmax + $tilt_dx1]]
			$target_canvas_ create rect $xmin_shade1 $spec_ymax $xmax_shade1 $spec_ymin \
			    -fill #999 -stipple gray12 -width 0 -tags "$tags $tags_shading"
			$target_canvas_ create rect $xmin_shade2 $spec_ymax $xmax_shade2 $spec_ymin \
			    -fill #999 -stipple gray12 -width 0 -tags "$tags $tags_shading"
		    }

		    # Draw spectrum box using lines: right, bottom, left, top lines (in this order)
		    $target_canvas_ create line \
			[expr $spec_xmax + $tilt_dx1] $spec_ymax \
			[expr $spec_xmax - $tilt_dx1] $spec_ymin \
			-fill $footprintcolor -width 1 -tags "$tags $fg"
		    $target_canvas_ create line \
			[expr $spec_xmax - $tilt_dx1] $spec_ymin \
			[expr $spec_xmin - $tilt_dx1] $spec_ymin \
			-fill $footprintcolor -width 1 -tags "$tags $fg"
		    $target_canvas_ create line \
			[expr $spec_xmin - $tilt_dx1] $spec_ymin \
			[expr $spec_xmin + $tilt_dx1] $spec_ymax \
			-fill $footprintcolor -width 1 -tags "$tags $fg"
		    $target_canvas_ create line \
			[expr $spec_xmin + $tilt_dx1] $spec_ymax \
			[expr $spec_xmax + $tilt_dx1] $spec_ymax \
			-fill $footprintcolor -width 1 -tags "$tags $fg"
		# No tilt:
		} else {

		    # Shading
		    if {$cbo_shading == 1} {
			set stipple gray12
			set fill #999
			if {$prior == "0"} {
			    set stipple gray25
			    set fill $acq_color
			    set footprintcolor $acq_color
			}
			# Draw a hashed area to the left and to the right of the slit
			$target_canvas_ create rect $spec_xmin $spec_ymax $slit_xmin $spec_ymin \
			    -fill $fill -stipple $stipple -width 0 -tags "$tags $tags_shading"
			$target_canvas_ create rect $slit_xmax $spec_ymax $spec_xmax $spec_ymin \
			    -fill $fill -stipple $stipple -width 0 -tags "$tags $tags_shading"
		    }
		    # Draw spectrum box over it
		    $target_canvas_ create rect $spec_xmin $spec_ymax $spec_xmax $spec_ymin \
			-outline $footprintcolor -width 1 -tags "$tags $fg"
		}

                if {$waveccd != "" } {
		    # Draw red line indicating central wavelength position
		    $target_canvas_ create line \
			[expr { $waveccd + $tilt_dx1 }] $slit_ymax \
			[expr { $waveccd - $tilt_dx1 }] $slit_ymin \
			-fill red -width 2 -tags "$tags $fg"

		    # Draw green line from CWL to science object
		    $target_canvas_ create line \
			[expr $waveccd + $tilt_dcwl] $yobj \
			[expr $xccd    + $tilt_dcwl] $yobj \
			-fill green -width 1 -tags "$tags $fg2 "
		}

		# Draw the tilted slit; first the right edge, then the left edge
		$target_canvas_ create line \
		    [expr $xccd - $tilt_dx1 + $tilt_dx2] $slit_ymin \
		    [expr $xccd + $tilt_dx1 + $tilt_dx2] $slit_ymax \
		    -fill $fg2 -width 2 -tags "$tags $fg2 " 
		$target_canvas_ create line \
		    [expr $xccd - $tilt_dx1 - $tilt_dx2] $slit_ymin \
		    [expr $xccd + $tilt_dx1 - $tilt_dx2] $slit_ymax \
		    -fill $fg2 -width 2 -tags "$tags $fg2 "

	    } else {
		
		######################################
		# vertical dispersion (e.g. F2)
		######################################

		# Zero the offsets in case of no tilt
		set tilt_dy1  0.
		set tilt_dy2  0.
		set tilt_dcwl 0.

		set tilt_dy1 [expr $dimx * tan ($slittilt * $rad)]
		set tilt_dy2 [expr $dimy / cos ($slittilt * $rad)]
		set tilt_dcwl [expr $tilt_dy1 * $xshift / $dimx]

		# Determine if the spectrum is truncated by the detector edges
		# If 0% <= truncation <= 10%, draw it normally
		# If 10% < truncation <= 20%, draw it with a yellow
		# If truncation > 20%, draw it with a red outline
		set totSpecLength [expr abs($spec_ymax - $spec_ymin)]
		set truncFraction 0.
		# canvas coordinates might be "inverted", hence the size comparisons might fail
		# and thus I need to check whether the minimum value is actually the numeric minimum 
		# in canvas coordinates. if not, flip.
		set spec_ymin_cv $spec_ymin
		set spec_ymax_cv $spec_ymax
		if {$spec_ymin_cv > $spec_ymax_cv} {
		    set spec_ymin_cv $spec_ymax
		    set spec_ymax_cv $spec_ymin
		}
		set detspecmin_cv $detspecmin
		set detspecmax_cv $detspecmax
		if {$detspecmin_cv > $detspecmax_cv} {
		    set detspecmin_cv $detspecmax
		    set detspecmax_cv $detspecmin
		}
		if {$spec_ymin_cv < $detspecmin_cv} {
		    set truncFraction [expr abs($detspecmin_cv - $spec_ymin_cv) / $totSpecLength]
		}
		if {$spec_ymax_cv > $detspecmax_cv} {
		    set truncFraction [expr abs($spec_ymax_cv - $detspecmax_cv) / $totSpecLength]
		}

		if {$prior != "0"} {
		    if {$truncFraction <= 0.1} {
			set footprintcolor $fg
		    } elseif {$truncFraction > 0.1 && $truncFraction <= 0.20 && $prior != "0"} {
			set footprintcolor yellow
			set num_yellow [expr $num_yellow + 1]
		    } elseif {$truncFraction > 0.20 && $prior != "0"} {
			set footprintcolor red
			set num_red [expr $num_red + 1]
		    }
		}

		# Skip this object if not an acquisition object and only acq objects should be plotted
		if {$cbo_acqonly == 1 && $prior != "0"} {
		    continue
		}

		# If there is a tilt
		if { $slittilt != 0. } {
		    if {$cbo_shading == 1} {
			set ymin_shade1 [maxval [expr $spec_ymin - $tilt_dy1] [expr $spec_ymin + $tilt_dy1]]
			set ymax_shade1 [minval [expr $yccd - $tilt_dy1 - $tilt_dy2] [expr $yccd + $tilt_dy1 - $tilt_dy2]]
			set ymin_shade2 [maxval [expr $yccd - $tilt_dy1 + $tilt_dy2] [expr $yccd + $tilt_dy1 + $tilt_dy2]]
			set ymax_shade2 [minval [expr $spec_ymax - $tilt_dy1] [expr $spec_ymax + $tilt_dy1]]
			# Draw a hashed area to the left and to the right of the slit
			$target_canvas_ create rect $spec_xmin $ymax_shade1 $spec_xmax $ymin_shade1 \
			    -fill #999 -stipple gray12 -width 0 -tags "$tags $tags_shading"
			$target_canvas_ create rect $spec_xmin $ymax_shade2 $spec_xmax $ymin_shade2 \
			    -fill #999 -stipple gray12 -width 0 -tags "$tags $tags_shading"
		    }
		    # Draw spectrum box using lines: right, bottom, left, top lines (in this order)
		    $target_canvas_ create line \
			$spec_xmax [expr $spec_ymax - $tilt_dy1] \
			$spec_xmax [expr $spec_ymin - $tilt_dy1] \
			-fill $footprintcolor -width 1 -tags "$tags $fg"   		
		    $target_canvas_ create line \
			$spec_xmax [expr $spec_ymin - $tilt_dy1] \
			$spec_xmin [expr $spec_ymin + $tilt_dy1] \
			-fill $footprintcolor -width 1 -tags "$tags $fg"   		
		    $target_canvas_ create line \
			$spec_xmin [expr $spec_ymin + $tilt_dy1] \
			$spec_xmin [expr $spec_ymax + $tilt_dy1] \
			-fill $footprintcolor -width 1 -tags "$tags $fg"   		
		    $target_canvas_ create line \
			$spec_xmin [expr $spec_ymax + $tilt_dy1] \
			$spec_xmax [expr $spec_ymax - $tilt_dy1] \
			-fill $footprintcolor -width 1 -tags "$tags $fg" 			
		} else {
		    if {$cbo_shading == 1} {
			set stipple gray12
			set fill #999
			if {$prior == "0"} {
			    set stipple gray25
			    set fill $acq_color
			    set footprintcolor $acq_color
			}
			# Draw a hashed area above and below the slit
			$target_canvas_ create rect $spec_xmin $spec_ymax $spec_xmax $slit_ymax \
			    -fill $fill -stipple $stipple -width 1 -tags "$tags $tags_shading"
			$target_canvas_ create rect $spec_xmin $slit_ymin $spec_xmax $spec_ymin \
			    -fill $fill -stipple $stipple -width 1 -tags "$tags $tags_shading"
		    }
		    # Draw spectrum box over it
		    $target_canvas_ create rect $spec_xmin $spec_ymax $spec_xmax $spec_ymin \
			-outline $footprintcolor -width 1 -tags "$tags $fg"
		}

                if {$waveccd != "" } {
		    # Draw red line indicating central wavelength position
		    $target_canvas_ create line \
			$slit_xmin [expr { $waveccd + $tilt_dy1 }] \
			$slit_xmax [expr { $waveccd - $tilt_dy1 }] \
			-fill red -width 2 -tags "$tags $fg"
		    # Draw green line from CWL to science object
		    $target_canvas_ create line \
			$xobj [expr $waveccd  + $tilt_dcwl ] \
			$xobj [expr $yccd     + $tilt_dcwl ] \
			-fill green -width 1 -tags "$tags $fg2 "
		}
		
		# Draw the tilted slit; first the upper edge, then the lower edge
		$target_canvas_ create line \
		    $slit_xmax [expr $yccd - $tilt_dy1 + $tilt_dy2] \
		    $slit_xmin [expr $yccd + $tilt_dy1 + $tilt_dy2] \
		    -fill $fg2 -width 2 -tags "$tags $fg2 "
		$target_canvas_ create line \
		    $slit_xmax [expr $yccd - $tilt_dy1 - $tilt_dy2] \
		    $slit_xmin [expr $yccd + $tilt_dy1 - $tilt_dy2] \
		    -fill $fg2 -width 2 -tags "$tags $fg2 "
	    }
	}
	
	# Is this a N&S mask?
	set NSmode ""
	set bandFile ""
	if {$instType == "GMOS-N" || $instType == "GMOS-S"} {
	    # get the shuffling mode from the ODF
      	    catch {set bandFile [exec grep "#fits BDFILE" $itk_option(-catalog) | cut -f2 -d= ]}
	    set bandFile [regsub -all "\[' \t]+" $bandFile {}]
	    set path [file dirname $itk_option(-catalog) ]
	    if {[file exists $path/$bandFile] && $bandFile != ""} {
		set NSmode [exec grep "shuffleMode" $bandFile | cut -f2 -d: ]
		set NSmode [string trim $NSmode]
		if {$NSmode == "microShuffle"} {
		    set NSmode " (micro shuffling)"
		}
		if {$NSmode == "bandShuffle"} {
		    set NSmode " (band shuffling)"
		}
	    }
	}

	$w_.odfInfoFrame.03 delete 1.0 end
	$w_.odfInfoFrame.03 insert end "Instrument: "
	$w_.odfInfoFrame.03 insert end "${instType} ${NSmode}\n" "instrument"
	$w_.odfInfoFrame.03 tag configure instrument -foreground #069 -font "helvetica 12 bold"
	set evaluation ""
	if {$num_acq <= 1 } {
	    set acq_color #c00
	    set evaluation "ERROR: "
	} elseif {$num_acq == 2 } {
	    set acq_color #c00
	    set evaluation "WARNING: "
	} else {
	    set acq_color black
	}
	$w_.odfInfoFrame.03 insert end "${evaluation}Number of good acq. stars: $num_acq\n" "acquisition"
	$w_.odfInfoFrame.03 tag configure acquisition -foreground $acq_color
	if { $proximity_alert_print != ""} {
	    $w_.odfInfoFrame.03 insert end "$proximity_alert_print" "proximity_alert"
	    $w_.odfInfoFrame.03 tag configure proximity_alert -foreground #c00
	}

	# if not in acq highlighting mode, print some extra information about science slitlets
	if {$cbo_acqonly == "0"} {
	    if {$num_yellow > 0} {
		$w_.odfInfoFrame.03 insert end "$num_yellow spectra truncated >10% (yellow)\n"
	    }
	    if {$num_red > 0} {
		$w_.odfInfoFrame.03 insert end "$num_red spectra truncated >20% (red)\n"
	    }
	}

	return
    }


    #############################################################
    #  Name: minval
    #
    #  Description:
    #  Returns the smaller of two values
    #############################################################
    #############################################################

    public method minval {v1 v2} {
	if {$v1<$v2} {
	    return $v1
	} else {
	    return $v2
	}
    }


    #############################################################
    #  Name: maxval
    #
    #  Description:
    #  Returns the smaller of two values
    #############################################################
    #############################################################

    public method maxval {v1 v2} {
	if {$v1>$v2} {
	    return $v1
	} else {
	    return $v2
	}
    }

    #############################################################
    #  Name: slits_OT
    #
    #  Description:
    #   This function draws the slits for OT files.
    #############################################################
    #############################################################

    public method slits_OT {} {
	global ::cbo_ot_slits

	# Clear slits if requested and return
	if {$cbo_ot_slits == 0} {
	    clearslits_OT
	    return
	}

	# Leave if the pixelscale is unknown
	set pixelScale $itk_option(-pixs)
	if { $pixelScale == 0.0 || $pixelScale == "NOT FOUND" } { 
	    error_dialog "Error: Pixel scale undetermined!"
	    puts "ERROR: Pixel scale undetermined!"
	    return
	}

	# OK, Continue plotting slits
	set tags slitMarkOT
	set target_canvas_ .skycat1.image.imagef.canvas
	set instType $itk_option(-instType)
	set dispDirection [getDispDirection]
	# Pi/180
	set rad 0.01745329252

	# This is the user INPUT catalog, columns may be in ANY order!
	# the following variables represent the column number, starting with 0, where the respective entry is found
	set head $itk_option(-headingx)
	set PRIORITY  [lsearch -regex $head {(?i)priority}]
	set TYPE      [lsearch -regex $head {(?i)slittype}]
	set ID        [lsearch -regex $head {(?i)ID}]
	set OBJX      [lsearch -regex $head {(?i)x_ccd}]
	set OBJY      [lsearch -regex $head {(?i)y_ccd}]
	set SLITSIZEX [lsearch -regex $head {(?i)slitsize_x}]
	set SLITSIZEY [lsearch -regex $head {(?i)slitsize_y}]
	set SLITTILT  [lsearch -regex $head {(?i)slittilt}]
	set SLITPOSX  [lsearch -regex $head {(?i)slitpos_x}]
	set SLITPOSY  [lsearch -regex $head {(?i)slitpos_y}]

	#  Cycle through each item in the objects list
	#  This gets num rows from total_rows(vmTableList)
	#  and info_ connections all rows in the display.
	set nobj [$results_ total_rows]
	if {$nobj == 0} {
	    return
	}

	for {set n 0} {$n < $nobj} {incr n} {
	    set row [lindex $info_ $n]

	    # cartesian object coordinates 
	    set xobj [lindex $row $OBJX]
	    set yobj [lindex $row $OBJY]
	    # cartesian slit position coordinates (same as object coordinates)
	    set xccd [expr [lindex $row $OBJX] + ([lindex $row $SLITPOSX]/$itk_option(-pixs)) ]
	    set yccd [expr [lindex $row $OBJY] + ([lindex $row $SLITPOSY]/$itk_option(-pixs)) ]
	    # slit offsets
	    set xshift [expr [lindex $row $SLITPOSX ]/$itk_option(-pixs)]
	    set yshift [expr [lindex $row $SLITPOSY ]/$itk_option(-pixs)]
	    # slit dimensions
	    set dimx [expr [lindex $row $SLITSIZEX] / 2.0 / $itk_option(-pixs)]
	    set dimy [expr [lindex $row $SLITSIZEY] / 2.0 / $itk_option(-pixs)]
	    set prior [lindex $row $PRIORITY]
	    set slittilt [lindex $row $SLITTILT]
	    set tmp 0
            
	    #  Set color of slit and spectrum drawn based on priority,
	    #  color Acq. objects different color.
	    if { $prior == "0" } {
		set fg cyan 
	    } else {
		set fg #ff6
	    }

	    #debugStatement "slits: Object position before: $x, $y"
	    $target_image_ convert coords $xobj $yobj image xobj yobj canvas
	    $target_image_ convert coords $xccd $yccd image xccd yccd canvas
	    $target_image_ convert dist $dimx $dimy image dimx dimy canvas

	    set slit_xmin [expr $xccd - $dimx]
	    set slit_xmax [expr $xccd + $dimx]
	    set slit_ymin [expr $yccd - $dimy]
	    set slit_ymax [expr $yccd + $dimy]

	    if {$dispDirection == "horizontal"} {

		######################################
		# horizontal dispersion (e.g. GMOS)
		######################################
		
		# For plotting tilted slits we need to shift vertices. Introducing three helper variables:
		# tilt_dx1 (offset for tilting a 1D slit)
		# tilt_dx2 (additional offset to compensate for narrowing effect when tilting 2D slit)
		# tilt_dcwl (offset by which we have to shift the "green line"; a simple fraction of tilt_dx1)

		# Zero the offsets in case of no tilt
		set tilt_dx1  0.
		set tilt_dx2  0.
		set tilt_dcwl 0.

		set tilt_dx1 [expr $dimy * tan ($slittilt * $rad)]
		set tilt_dx2 [expr $dimx / cos ($slittilt * $rad)]
		set tilt_dcwl [expr $tilt_dx1 * $yshift / $dimy]

		# Draw the tilted slit; right left top bottom
		$target_canvas_ create line \
		    [expr $xccd - $tilt_dx1 + $tilt_dx2] $slit_ymin \
		    [expr $xccd + $tilt_dx1 + $tilt_dx2] $slit_ymax \
		    -fill $fg -width 2 -tags "$tags $fg " 
		$target_canvas_ create line \
		    [expr $xccd - $tilt_dx1 - $tilt_dx2] $slit_ymin \
		    [expr $xccd + $tilt_dx1 - $tilt_dx2] $slit_ymax \
		    -fill $fg -width 2 -tags "$tags $fg "
		$target_canvas_ create line \
		     [expr $xccd + $tilt_dx1 - $tilt_dx2] $slit_ymax \
		     [expr $xccd + $tilt_dx1 + $tilt_dx2] $slit_ymax \
		    -fill $fg -width 2 -tags "$tags $fg " 
		$target_canvas_ create line \
		     [expr $xccd - $tilt_dx1 - $tilt_dx2] $slit_ymin \
		     [expr $xccd - $tilt_dx1 + $tilt_dx2] $slit_ymin \
		    -fill $fg -width 2 -tags "$tags $fg " 
	    } else {
		
		######################################
		# vertical dispersion (e.g. F2)
		######################################

		# Zero the offsets in case of no tilt
		set tilt_dy1  0.
		set tilt_dy2  0.
		set tilt_dcwl 0.

		set tilt_dy1 [expr $dimx * tan ($slittilt * $rad)]
		set tilt_dy2 [expr $dimy / cos ($slittilt * $rad)]
		set tilt_dcwl [expr $tilt_dy1 * $xshift / $dimx]

		# Draw the tilted slit; upper lower left right
		$target_canvas_ create line \
		    $slit_xmax [expr $yccd - $tilt_dy1 + $tilt_dy2] \
		    $slit_xmin [expr $yccd + $tilt_dy1 + $tilt_dy2] \
		    -fill $fg -width 2 -tags "$tags $fg "
		$target_canvas_ create line \
		    $slit_xmax [expr $yccd - $tilt_dy1 - $tilt_dy2] \
		    $slit_xmin [expr $yccd + $tilt_dy1 - $tilt_dy2] \
		    -fill $fg -width 2 -tags "$tags $fg "
		$target_canvas_ create line \
		    $slit_xmin [expr $yccd + $tilt_dy1 - $tilt_dy2] \
		    $slit_xmin [expr $yccd + $tilt_dy1 + $tilt_dy2] \
		    -fill $fg -width 2 -tags "$tags $fg "
		$target_canvas_ create line \
		    $slit_xmax [expr $yccd - $tilt_dy1 - $tilt_dy2] \
		    $slit_xmax [expr $yccd - $tilt_dy1 + $tilt_dy2] \
		    -fill $fg -width 2 -tags "$tags $fg "
	    }
	}
	
	return
    }


    #############################################################
    #  Name: clearslits
    #
    #  Description:
    #   Clear the slits
    #############################################################
    #############################################################

    public method clearslits {} {
	global ::cbo_slits ::cbo_shading
        set target_canvas_ .skycat1.image.imagef.canvas
        $target_canvas_ delete slitMark
        $target_canvas_ delete shading
#	set cbo_slits 0
        return
    }

    #############################################################
    #  Name: clearslits
    #
    #  Description:
    #   Clear the slits
    #############################################################
    #############################################################

    public method clearslits_OT {} {
	global ::cbo_ot_slits
        set target_canvas_ .skycat1.image.imagef.canvas
        $target_canvas_ delete slitMarkOT
#	set cbo_ot_slits 0
        return
    }


    #########################################################################
    #  Name: _sel
    #
    #  Modifies a selection of slitlets
    #  Description: 
    #  The "del" argument is UNUSED
    #########################################################################
    #########################################################################
    public method _sel {flag {del 0}} {
	#  Get the results from the display (might be multiple rows)
	set rows [$results_ get_selected ]
	
	#puts "sel entry: rows is:\n$rows"
	
	# Save selection.
	$results_ save_yview
	$results_ save_selection		
	
	set head $itk_option(-headingx)
	set PRIORITY  [lsearch -regex $head {(?i)priority}]
	set TYPE      [lsearch -regex $head {(?i)slittype}]
	set ID        [lsearch -regex $head {(?i)ID}]
	set SLITSIZEX [lsearch -regex $head {(?i)slitsize_x}]
	set SLITSIZEY [lsearch -regex $head {(?i)slitsize_y}]
	set SLITTILT  [lsearch -regex $head {(?i)slittilt}]
	set SLITPOSX  [lsearch -regex $head {(?i)slitpos_x}]
	set SLITPOSY  [lsearch -regex $head {(?i)slitpos_y}]
	
	set originalRowIds [list ]
	foreach row $rows {
	    set originalRowIds [lappend originalRowIds [lindex $row $ID ]]
	}
	
	set catname $itk_option(-catalog)
	set cattype $itk_option(-catalogtype)
	
	# Make sure the list of rows is in increasing order by ID.
	#if { ![is_sorted $rows ]} {	
	#	set rows [sort_rowlist $rows ]
	#}
	
	#  Check that the flag value pasted in is valid.
	# Seriously? Y? Never seen that being used... if slittype then that would be R(ectangle)
	# but we don't offer to change the slit type
#	set flagCheck [regexp {Ssx|Ssy|Spx|Spy|St|Sall|0|1|2|3|X|Y} $flag ]
	set flagCheck [regexp {Sall|0|1|2|3|X|Y} $flag ]
	
	if { $flagCheck == 0 } {
	    error_dialog "Invalid priority or slittype <$flag>."
	    return 1
	}
	
	# The current row index in rows that we are replacing.
	set allRows [$results_ get_contents ]
	
	# construct list of columns that will be changed. 
#	if {$flag == "Ssx"} {
#	    set cols [list $SLITSIZEX ]
#	} elseif {$flag == "Ssy"} {
#	    set cols [list $SLITSIZEY ]
#	} elseif {$flag == "Spx"} {
#	    set cols [list $SLITPOSX ]
#	} elseif {$flag == "Spy"} {
#	    set cols [list $SLITPOSY ]
#	} elseif {$flag == "St"} {
#	    set cols [list $SLITTILT ]
#	}
 
	if {$flag == "Sall"}  {
	    set cols [list $SLITSIZEX $SLITSIZEY $SLITPOSX $SLITPOSY $SLITTILT ]
	} else {
	    set cols [list $PRIORITY ]
	}
	
	# Send start signal to undo object.
	
	# Here we are hoping that the rows are sorted in order by ID, 
	# if they are not then we will need several iterations of the while
	# loop... but, this is still going to be more efficient than before.
	while { [llength $rows ] > 0 } {
	    
	    #puts "while start rows is:\n$rows"
	    
	    # Clear the catalog widget. 
	    $results_ clear
	    
	    # Set found flag to true for init.
	    set found 1
	    
	    # Scan through all rows			
	    foreach catRow $allRows {
		
		if { $found == 1 && [llength $rows ] > 0 } {
		    set row [lindex $rows 0 ]
		    
		    # Remove first item from the list.
		    set rows [lrange $rows 1 end ]
		    
		    # found flag off.
		    set found 0
		    #puts "searching for item: $row"
		    #puts "rows is:\n$rows"
		}
		
		if { [llength $rows ] >= 0 && [lindex $row $ID ] == [lindex $catRow $ID ] } { 
		    # ID match, found a row to edit...
		    
		    #puts "row [lindex $row $ID ] found."
		    
		    set found 1
		    set oldrow $row
		    set newrow $row
		    set problemsStr ""
		    set prio [lindex $row $PRIORITY]
		    
		    # Confirm when editing acq objects.
		    if {$prio == "0"} {
			set answer [tk_messageBox -message "Object [lindex $row $ID ] is an acquisition object.\n\nAre you sure you want to edit this object?" -title "WARNING" -parent $w_ -type yesno ]
			
			if {$answer == "no"} {
			    tk_messageBox -message "Edit canceled for object [lindex $row $ID ]." -parent $w_
			    $results_ append_row $catRow
			    continue
			}
		    }
		    
		    if { $flag == "A" || $flag == "R" } {
			set newrow [lreplace $row $TYPE $TYPE $flag]
		    } elseif { [string index $flag 0] == "S" } {
			# I THINK that SLITSIZEX/Y etc will never be -1 because I force added it earlier
			# Leave it in for the time being
			switch -regexp $flag {
			    Sall {
				set newrow $row
				set tsx  [$w_.slitsize_x get]
				set tsy  [$w_.slitsize_y get]
				set tt   [$w_.slittilt get]
				set tspx [$w_.slitpos_x get]
				set tspy [$w_.slitpos_y get]
				if {$tsx != "" && $SLITSIZEX != -1} {
				    if {$prio != 0} {
					set newrow [lreplace $newrow $SLITSIZEX $SLITSIZEX [format "%.6f" $tsx]]
				    } else {
					set newrow [lreplace $newrow $SLITSIZEX $SLITSIZEX 2.0]
				    }
				}
				if {$tsy != "" && $SLITSIZEY != -1} {
				    if {$prio != 0} {
					set newrow [lreplace $newrow $SLITSIZEY $SLITSIZEY [format "%.6f" $tsy]]
				    } else {
					set newrow [lreplace $newrow $SLITSIZEY $SLITSIZEY 2.0]
				    }
				}
				if {$tt != "" && $SLITTILT != -1} {
				    # Make sure slit tilts are valid.
				    if {($tt > 45.0 && !($tt <= 360.0 && $tt >= 315.0) || $tt < -45.0)} {
					append problemsStr "Slit Tilt $tt is outside valid range (-45.0 to 45 degrees).\n"
				    } else {
					if {$prio != 0} {
					    set newrow [lreplace $newrow $SLITTILT $SLITTILT [format "%.6f" $tt]]
					} else {
					    set newrow [lreplace $newrow $SLITTILT $SLITTILT 0.0]
					}
				    }
				}
				if {$tspx != "" && $SLITPOSX != -1} {
				    set newrow [lreplace $newrow $SLITPOSX $SLITPOSX [format "%.6f" $tspx]]
				}
				if {$tspy != "" && $SLITPOSY != -1} {
				    set newrow [lreplace $newrow $SLITPOSY $SLITPOSY [format "%.6f" $tspy]]
				}
			    }
			}
		    } elseif {$flag == "X"} { 
			# If we are ignoring an object be careful not to lose priority info 
			# for that object.
			set newrow [lreplace $row $PRIORITY $PRIORITY "X"]
		    } else {
			# Change priority. Here the priority will be changed to one of 0, 1, 2, 3 from any
			# other priority. Flag can have any of the aformentioned values. 
			if {$SLITTILT != -1} { 
			    set tt [lindex $row $SLITTILT ]
			    if {($tt > 45.0 && !($tt <= 360.0 && $tt >= 315.0) || $tt < -45.0)} {
				append problemsStr "Cannot restore slit. Slittilt $tt is outside valid range (-45.0 to 45 degrees).\n"	
			    }
			}
			
			set newrow [lreplace $row $PRIORITY $PRIORITY $flag]
			
			# Force 2x2 arcsec square box and zero tilt for acquisition objects
			if {$flag == 0} {
			    set newrow [lreplace $newrow $SLITSIZEX $SLITSIZEX 2.0]
			    set newrow [lreplace $newrow $SLITSIZEY $SLITSIZEY 2.0]
			    set newrow [lreplace $newrow $SLITTILT $SLITTILT 0.0]
			}
		    }
		    
		    # If there were problems add the old unmodified row. Else add new row. 
		    if {$problemsStr != ""} {
			error_dialog $problemsStr
			$results_ append_row $catRow
		    } else {		    
			$results_ append_row $newrow
		    }
		} else {
		    # This row is not one we wish to edit at this time. 
		    # So add it back to the catalog unmodified.
		    $results_ append_row $catRow
		}
	    }
	}
	
	# Finished input signal to catalog display widget. 
	$results_ new_info
	
	$results_ save_to_file $itk_option(-catalog) [$this.results get_contents ] "" 1
	
	# Restore the selection.
	reload_catalog OT
	$results_ restore_yview
	$results_ restore_selection
	
	# Check to see if row order or contents has changed. If so warn user.
	set post_rows [$results_ get_selected ]
	set cur 0
	foreach row $post_rows {
	    if {[lindex $row $ID ] != [lindex $originalRowIds $cur ]} {
		#tk_messageBox -message "Warning: Table order changed.\nSelected rows may be different."
		tk_messageBox -message "Warning: Table row order changed.\nHighlighted rows may not reflect previous selection." -parent $w_ -title "Row Order Changed"
		break
	    }
	    incr cur
	}
	
	return 0
    }


    #############################################################
    #  Name: define_bands
    #
    #  Description:
    #   opens the "define bands" interface 
    # 
    #  Author: cba
    #############################################################
    #############################################################
    
    public method define_bands {args} {
	global ::cbo_bands

	if {$args == "drawBandsOnly" && $cbo_bands == 0} {
	    # Clear band objects.
	    .skycat1.image.imagef.canvas delete bandrects
	    return 
	} elseif {$args == "drawBandsOnly"} {
	    set bandPlotFail [::gmmps::band_def_UI::drawBands $w_ $itk_option(-catalog)]
	    # the prevuous function returns 1 in case of failure (and error messages itself)
	    return
	}
	
	promptBandFileSelect
	
	if {$itk_option(-bdfile) == ""} {
	    debugStatement "No band file selected, cannot enter N&S."
	    return
	}
	
	set instType $itk_option(-instType)
	
	#  Determine if slitpos_x and slitpos_y exist in the
	#  catalog files.
	set addPX ""
	set addPY ""
	
	catch {set addPX [exec grep slitpos_x $itk_option(-catalog) ]}
	catch {set addPY [exec grep slitpos_y $itk_option(-catalog) ]}
	if { $addPX == "" } {
	    set addPX "yes" 
	} else {
	    set addPX "no" 
	}
	
	if { $addPY == "" } {
	    set addPY "yes" 
	} else {
	    set addPY "no" 
	}
	
	# Make sure there are no invalid slit tilts. 
	if {[check_tilts 0.0 0.0 ]} {
	    return 
	}
	
	# Destroy any open band_def_ui and spoc window.
	catch {destroy $bw_}
	catch {destroy $spoc_}
	
	update idletasks

	# Find detector info.		
	setDetInfo $instType

	# Find Ra/Dec
	setRaDecInfo

	set dispDirection [getDispDirection]
	if {$dispDirection == "horizontal"} {
	    set slitWidth $itk_option(-xsize)
	} else {
	    set slitWidth $itk_option(-ysize)
	}
	
	#  Call the gmmps_spoc.tcl init function
	set CRPIX1 [get_crpix_image CRPIX1]
	set CRPIX2 [get_crpix_image CRPIX2]
	set bw_ [utilReUseWidget gmmps::band_def_UI .skycat1.bandDefUI $this $itk_option(-catalog) \
		     $slitWidth $itk_option(-tilt) $addPX $addPY $itk_option(-pixs) $itk_option(-bw) \
		     $this.results $config_file_ $DET_IMG_ $DET_SPEC_ $RA_IMG_ $DEC_IMG_ $CRPIX1 $CRPIX2 \
		     $dispDirection $itk_option(-instType) $itk_option(-bdfile)]
	# if {[catch {set bw_ [utilReUseWidget gmmps::band_def_UI .skycat1.bandDefUI $this $itk_option(-catalog) \
	#			 $slitWidth $itk_option(-tilt) $addPX $addPY $itk_option(-pixs) $itk_option(-bw) \
	#			 $this.results $config_file_ $DET_IMG_ $DET_SPEC_ $RA_IMG_ $DEC_IMG_ $CRPIX1 $CRPIX2 \
	#			 $itk_option(-instType)] $itk_option(-bdfile)} msg]} {
	#	    debugStatement "Failed to spoc here"
	#	    if { $args == "drawBandsOnly" } { 
	#		error_dialog "Error Drawing Bands... Band Def UI must be closed as it is configured and used for this operation."
	#	    } else {
	#		error_dialog "Error Opening Band Def UI\nIs the UI already up? $msg"
	#	    }
	return 
    }
    return
}


#########################################################################
#  Name: promptBandFileSelect
#
#  Description: Creates a dialog window prompting the user to select or 
# 	create a band definition file for use with any created ODF.
#########################################################################
#########################################################################

protected method promptBandFileSelect {} {
    # Search for bandfiles associated with our window.
    # (needs to be global for use with listbox widget)
    global foundBandFiles
    set foundBandFiles [findBandFiles [file dirname $itk_option(-catalog)]]
    set foundBandFiles [linsert $foundBandFiles 0 "tmp.bands" ]
    
    # find max length so we can configure the listbox correctly.
    # We want a minimum of 30 chars such that it looks nice
    set maxNameLen 30
    foreach foundFile $foundBandFiles {
	if {[string length $foundFile ] > $maxNameLen} {
	    set maxNameLen [string length $foundFile ]
	}
    }
    incr maxNameLen

    # Create dialog window.
    set pbf [toplevel $w_.pickbf ]
    wm title $w_.pickbf "Pick .bands File"
    
    # Associate pressing the 'x' with choosing tmp.bands.
    wm protocol $pbf WM_DELETE_WINDOW [code $this chooseBandFile $pbf "cancel"]
    
    pack \
	[label $pbf.bandfileprompt -anchor w -fg #55f\
	     -text "Select or create new band definition file (BDF)."] \
	[label $pbf.explainTextBands1 -anchor w \
	     -text "\nThe BDF chosen will be associated with any"] \
	[label $pbf.explainTextBands2 -anchor w \
	     -text "ODF(s) created during this N&S design session."] \
	[label $pbf.explainTextBands3 -anchor w \
	     -text "Choose 'tmp.bands' if you do not wish to have"] \
	[label $pbf.explainTextBands4 -anchor w \
	     -text "a dedicated BDF."] \
	-fill x -side top -expand 1 -padx 5 -ipadx 5

    pack \
	[frame $pbf.oldFiles ] \
	-fill x -side top -expand 1 -padx 5 -pady 5 -ipadx 5 -ipady 5

    pack \
	[frame $pbf.bandButtons1 ] \
	[frame $pbf.bandButtons2 ] \
	-fill x -side top -expand 1 -padx 10 -ipadx 10

    pack \
	[button $pbf.newFile -text "Create new BDF" \
	     -command [code $this chooseBandFile $pbf "new"]] \
	[button $pbf.selectOld -text "Use Selected" \
	     -command [code $this chooseBandFile $pbf "selected"]] \
	-side right -padx 10 -ipadx 10 -in $pbf.bandButtons1

    pack \
	[button $pbf.cancel -text "Cancel" \
	     -command [code $this chooseBandFile $pbf "cancel"]] \
	-side right -padx 10 -ipadx 10 -pady 10 -in $pbf.bandButtons2
    
    pack \
	[listbox $pbf.selectBox \
	     -listvariable foundBandFiles \
	     -selectmode single \
	     -width $maxNameLen \
	     -bg white] \
	-side left -in $pbf.oldFiles -padx 5 -pady 5 -ipadx 5 -ipady 5

    $pbf.selectBox selection set 0
    
    tkwait window $pbf
}


#########################################################################
#  Name: chooseBandFile
#
#  Description: Creates a dialog window prompting the user to select or 
# 	create a band definition file for use with any created ODF.
#########################################################################
#########################################################################

protected method chooseBandFile {dialog_window mode} {
    
    if {$mode == "cancel"} {
	
	set itk_option(-bdfile) ""
	
	destroy $dialog_window
	
    } elseif {$mode == "new"} {
	set bandType [list [list ".bands Band Definition File" {".bands" ".BANDS"}]]
	
	# Search for appropriate initfile name.
	set cnt 1
	set initFile [format "%sbanddef%d.bands" [file rootname $itk_option(-catalog) ] $cnt]
	while {[file exists $initFile ]} {
	    incr cnt
	    set initFile [format "%sbanddef%d.bands" [file rootname $itk_option(-catalog) ] $cnt]
	}
	set initFile [file tail $initFile ]
	
	# Open "new file" dialog.
	set newBandFile [tk_getSaveFile \
			     -defaultextension ".bands" \
			     -filetypes $bandType \
			     -title "New Band Definition" \
			     -initialdir [file dirname $itk_option(-catalog) ]\
			     -initialfile $initFile \
			     -parent $w_ ]
	
	# If a name was choosen, create that file.
	if {$newBandFile != ""} {
	    
	    set bdfile [open $newBandFile "w" ]
	    puts $bdfile "OT:$itk_option(-catalog)"
	    puts $bdfile "shuffleMode:microShuffle"
	    puts $bdfile "shuffleAmt:5.0"
	    puts $bdfile "slitlen:5.0"				
	    ::close $bdfile
	    
	    # Set bdfile.
	    set itk_option(-bdfile) $newBandFile
	    
	    gmmps_config_ [list "shuffle_mode" "slitlen_ms" "shuffleamt_ms" ] [list "microShuffle" "5.0" "5.0" ]
	    
	    destroy $dialog_window
	}		

    } elseif {$mode == "selected"} {

	# Query listbox for selected bdfile name.
	set cur [$dialog_window.selectBox curselection ]

	if {$cur == ""} {
	    error_dialog "Please select a band definition file before pressing this button."
	    return
	}
	
	set bandFileName [$dialog_window.selectBox get $cur ]
	
	set bandFileLongName [format %s/%s [file dirname $itk_option(-catalog)] $bandFileName ]
	
	# If in "tmp.bands" mode and "tmp.bands" doesn't exist, create it. 
	if {$bandFileName == "tmp.bands"} {
	    makeTmpBands
	    
	}
	
	set bandFileName $bandFileLongName
	
	set shufMode ""
	set slitlen_ms ""
	set shuffleAmt_ms ""
	set bandSize_bs ""
	set shufflePx_bs ""
	set yOffset_bs ""
	
	# Parse file data.
	set bandFile [open $bandFileName "r" ]
	while {[gets $bandFile line] >= 0} {
	    set line [split $line ":" ]

	    if {[lindex $line 0 ] == "shuffleMode"} {
		set shufMode [string trim [lindex $line 1 ]]
		#debugStatement "shuffleMode : $shufMode loaded from file."
	    } elseif {[lindex $line 0 ] == "shuffleAmt"} {
		set shuffleAmt_ms [string trim [lindex $line 1 ]]
		#debugStatement "sA_ms $shuffleAmt_ms"
	    } elseif {[lindex $line 0 ] == "yOffset"} {
		set yOffset_bs [string trim [lindex $line 1 ]]
		#debugStatement "yOffset $yOffset_bs"
	    } elseif {[lindex $line 0 ] == "shufflePx"} {
		set shufflePx_bs [string trim [lindex $line 1 ]]
		#debugStatement "shufflePx_bs $shufflePx_bs"
	    } elseif {[lindex $line 0 ] == "slitlen"} {
		set slitlen_ms [string trim [lindex $line 1 ]]
		set bandSize_bs $slitlen_ms
		#debugStatement "slitlen_ms $slitlen_ms"
		#debugStatement "bandSize_bs $slitlen_ms"
	    }
	}			
	::close $bandFile
	
	set itk_option(-bdfile) $bandFileName
	
	# Load the data into the .gmmps.cfg file, so that it will be loaded
	# into the N&S GUI when it is created.
	if {$shufMode == "bandShuffle"} {
	    debugStatement "Sending $shufMode $bandSize_bs $shufflePx_bs $yOffset_bs to config file"
	    gmmps_config_ [list "shuffle_mode" "bandsize_bs" "shufflepx_bs" "yoffset_bs" ] [list $shufMode $bandSize_bs $shufflePx_bs $yOffset_bs ]
	} else { 
	    debugStatement "Sending $shufMode $slitlen_ms $shuffleAmt_ms to config file."
	    gmmps_config_ [list "shuffle_mode" "slitlen_ms" "shuffleamt_ms" ] [list $shufMode $slitlen_ms $shuffleAmt_ms ]
	}
	
	destroy $dialog_window
	
	# Tell the N&S GUI which mode we are in.
	global shuffleMode 
	set shuffleMode $shufMode
    }
}


#########################################################################
#  Name: makeTmpBands
#
#  Description: makes a fresh bands.tmp file, if one doesn't already exist.
#########################################################################
#########################################################################	

protected method makeTmpBands {args} {

    # If in "tmp.bands" mode and "tmp.bands" doesn't exist, create it. 
    set tmpName [format %s/tmp.bands [file dirname $itk_option(-catalog) ]]
    if {![file exists $tmpName ]} {
	
	set mkTmp [open $tmpName "w" ]
	puts $mkTmp "shuffleMode:microShuffle"
	puts $mkTmp "slitlen:5.000"
	puts $mkTmp "shuffleAmt:5.000"
	puts $mkTmp "shufflePx:69"
	::close $mkTmp
    }
    
    set itk_option(-bdfile) $tmpName
    
    if {$args == "closeafter"} {
	catch {destroy $w_.pickbf}
    }
}


#########################################################################
#  Name: odfBandSearch
#
#  Description: Finds an .bands file associated with the given odf.
#########################################################################
#########################################################################	

#protected proc odfBandSearch {odfFile} {
#    set dir [file dirname $odfFile ]
#    set allBandFiles ""
#    catch {set allBandFiles [lsort [glob -directory $dir *.bands ]]}	
#    
#    set foundBFs [list ]
#    
#    foreach bandFileName $allBandFiles {
#	# tmp.bands will be inserted into the list later.
#	if {[file tail $bandFileName ] == "tmp.bands"} {
#	    continue
#	}
#	
#	set bandFile [open $bandFileName "r" ]
#	
#	while {[gets $bandFile line ] >= 0} {
#	    if {$line == "ODF:$odfFile"} {
#		set  [split $bandOdf ":" ]
#		set foundBFs [lappend foundBFs [file tail $bandFileName ]]
#	    }
#	}
#	::close $bandFile
#    }
#    
#    
#    if {[llength $foundBFs ] > 1} {
#	# If more than one associated band file was found prompt user to choose
#	set maxNameLen 0
#	foreach bandName $foundBFs {
#	    if {[string length $bandName ] > $maxNameLen} {
#		set maxNameLen $[string length $bandName ]
#	    }
#	}
#	
#	# Create dialog window.
#	set pbf [toplevel $w_.pickbf2 ]
#	wm title $w_.pickbf "Pick .bands File"
#	
#	# Associate pressing the 'x' with choosing tmp.bands.
#	wm protocol $pbf WM_DELETE_WINDOW [code $this makeTmpBands "closeafter"]
#	
#	pack \
#	    [label $pbf.bandfileprompt \
#		 -text "More than one bandfile found. Please select a bandfile to display." ] \
#	    [frame $pbf.oldFiles ] \
#	    [button $pbf.selectOld \
#		 -text "Select" \
#		 -command {set chosenBF [$pbf.selectBox curselection ]} ]
#	
#	pack \
#	    [label $pbf.prompt -text "Select file:" ]\
#	    [listbox $pbf.selectBox \
#		 -listvariable foundBFs \
#		 -selectmode single \
#		 -width $maxNameLen \
#		 -bg #ffc] \
#	    -side left \
#	    -in $pbf.oldFiles
#	
#	$pbf.selectBox selection set 0
#	
#	tkwait variable chosenBF $pbf2			
#	
#	global chosenBF
#	
#	#debugStatement "chosenBF: $chosenBF"
#	
#	#debugStatement [format "returning: %s/%s" [file dirname $odfFile ] [pbf.selectBox get $chosenBF ]]
#	
#	return [format "%s/%s" [file dirname $odfFile ] [pbf.selectBox get $chosenBF ]]
#	
#    } elseif {[llength $foundBFs ] == 1} {
#	return [lindex $foundBFs 0 ]
#	
#   } else {
#	
#	return ""
#    }
#}


#########################################################################
#  Name: findBandFiles
#
#  Description: Searches a directory for .bands files associated with the 
#	current OT file that created the N&S window.
#########################################################################
#########################################################################

protected method findBandFiles {dir} {
    set foundFiles [list ]
    set allBandFiles ""
    catch {set allBandFiles [glob -directory $dir *.bands ]}
    
    foreach bandFileName $allBandFiles {
	
	# tmp.bands will be inserted into the list later.
	if {[file tail $bandFileName ] == "tmp.bands"} {
	    continue
	}
	
	set associatedCat ""
	catch {set associatedCat [string trim [exec grep "OT:" $bandFileName | cut -f2 -d: ]]}
	
	#debugStatement "findBandFiles checked: $bandFileName\n.$associatedCat"
	
	# Match! 
	if {$associatedCat == $itk_option(-catalog)} {
	    set foundFiles [lappend foundFiles [file tail $bandFileName ]]
	}
    }
    
    return $foundFiles
}

#########################################################################
#  Name: _sel
#
#  Modifies a selection of slitlets
#  Description: I need this because the "Update slits" button would 
#  not update the slit drawings in the OT window.
#########################################################################
#########################################################################
public method _sel_parent {flag {del 0}} {
    global ::cbo_ot_slits
    _sel $flag $del
    set cbo_ot_slits 0
    clearslits_OT
    # the following does not work (the catalog known to slits_OT is empty)
    #    set cbo_ot_slits 1
    #    slits_OT
}



#############################################################
#  Name: open_catalog
#
#  Description:
#   This function opens the catalog for this window.
#   Called by layoutdialog(), which is called by init()
#   Specifically:
#        This create the w_ object used throughout this code.
#        Initializes it
#        
#############################################################
#############################################################

public method open_catalog {} {

    #
    # create tcs object
    #
    global ::$w_.tcs

    astrocat $w_.cat

    # normally -catalog should be specified when creating this widget
    # if not, choose a default...
    if {"$itk_option(-catalog)" == ""} {
	return
	if {[catch {set catalog_list [$w_.cat info $itk_option(-catalogtype)]} msg]} {
	    error_dialog $msg $w_
	    return
	}
	set itk_option(-catalog) [lindex $catalog_list [expr [llength $catalog_list]-1]]
    }

    # open the catalog file
    #
    set name $itk_option(-catalog)
    if {[catch {$w_.cat open $name} msg]} {
	error_dialog $msg $w_
	return
    }
    
    # set iscat_ to true if the catalog is not an image server
    #
    set iscat_ 1
    if {"[$w_.cat servtype]" == "imagesvr"} {
	set iscat_ 0
    } 

    # if this is a local catalog, add it to the catalog menus and tree
    # Should check if local catalog was already known
    if {"[$w_.cat servtype]" == "local"} {
	update_catalog_menus

	# add to catalog tree, if there is one
	catch {.catinf insert_node $name}
	
	# add an entry to the config file for this catalog
	cat::CatalogInfo::save "" $w_ 0
    }
    
    # display catalog name in header and icon
    #debugStatement [format "/n/n$itk_option(-catalog) ... %s ... $name" [$w_.cat longname $name ]]
    #debugStatement [format "Setting cat window title to: %s" "[$w_.cat longname $name] ($itk_option(-number))" ]
    
    wm title $w_ "[file tail $name] ($itk_option(-number))"
    wm iconname $w_ "[$w_.cat shortname $name]"
    
}


#############################################################
#  Name: reload_catalog
#
#  Description:
#	Reload catalog.
#############################################################
#############################################################

method reload_catalog {catType} {
    global ::cbo_slits ::cbo_ot_slits

    set catalogName $itk_option(-catalog)
    set instType $itk_option(-instType)
    
    # Call select catalog again with specs for this catalog.
    select_catalog $catalogName local "namespace inscope ::skycat::SkyCat .skycat1.image" \
	"::skycat::vmSkySearch" 0 1 "" $instType
    
    # This is inconsistent!!!
    # The slits_OT() and slits() functions should redraw the slits, but they don't because for some reason at that
    # point the catalog is empty. Since redraw_canvas_items deletes the slitlets, I uncheck the checkboxes
#    if {$catType == "OT" && $cbo_ot_slits == 1} {
#	slits_OT
 #   }
  #  if {$catType == "ODF" && $cbo_slits == 1} {
#	slits
   # }

    update idletasks
    after idle [code $this redraw_canvas_items ]

}


method redraw_canvas_items {} {
    global ::cbo_slits ::cbo_ot_slits
    
    set catType $itk_option(-catType)
    
    # Redraw slits if reloading OT/ODF

    # why do we need cbo_slits in here?
    if {($catType == 1 || $catType == 3) && $cbo_ot_slits == 1} {
	# Clear slits on canvas.
	set target_canvas_ .skycat1.image.imagef.canvas
	$target_canvas_ delete slitMarkOT
#	slits_OT
	set cbo_ot_slits 0
    }

    if {($catType == 2 || $catType == 4) && $cbo_slits == 1} {
	# Clear slits on canvas.
	set target_canvas_ .skycat1.image.imagef.canvas
	$target_canvas_ delete slitMark
	$target_canvas_ delete shading
#	slits
	set cbo_slits 0
    }
    
    # Load the plotting symbol config
    # $this set_plot_symbols
}


#############################################################
#  Name: add_search_options
#
#  Description:
# add the search options panel
#############################################################
#############################################################

protected method add_search_options {} {
    # vmAstroQuery(n) widget for displaying catalog search options.
    itk_component add searchopts {
	set searchopts_ [::cat::vmAstroQuery $w_.searchopts \
			     -relief groove \
			     -borderwidth 2 \
			     -debug $itk_option(-debug) \
			     -astrocat [code $w_.cat] \
			     -searchcommand [code $this search] \
			     -command [code $this query_done]]
    }
    pack $itk_component(searchopts) \
	-side top -fill x
}


#############################################################
#  Name: add_result_table
#
#  Description:
#   Create the results_ table, by calling vmQueryResult   
#   Add the table for displaying the query results
#############################################################
#############################################################

protected method add_result_table {} {
    # QueryResult(n) widget to display catalog query results
    #debugStatement "add_result_table: entry... "
    
    itk_component add results {
	set results_ [::cat::vmQueryResult $w_.results \
			  -astrocat [code $w_.cat] \
			  -title "Search Results" \
			  -hscroll 1 \
			  -height 12 \		      
		      -sortcommand [code $this set_sort_cols] \
			  -layoutcommand [code $this set_show_cols] \
			  -selectmode extended \
			  -exportselection 0]
    } {
    }
    
    pack $itk_component(results) -side top -fill both -expand 1
    bind $results_.listbox <ButtonRelease-1> [code $this select_result_row]
    $results_ set_options {MORE PREVIEW more preview} Show 0
}




#############################################################
#  Name: check_keyword_presence
#
#  Description:
#   This function checks if a certain keyword (substring) 
#   is present in the fov file. Returns 0 or 1
#############################################################
#############################################################
protected method check_keyword_presence {keyword} {

    set home $::env(GMMPS)
    set instType $itk_option(-instType)	
    set fovfilename $home/config/${instType}_current_fov.dat
    
    catch { set fovList [open $fovfilename r] }
    
    set keyword_found 0
    
    # Read fov values into lists
    while {[gets $fovList line] >= 0} {
	# remove all leading and trailing white space
	# condense internal white space to single blanks
	set line [string trim $line ]
	set line [regsub -all "\[ \t]+" $line { }]
	# search for the keyword
	if {[string match #* $line] == 0} {
	    if {[string match $keyword* $line] == 1} {
		set keyword_found 1
		break
	    }
	}
    }
    return $keyword_found
}


#############################################################
#  Name: populate_olddetector_menu
#
#  Description:
#   This function extracts all detector names from
#   Detectors.dat 
#
#############################################################
#############################################################
protected method populate_olddetector_menu {instType comboBox} {
    set home $::env(GMMPS)

    # Open detector config file for reading. 
    set detFile [open "$home/config/Detectors.dat" r ]
    
    $comboBox add -label "None" \
	-command [code $this drawBoundaries "EraseAltDet" ""]

    while {[gets $detFile line] >= 0} {
	if {$line == ""} {
	    continue
	} elseif {[string index $line 0 ] == "#" } {
	    continue
	}
	
	# remove all leading and trailing white space
	# condense internal white space to single blanks
	# split the string into a list
	set line [string trim $line ]
	set line [regsub -all "\[ \t]+" $line { }]
	set line [split $line " " ]
	
	# Extract instrument type, detector id and pixel scale from detector property list. 
	set inst_status [lindex $line 0 ]
	set inst_short  [lindex $line 1 ]
	set inst_long   [lindex $line 3 ]
	
	# Only keep the detector if it is not the current (and live at the telescope) one
	if { $inst_short == $instType && $inst_status == 1} {
	    # That's the one we do the mask for. Skipping...
	    continue
	} else {
	    # Some reformatting of the instrument strings
	    set inst_mainID [lindex [split $inst_long "_"] 0]
	    set inst_secondID [lindex [split $inst_long "_"] 1]
	    if {$inst_secondID == "HAM"} {
		set inst_secondID "Hamamatsu"
	    }
	    if {$inst_secondID != ""} {
		if {$inst_status == 1} {
		    set inst_secondID "($inst_secondID)"
		} else {
		    set inst_secondID "($inst_secondID, old)"
		}
	    }
	    # Now compose the final instrument string
	    set inststring "$inst_mainID $inst_secondID"
	    $comboBox add -label $inststring \
		-command [code $this drawBoundaries $inst_long "$inststring"]
	}
    }
    
    ::close $detFile
}

#############################################################
#  Name: mps_add_dialog_buttons
#
#  Description:
#   This function is called when init is run for an OT catalog.
#	It creates the dialog buttons in the object table window.
#
#############################################################
#############################################################
protected method mps_add_dialog_buttons {instType} {
    
    # I know, stupid globals, but that's the only way I can make that work...
    global ::cbo_barcode ::cbo_maskarea ::cbo_detarea
    global ::cbo_detgaps ::cbo_pointing ::cbo_grayscale

    # The master frame containing everything
    pack [frame $w_.masterFrame -borderwidth 2 -relief groove -background $ot_bg_ ] \
	-side top -anchor w -padx 5 -pady 5 -ipadx 5 -ipady 5

    #############################################################
    #    There are three main frames: EDIT SHOW CONTROL
    #############################################################
    pack [frame $w_.editMainFrame -background $ot_bg_ ] \
	-side left -anchor n -fill y -padx 5 -pady 5 -ipadx 5 -ipady 5 -in $w_.masterFrame

    pack [frame $w_.showMainFrame1 -background $ot_bg_] \
	-side left -anchor n -fill y -padx 5 -pady 5 -ipadx 5 -ipady 5 -in $w_.masterFrame

    pack [frame $w_.showMainFrame2 -background $ot_bg_] \
	-side left -anchor s -fill y -padx 5 -pady 5 -ipadx 5 -ipady 5 -in $w_.masterFrame

    pack [frame $w_.controlMainFrame -background $ot_bg_] \
	-side left -anchor n -fill y -padx 5 -pady 5 -ipadx 5 -ipady 5 -in $w_.masterFrame

    #############################################################
    #                The EDIT Main Frame
    #############################################################
    # Frame for editing slit parameters
    pack [frame $w_.editSlitFrame -background $ot_bg_] \
	-side left -in $w_.editMainFrame -anchor n -fill x -padx 5 -ipadx 5

    # Frame for setting object priorities
    pack [frame $w_.editPriorityFrame -background $ot_bg_] \
	-side left -in $w_.editMainFrame -anchor n -fill x -padx 5 -ipadx 5

    # The 5 slit parameters we keep available for editing
    pack \
	[label $w_.slitLabel -text "(Bulk) Edit Slits" -bg $ot_bg_ -foreground #55f -anchor w] \
	[frame $w_.slitEditSpacer1 -height 10 -bg $ot_bg_] \
	[LabelEntry $w_.slitsize_x -text "Size X  :" -valuewidth 3 -background $ot_bg_ \
	     -labelfont $itk_option(-labelfont) -command [code $this _sel_parent Sall]] \
	[LabelEntry $w_.slitsize_y -text "Size Y  :" -valuewidth 3 -background $ot_bg_ \
	     -labelfont $itk_option(-labelfont) -command [code $this _sel_parent Sall]] \
	[LabelEntry $w_.slittilt   -text "Tilt    :" -valuewidth 3 -background $ot_bg_ \
	     -labelfont $itk_option(-labelfont) -command [code $this _sel_parent Sall]] \
	[LabelEntry $w_.slitpos_x  -text "Offset X:" -valuewidth 3 -background $ot_bg_ \
	     -labelfont $itk_option(-labelfont) -command [code $this _sel_parent Sall]] \
	[LabelEntry $w_.slitpos_y  -text "Offset Y:" -valuewidth 3 -background $ot_bg_ \
	     -labelfont $itk_option(-labelfont) -command [code $this _sel_parent Sall]] \
	-side top -anchor w -in $w_.editSlitFrame -fill x
    
    pack [button $w_.updateslit -text "Update Slit(s)" -bg #9ff -fg $button_text_ot_ \
	      -activebackground #bff -activeforeground $button_text_active_ot_ \
	      -command [code $this _sel_parent Sall] -anchor w] \
	-side bottom -anchor w -in $w_.editSlitFrame -fill x

    #-command [code $this "_sel Sall \n slits_OT"] -anchor w]

    # Apply colors to entry boxes of LabelEntry widgets.
    $w_.slitsize_y.entry configure -background $ot_bg_ -highlightbackground $ot_bg_
    $w_.slitsize_x.entry configure -background $ot_bg_ -highlightbackground $ot_bg_
    $w_.slittilt.entry   configure -background $ot_bg_ -highlightbackground $ot_bg_
    $w_.slitpos_y.entry  configure -background $ot_bg_ -highlightbackground $ot_bg_
    $w_.slitpos_x.entry  configure -background $ot_bg_ -highlightbackground $ot_bg_

    # Help
    add_short_help $w_.slitsize_x \
	{{bitmap b1} = Edit slit length (<cr> updates just this item on selection!)}
    add_short_help $w_.slitsize_y \
	{{bitmap b1} = Edit slit width (<cr> updates just this item on selection!)}
    add_short_help $w_.slittilt   \
	{{bitmap b1} = Edit slit tilt (<cr> updates just this item on selection!)}
    add_short_help $w_.slitpos_x  \
	{{bitmap b1} = Edit slit x-offset with respect to object (<cr> updates just this item on selection!)}
    add_short_help $w_.slitpos_y  \
	{{bitmap b1} = Edit slit y-offset with respect to object (<cr> updates just this item on selection!)}
    add_short_help $w_.updateslit \
	{{bitmap b1} = Update selected object with values from the edit fields}
    
    # Add the priority buttons
    pack \
	[label $w_.prioLabel -text "Set Priority" -bg $ot_bg_ -foreground #55f -anchor w] \
	[frame $w_.slitEditSpacer2 -height 10 -bg $ot_bg_] \
	[button $w_.prio1button -text "Priority 1" -bg #fcc -fg $button_text_ot_ \
	     -activebackground #fee -activeforeground $button_text_active_ot_ \
	     -command [code $this _sel_parent 1]] \
	[button $w_.prio2button -text "Priority 2" -bg #fcc -fg $button_text_ot_ \
	     -activebackground #fee -activeforeground $button_text_active_ot_ \
	     -command [code $this _sel_parent 2]] \
	[button $w_.prio3button -text "Priority 3" -bg #fcc -fg $button_text_ot_ \
	     -activebackground #fee -activeforeground $button_text_active_ot_ \
	     -command [code $this _sel_parent 3]] \
	[button $w_.prio0button -text "Acquisition" -bg #fcc -fg $button_text_ot_ \
	     -activebackground #fee -activeforeground $button_text_active_ot_ \
	     -command [code $this _sel_parent 0]] \
	[button $w_.prioXbutton -text "Ignore" -bg #fcc -fg $button_text_ot_ \
	     -activebackground #fee -activeforeground $button_text_active_ot_ \
	     -command [code $this _sel_parent X]] \
	-side top -anchor w -in $w_.editPriorityFrame -fill x

    # Help
    add_short_help $w_.prio0button {{bitmap b1} = Make this an acquisition object (Priority 0)}
    add_short_help $w_.prio1button {{bitmap b1} = Highest priority for this object (Priority 1)}
    add_short_help $w_.prio2button {{bitmap b1} = Second highest priority for this object (Priority 2)}
    add_short_help $w_.prio3button {{bitmap b1} = Third highest priority for this object (Priority 3)}
    add_short_help $w_.prioXbutton {{bitmap b1} = Ignore this object (Priority X)}


    ##################################################################
    #               The SHOW Main Frame
    ##################################################################
    pack [label $w_.gmmpsFuncLabel -text "Display options" -background $ot_bg_ -foreground #55f] \
	[frame $w_.showSpacer1 -height 10 -bg $ot_bg_] \
	-in $w_.showMainFrame1 -anchor w

    # CheckBoxes for plotting things
    # The OBJECTS checkbutton is defined in vmSkySearch as we need access to its plotting functions
    checkbutton $w_.plotCBO_slits -background $ot_bg_ -command [code $this slits_OT] \
	-activebackground $button_bg_active_ot_ -text "Slits" -variable cbo_ot_slits
    checkbutton $w_.plotCBO_pointing -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Pointing center" -variable cbo_pointing
    checkbutton $w_.plotCBO_detarea  -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Detector area" -variable cbo_detarea
    checkbutton $w_.plotCBO_detgaps  -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Detector gaps" -variable cbo_detgaps -disabledforeground #666
    checkbutton $w_.plotCBO_maskarea -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Slit placement\narea" -variable cbo_maskarea -justify left
    checkbutton $w_.plotCBO_barcode  -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Barcode" -variable cbo_barcode -disabledforeground #666
    checkbutton $w_.plotCBO_grayscale  -background $ot_bg_ -command [code $this toggle_grayscale] \
	-activebackground $button_bg_active_ot_ -text "Grayscale" -variable cbo_grayscale

    # default states
    $w_.plotCBO_slits     deselect
    $w_.plotCBO_pointing  deselect
    $w_.plotCBO_detarea   select
    $w_.plotCBO_detgaps   select
    $w_.plotCBO_maskarea  select
    $w_.plotCBO_barcode   deselect
    $w_.plotCBO_grayscale select

    # Pack checkboxes
    # The objects checkbutton gets inserted at the top from within vmSkySearch
    pack $w_.plotCBO_slits -in $w_.showMainFrame1 -anchor w
    pack $w_.plotCBO_detarea -in $w_.showMainFrame1 -anchor w
    pack $w_.plotCBO_detgaps -in $w_.showMainFrame1 -anchor w
    pack $w_.plotCBO_maskarea -in $w_.showMainFrame1 -anchor w

    pack [frame $w_.showSpacer2 -height 35 -bg $ot_bg_] -in $w_.showMainFrame2
    pack $w_.plotCBO_pointing -in $w_.showMainFrame2 -anchor w
    pack $w_.plotCBO_barcode -in $w_.showMainFrame2 -anchor w
    pack $w_.plotCBO_grayscale -in $w_.showMainFrame2 -anchor w
    # Reveal and populate the detector ComboBox
    pack [LabelMenu $w_.plotOldDets -relief groove -text "Overlay Detectors" \
	      -orient vertical -background $ot_bg_ -labelfont "Arial 12 bold"] \
	-anchor w -in $w_.showMainFrame2 -fill x
    populate_olddetector_menu $instType $w_.plotOldDets 

    # Deactivate some checkboxes if they don't apply
    if {[check_keyword_presence "GAP"] == 0} {
	$w_.plotCBO_detgaps deselect
	$w_.plotCBO_detgaps config -state disabled
    }
    if {[check_keyword_presence "BAR"] == 0} {
	$w_.plotCBO_barcode deselect
	$w_.plotCBO_barcode config -state disabled
    }

    # Help
    add_short_help $w_.plotCBO_detarea  {{bitmap b1} = Show the full detector extent}
    add_short_help $w_.plotCBO_maskarea {{bitmap b1} = Show the area where slitlets may be placed}
    add_short_help $w_.plotCBO_pointing {{bitmap b1} = Show the pointing center (CRPIX1/CRPIX2)}
    add_short_help $w_.plotCBO_barcode  {{bitmap b1} = Show the position of the mask bar code}
    add_short_help $w_.plotCBO_detgaps  {{bitmap b1} = Show the gaps between detectors. \
					     Acquisition stars must not fall into the gaps.}


    ##################################################################
    #                 The CONTROL Main Frame
    ##################################################################    
    pack [frame $w_.controlMainFrame.toplabel -background $ot_bg_] \
	-side top -anchor w -fill x
    pack [frame $w_.controlMainFrame.toplabel.miniframe -relief sunken -borderwidth 2 -background $ot_bg_] \
	-anchor w -side left
    pack [label $w_.l1 -text $instType -foreground #55f -background #ccf] \
	-in $w_.controlMainFrame.toplabel.miniframe -anchor w -side left
    pack [label $w_.l2 -text "Mask Design" -background $ot_bg_ -foreground #55f] \
	-in $w_.controlMainFrame.toplabel -anchor w -side left


#    pack [label $w_.controlLabel -text "Design $instType Masks" -background $ot_bg_ -foreground #55f] \
#	-in $w_.controlMainFrame -anchor w

    pack \
	[frame $w_.spocSpacer1 -height 5 -bg $ot_bg_] \
	[button $w_.spoc -text "Make Mask" \
	      -bg #bfb -fg $button_text_ot_ \
	      -activebackground #dfd \
	      -activeforeground $button_text_active_ot_ \
	      -command [code $this export $instType]] \
	-side top -in $w_.controlMainFrame -fill x
    

    if {$instType == "GMOS-N" || $instType == "GMOS-S"} {
	pack [button $w_.defineBands -text "Make Nod&Shuffle Mask" \
		  -bg #bfb -fg $button_text_ot_ \
		  -activebackground #dfd \
		  -activeforeground $button_text_active_ot_ \
		  -command [code $this define_bands]] \
	    -side top -in $w_.controlMainFrame -fill x

	add_short_help $w_.defineBands {{bitmap b1} = Design a bandshuffle or microshuffle mask}
    }
    
    pack \
	[frame $w_.spacer -height 30 -bg $button_bg_ot_] \
	[button $w_.reloadCat -text "Reload Catalog" \
	     -bg $button_bg_ot_ -fg $button_text_ot_ \
	     -activeforeground $button_text_active_ot_ \
	     -activebackground $button_bg_active_ot_ \
	     -command [code $this reload_catalog OT]] \
	[button $w_.closeCat -text "Close Catalog" \
	     -bg $button_bg_ot_ -fg $button_text_ot_ \
	     -activebackground $button_bg_active_ot_ \
	     -activeforeground $button_text_active_ot_ \
	     -command [code $this close ]] \
	-side top -in $w_.controlMainFrame -fill x
    
    add_short_help $w_.spoc      {{bitmap b1} = Design a normal mask}
    add_short_help $w_.reloadCat {{bitmap b1} = Reload catalog from file}
    add_short_help $w_.closeCat  {{bitmap b1} = Close this catalog}

    # Lastly, update the boundaries such that they reflect the checkbutton choices
    drawBoundaries
    toggle_grayscale
}


#############################################################
# A nicer error dialog that does not suffer from HUGE fonts
# This overrides a default skycat 'error_dialog' function
#############################################################
#############################################################
public method error_dialog {message} {
    tk_messageBox -type ok -icon error -title Error -message $message
}


#############################################################
#  Name: slit_add_dialog_buttons
#
#  Description:
#   This function is called when an ODF is loaded.
#	It creates the dialog buttons.
#
#############################################################
#############################################################

protected method slit_add_dialog_buttons {instType} {

    # Globals...
    global ::cbo_barcode ::cbo_maskarea ::cbo_detarea ::cbo_bands ::cbo_acqonly
    global ::cbo_detgaps ::cbo_pointing ::cbo_slits ::cbo_shading ::cbo_grayscale

    ##########################################################
    # The main frame containing all elements
    ##########################################################
    pack [frame $w_.odfMainFrame -borderwidth 2 -relief groove -background $odf_bg_] \
	-side left -anchor w -padx 5 -pady 5 -ipadx 5 -ipady 5

    pack [frame $w_.odfInfoFrame -borderwidth 2 -relief groove -background $odf_bg_] \
	-side left -anchor w -padx 5 -pady 5 -ipadx 5 -ipady 5

#    pack [frame $w_.odfTopFrame -background $odf_bg_] \
#	-side top -anchor n -padx 5 -pady 5 -in $w_.odfMainFrame -fill x -expand 1

#    pack [frame $w_.odfTopFrame.miniframe -relief sunken -borderwidth 2 -background $ot_bg_] \
#	-anchor e -side right -padx 5

#    pack [label $w_.odfInstTypeLabel -text $instType -foreground #55f -background #ccf] \
#	-in $w_.odfTopFrame.miniframe -anchor e -side right

    ##########################################################
    # The two frames containing the checkbuttons, 
    # and one for the Reload/Close buttons
    ##########################################################
    pack \
	[frame $w_.odfButtonFrame1 -background $ot_bg_] \
	-in $w_.odfMainFrame -fill x -padx 10 -pady 10 -side left -anchor n
    pack \
	[frame $w_.odfButtonFrame2 -background $ot_bg_] \
	-in $w_.odfMainFrame -fill x -padx 10 -pady 10 -side left -anchor s
    pack \
	[frame $w_.odfButtonFrame3 -background $ot_bg_] \
	-in $w_.odfMainFrame -fill x -padx 10 -pady 10 -side right -anchor n

    # CheckBoxes for plotting things
    # The OBJECTS checkbutton is defined in vmSkySearch as we need access to its plotting functions
    checkbutton $w_.odfPlotCBO_slits     -background $ot_bg_ -command [code $this slits] \
	-activebackground $button_bg_active_ot_ -text "Slits & Spectra" -variable cbo_slits
    checkbutton $w_.odfPlotCBO_shading   -background $ot_bg_ -command [code $this slits] \
	-activebackground $button_bg_active_ot_ -text "Slit shading" -variable cbo_shading -disabledforeground #666
    checkbutton $w_.odfPlotCBO_pointing  -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Pointing center" -variable cbo_pointing
    checkbutton $w_.odfPlotCBO_detarea   -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Detector area" -variable cbo_detarea
    checkbutton $w_.odfPlotCBO_detgaps   -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Detector gaps" -variable cbo_detgaps -disabledforeground #666
    checkbutton $w_.odfPlotCBO_maskarea  -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Slit placement area" -justify left -variable cbo_maskarea
    checkbutton $w_.odfPlotCBO_barcode   -background $ot_bg_ -command [code $this drawBoundaries] \
	-activebackground $button_bg_active_ot_ -text "Barcode" -variable cbo_barcode -disabledforeground #666
    checkbutton $w_.odfPlotCBO_banddef   -background $ot_bg_ -command [code $this define_bands drawBandsOnly] \
	-activebackground $button_bg_active_ot_ -text "N&S Bands" -variable cbo_bands -disabledforeground #666
    checkbutton $w_.odfPlotCBO_grayscale -background $ot_bg_ -command [code $this toggle_grayscale] \
	-activebackground $button_bg_active_ot_ -text "Grayscale" -variable cbo_grayscale
    checkbutton $w_.odfPlotCBO_acqonly   -background $ot_bg_ -command [code $this slits "acqOnly"] \
	-activebackground $button_bg_active_ot_ -text "Highlight acq stars" -variable cbo_acqonly

    # default states
    $w_.odfPlotCBO_slits     deselect
    $w_.odfPlotCBO_shading   select
    $w_.odfPlotCBO_pointing  deselect
    $w_.odfPlotCBO_detarea   select
    $w_.odfPlotCBO_detgaps   select
    $w_.odfPlotCBO_maskarea  select
    $w_.odfPlotCBO_grayscale select
    $w_.odfPlotCBO_banddef   deselect
    $w_.odfPlotCBO_barcode   deselect
    $w_.odfPlotCBO_acqonly   deselect

    # Deactivate some checkboxes if they don't apply
    if {[check_keyword_presence "GAP"] == 0} {
	$w_.odfPlotCBO_detgaps deselect
	$w_.odfPlotCBO_detgaps config -state disabled
    }
    if {[check_keyword_presence "BAR"] == 0} {
	$w_.odfPlotCBO_barcode deselect
	$w_.odfPlotCBO_barcode config -state disabled
    }

    set detName [lindex $detector_ 3 ]

    # Pack checkboxes

    # Button frame 1
    # The objects checkbutton gets inserted at the top from within vmSkySearch
    pack [label $w_.odfButtonLabel -text "Display options" -background $ot_bg_ -foreground #55f] \
	-in $w_.odfButtonFrame1 -anchor w -ipady 5
    pack $w_.odfPlotCBO_slits   -in $w_.odfButtonFrame1 -anchor w
    pack $w_.odfPlotCBO_shading -in $w_.odfButtonFrame1 -anchor w
    pack $w_.odfPlotCBO_detarea -in $w_.odfButtonFrame1 -anchor w
    pack $w_.odfPlotCBO_detgaps -in $w_.odfButtonFrame1 -anchor w
    pack $w_.odfPlotCBO_banddef -in $w_.odfButtonFrame1 -anchor w
    $w_.odfPlotCBO_banddef configure -state disabled

    # Activate the N&S band display button if applicable
    set NSmode ""
    set bandFile ""
    if {$instType == "GMOS-N" || $instType == "GMOS-S"} {
	# Is this a bandshuffle mask?

	# get the shuffling mode from the ODF
	catch {set bandFile [exec grep "#fits BDFILE" $itk_option(-catalog) | cut -f2 -d= ]}
	set bandFile [regsub -all "\[' \t]+" $bandFile {}]
	set path [file dirname $itk_option(-catalog) ]
	if {[file exists $path/$bandFile] && $bandFile != ""} {
	    set NSmode [exec grep "shuffleMode" $bandFile | cut -f2 -d: ]
	    set NSmode [string trim $NSmode ]
	    
	    if {$NSmode == "bandShuffle"} {
		$w_.odfPlotCBO_banddef select
		$w_.odfPlotCBO_banddef configure -state normal
		define_bands drawBandsOnly
	    }
	}	
	if {![file exists $path/$bandFile] && $bandFile != ""} {
	    error_dialog "Band definition file $bandFile not found!"
	    return
	}

	# If loaded from the ODF in GMMPS without having designed the mask before,
	# bandsOn will be empty because GMMPS has no record in memory of doing a bandshuffle mask.
	# So this should access the info directly in the ODF instead, but then the bandshuffle file 
	# might not be present (e.g. if the ODF was sent to us).
	# Have to store the bandshuffle file in the ODF instead, then we can do away with all these
	# case distinctions!

#	pack $w_.odfPlotCBO_banddef -in $w_.odfButtonFrame1 -anchor w \
#	    -after $w_.odfPlotCBO_detgaps
#	add_short_help $w_.odfPlotCBO_banddef {{bitmap b1} = Plot nod & shuffle bands}
#	set home [pwd ]
#	set canvas .skycat1.image.imagef.canvas
#	set EXTRABANDSEP 0
#	set target_im_ image2
#	
#	set bdfile ""
#	catch {set bdfile [string trim [exec grep "#fits BDFILE" $itk_option(-catalog) | cut -f2 -d= ]]}
#	
#	if {$bdfile != ""} {
#	    # Must be encased in ' ... ', remove. 
#	    set bdfile [string trim [string range $bdfile 1 end-1 ]]
#	    puts "$bdfile 0"
#	}
#	
#	if {$bdfile != "" && [file dirname $bdfile ] == "."} {
#	    # Make sure bdfile is a complete file path.
#	    set bdfile [format "%s/%s" [file dirname $itk_option(-catalog) ] $bdfile ]
#	    puts "$bdfile 1" 
#	}
    }

    # Button frame 2
    pack $w_.odfPlotCBO_acqonly   -in $w_.odfButtonFrame2 -anchor w
    pack $w_.odfPlotCBO_maskarea  -in $w_.odfButtonFrame2 -anchor w
    pack $w_.odfPlotCBO_pointing  -in $w_.odfButtonFrame2 -anchor w
    pack $w_.odfPlotCBO_barcode   -in $w_.odfButtonFrame2 -anchor w
    pack $w_.odfPlotCBO_grayscale -in $w_.odfButtonFrame2 -anchor w
    
    # Reveal and populate the detector ComboBox
    pack [LabelMenu $w_.odfPlotOldDets -relief groove -text "Overlay Detectors" \
	      -orient vertical -background $ot_bg_ -labelfont "Arial 12 bold"] \
	-anchor w -in $w_.odfButtonFrame2 -fill x
    populate_olddetector_menu $instType $w_.odfPlotOldDets

    # Help
    add_short_help $w_.odfPlotCBO_acqonly   {{bitmap b1} = Show acquisition sources only}
    add_short_help $w_.odfPlotCBO_detarea   {{bitmap b1} = Show the full detector extent}
    add_short_help $w_.odfPlotCBO_maskarea  {{bitmap b1} = Show the area where slitlets may be placed}
    add_short_help $w_.odfPlotCBO_pointing  {{bitmap b1} = Show the pointing center (CRPIX1/CRPIX2)}
    add_short_help $w_.odfPlotCBO_barcode   {{bitmap b1} = Show the position of the mask bar code}
    add_short_help $w_.odfPlotCBO_banddef   {{bitmap b1} = Plot nod & shuffle bands}
    add_short_help $w_.odfPlotCBO_grayscale {{bitmap b1} = Toggles between grayscale and classic skycat color maps}
    add_short_help $w_.odfPlotCBO_detgaps   {{bitmap b1} = Show the gaps between detectors. \
					     Acquisition stars must not fall into the gaps.}

    # Button frame 3
    pack [label $w_.odfButtonLabel2 -text "Catalog options" -background $ot_bg_ -foreground #55f] \
	-in $w_.odfButtonFrame3 -anchor w -ipady 5
    pack \
	[button $w_.odfCloseCat -text "Close Catalog" \
	     -bg $button_bg_ot_ -fg $button_text_ot_ \
	     -activebackground $button_bg_active_ot_ \
	     -activeforeground $button_text_active_ot_ \
	     -command [code $this close ]] \
	[button $w_.odfReloadCat -text "Reload Catalog" \
	     -bg $button_bg_ot_ -fg $button_text_ot_ \
	     -activeforeground $button_text_active_ot_ \
	     -activebackground $button_bg_active_ot_ \
	     -command [code $this reload_catalog ODF]] \
	-side bottom -anchor s -in $w_.odfButtonFrame3 -fill both

    add_short_help $w_.odfPlotCBO_slits {{bitmap b1} = Plot slits and spectrum overlay}
    add_short_help $w_.odfReloadCat     {{bitmap b1} = Reload catalog from file}
    add_short_help $w_.odfCloseCat      {{bitmap b2} = Close catalog window}

    # The ODF info window
    text $w_.odfInfoFrame.03 -width 40 -height 10 -setgrid true -wrap none \
	-xscrollcommand [list $w_.odfInfoFrame.01 set] \
	-yscrollcommand [list $w_.odfInfoFrame.02 set] \
	-background white
    scrollbar $w_.odfInfoFrame.01 \
	-command [list $w_.odfInfoFrame.03 xview] -orient horiz
    scrollbar $w_.odfInfoFrame.02 \
	-command [list $w_.odfInfoFrame.03 yview] -orient vert
    pack $w_.odfInfoFrame.01 -side bottom -fill x
    pack $w_.odfInfoFrame.02 -side right -fill y
    pack $w_.odfInfoFrame.03 -side left -fill both -expand true
    pack $w_.odfInfoFrame -side left -fill both -expand true

    # Populate ODF Info frame

    # Lastly, update the image display with the button choices
    drawBoundaries
    if { ($instType == "GMOS-N" || $instType == "GMOS-S") && $NSmode == "bandShuffle"} {
	define_bands drawBandsOnly
    }
    toggle_grayscale
}


#############################################################
#  Name: toggle_grayscale
#
#  Description:
#  Toggles between grayscale and classic skycat color maps
#
#############################################################
#############################################################
protected method toggle_grayscale {} {
    global ::cbo_grayscale
    global rtd_library


    if {$cbo_grayscale == 1} {
	image2 cmap file $rtd_library/colormaps/ramp.lasc
    } else {
	image2 cmap file $rtd_library/colormaps/real.lasc
    }
}


#############################################################
#  Name: symbol
#
#  Description:
#   This function is used to retrieve from the image information
#   and to write that information, including the symbol information
#   to the catalog file.
#
#############################################################
#############################################################
public proc symbol {name} {
    
    # debugStatement "\nproc symbol: Entry. catname = $name"
    
    
    #
    #  Open a file name.tmp for appending.
    #  Put in:
    #    the query results, 
    #    how to draw the symbols
    #    also fits keyword information that we extracted above.
    #    Finish with an "End config entry" line.
    #
    set tmp [open $name.tmp a]
    puts $tmp "QueryResult"
    puts $tmp ""
    puts $tmp "# Config entry for original catalog server:"
    puts $tmp "serv_type: local"
    puts $tmp "long_name: $name"
    puts $tmp "short_name: [file tail $name]"
    puts $tmp "url: $name"
    puts $tmp "symbol: \{x_ccd y_ccd priority\} \{circle white \{\} \{\} \{\} \{\$priority == \"3\"\}\} \{15 \{\}\} : \{x_ccd y_ccd priority\} \{triangle blue \{\} \{\} \{\} \{\$priority == \"1\"\}\} \{15 \{\}\} : \{x_ccd y_ccd priority\} \{diamond cyan \{\} \{\} \{\} \{\$priority == \"0\"\}\} \{20 \{\}\} : \{x_ccd y_ccd priority\} \{cross yellow \{\} \{\} \{\} \{\$priority == \"X\"\}\} \{15 \{\}\} : \{x_ccd y_ccd priority\} \{square green \{\} \{\} \{\} \{\$priority == \"2\"\}\} \{15 \{\}\}"
    puts $tmp "# Fits keywords"
    set buffer ""
    catch { set buffer [exec grep #fits $name ] }
    puts $tmp "$buffer"
    puts $tmp "# End fits keywords"
    puts $tmp "# Curved slits"
    puts $tmp "# End curved slits"
    puts $tmp "# End config entry"
    ::close $tmp
    
    #
    #  Grep in the catalog file the line "End config entry" and get
    #  the line number.  Which will give you the line where the
    #  headers and data actually starts.
    #  Then append to the above written name.tmp file all of the header
    #  and data out of the original configuration file.  
    #  Then move name.tmp to name.
    set n [exec grep -n "End config entry" $name | cut -d: -f1]
    #puts $n,"[]",$name
    set n [expr $n + 1]
    
    exec tail -n +$n $name >> $name.tmp
    catch {file delete $name}
    file rename -force $name.tmp $name
    
    #debugStatement "End proc symbol"
}


#############################################################
#  Name: export
#
#  Description:
#   This function is called when the "Design Mask" button
#   is pressed. It creates the Master ODF file(s). 
#   It calls the gmmps_spoc method.  Which will bring up
#	  the popup to allow you to run the spoc algorithm.
#
#############################################################
#############################################################

public method export {instType} {
    #
    #  Determine if slitpos_x and slitpos_y exist in the
    #  catalog files.
    set addPX ""
    set addPY ""
    catch {set addPX [exec grep slitpos_x $itk_option(-catalog) ]}
    catch {set addPY [exec grep slitpos_y $itk_option(-catalog) ]}
    if { $addPX == "" } {
	set addPX "yes" 
    } else {
	set addPX "no" 
    }
    if { $addPY == "" } {
	set addPY "yes" 
    } else {
	set addPY "no" 
    }
    
    # Make sure slit tilts are valid.
    if {[check_tilts -45.0 45.0 ]} {
	return
    }
    
    # Find detector info.
    setDetInfo $instType
    
    # Find Ra/Dec
    setRaDecInfo

    # destroy any open spoc window.
    catch {destroy $spoc_}
    
    set dispDirection [getDispDirection]
    if {$dispDirection == "horizontal"} {
	set slitWidth $itk_option(-xsize)
    } else {
	set slitWidth $itk_option(-ysize)
    }
    
    #  Call the gmmps_spoc.tcl init function
    set CRPIX1 [get_crpix_image CRPIX1]
    set CRPIX2 [get_crpix_image CRPIX2]

    set spoc_ [utilReUseWidget cat::gmmps_spoc .skycat1.spoc \
		   $this $itk_option(-catalog) $slitWidth \
		   $itk_option(-tilt) $addPX $addPY $itk_option(-pixs) \
		   $itk_option(-bw) $this.results $instType \
		   $config_file_ $DET_IMG_ $DET_SPEC_ $RA_IMG_ $DEC_IMG_ \
		   $CRPIX1 $CRPIX2 $dispDirection ]
}


#############################################################
#  Name: check_tilts
#
#  Description:
#   Makes sure there are no invalid slittilts in the catalog. 
#############################################################
#############################################################

protected method check_tilts {min max} {
    #
    # Make sure there are no invalid slittilts.
    set rows [$this.results get_contents ]
    set columns [$this.results get_headings ]
    set tilt_col [lsearch  -regex $columns {(?i)slittilt} ]
    set id_col [lsearch -regex $columns {(?i)ID} ]
    set prio_col [lsearch -regex $columns {(?i)priority} ]
    set bad_tilt_rows [list ]
    
    # No slit tilts is good slit tilts.
    if {$tilt_col == -1} {
	return 0
    }
    
    set cnt 0
    set changed 0
    foreach row $rows {
	set tilt [format %.6f [lindex $row $tilt_col ]]
	
	if {$tilt > $max || $tilt < $min} {
	    set id [lindex $row $id_col ]
	    
	    set choice [choice_dialog "Tilted slits (object $id) are currently not allowed in Band-shuffling mode!" {{Set Priority to Ignore} {Reset Angle to 0.0} Cancel} {Set Priority to Ignore} $w_ ]
	    
	    if {$choice == "Cancel"} {
		return 1
		
	    } elseif {$choice == "Reset Angle to 0.0" } {
		
		set newrow [lreplace $row $tilt_col $tilt_col 0.000000 ]
		$this.results set_row $row $newrow
		set changed 1
		
	    } elseif {$choice == "Set Priority to Ignore" } {
		
		set newrow [lreplace $row $prio_col $prio_col "X" ]
		$this.results set_row $row $newrow
		set changed 1
	    }
	    
	    incr cnt
	}
    }
    
    # Save changes to file.
    if {$changed} {
	$this.results save_to_file $itk_option(-catalog) [$this.results get_contents ] "" 1
    }
    
    return 0
}



#############################################################
#  Name: layout_dialog
#
#  Description:
#   Do the dialog window layout.  Specifically:
#     Open the catalog
#     Add the menu bar and search options,
#     Get the named servers
#     create a results_ table 
#     Add the dialog buttons and progress bar
#############################################################
#############################################################

protected method layout_dialog {} {
    set instType $itk_option(-instType)
    set catType  $itk_option(-catType)

    # debugStatement "vmAC:layout_dialog instType = $instType"

    open_catalog
    add_menubar
    add_search_options
    
    # If not an image server, then assume its our type.
    if {$iscat_} {
	add_result_table
    }
    
    # Add bindings for GUI editing.
    # Load colorscheme.
    if {$catType == 1 || $catType == 3} {
	# load OT colorscheme.
	set bg $ot_bg_
	set bg2 $ot_bg2_
	set fg_text $ot_text_
	set catalog_bg_active $catalog_bg_active_ot_
	set catalog_text_active $catalog_text_active_ot_
	set catalog_bg $catalog_bg_ot_
	set catalog_text $catalog_text_ot_
	set button_bg $button_bg_ot_
	set button_text $button_text_ot_
	set button_bg_active $button_bg_active_ot_
	set button_text_active $button_text_active_ot_
	set menu_bg $menu_bg_ot_
	set menu_text $menu_text_ot_
	set menu_bg_active $menu_bg_active_ot_
	set menu_text_active $menu_text_active_ot_
    } else {
	# load ODF colorscheme.
	set bg $odf_bg_
	set bg2 $odf_bg2_
	set fg_text $odf_text_
	set catalog_bg_active $catalog_bg_active_odf_
	set catalog_text_active $catalog_text_active_odf_
	set catalog_bg $catalog_bg_odf_
	set catalog_text $catalog_text_odf_
	set button_bg $button_bg_odf_
	set button_text $button_text_odf_
	set button_bg_active $button_bg_active_odf_
	set button_text_active $button_text_active_odf_	
	set menu_bg $menu_bg_odf_
	set menu_text $menu_text_odf_
	set menu_bg_active $menu_bg_active_odf_
	set menu_text_active $menu_text_active_odf_
    }

    # Change tables settings here because the widget doesn't seem to want to 
    # listen to args we pass in.  
    $results_ configure \
	-height 18 \
	-width 80 \
	-background $bg \
	-foreground $fg_text \
	-selectbackground $catalog_bg_active \
	-selectforeground $catalog_text_active \
	-activebackground $button_bg_active

    $results_.listbox configure \
	-background $catalog_bg \
	-foreground $catalog_text
    $results_.headbox configure \
	-background $catalog_bg \
	-foreground $catalog_text \
	-selectbackground $catalog_bg \
	-selectforeground $catalog_text
    $results_.tf configure \
	-background $bg
    $results_.vf.vscroll configure  \
	-troughcolor $bg2 \
	-background $button_bg
    $results_.hf.hscroll configure  \
	-troughcolor $bg2 \
	-background $button_bg
    
    #  Change menu colors here. 
    $w_.menubar configure \
	-background $menu_bg \
	-highlightbackground $menu_bg_active\
	-highlightcolor $menu_text_active
    
    $w_.menubar.file configure \
	-background $menu_bg \
	-foreground $menu_text \
	-activebackground $menu_bg_active\
	-activeforeground $menu_text_active 
    
    $w_.menubar.edit configure \
	-background $menu_bg \
	-foreground $menu_text \
	-activebackground $menu_bg_active\
	-activeforeground $menu_text_active 

    $w_.menubar.options configure \
	-background $menu_bg \
	-foreground $menu_text \
	-activebackground $menu_bg_active\
	-activeforeground $menu_text_active

#    $w_.menubar.instrument configure \
#	-background $menu_bg \
#	-foreground $menu_text \
#	-activebackground $menu_bg_active\
#	-activeforeground $menu_text_active
}


#############################################################
#  Name: make_short_help
#
#  Description:
#   Add a short help window and set the help texts
#############################################################
#############################################################

protected method make_short_help {} {
    TopLevelWidget::make_short_help
    
    add_short_help $results_ \
	{Query results: {bitmap b1} = select object, \
	     double click {bitmap b1} = label object, \
	     {bitmap dragb2} = scroll list}
}


#############################################################
#  Name: select_catalog
#
#  Description:
#   This proc is called when the named catalog is selected from the menu.
#   It will check for specific column headings, set flags if the PRIORITY
#   slittype, and X/Y column headings are missing. It will then open
#   the catalog and save an instance of it.
#
#  Args:
#   name is the name of the catalog file.
#
#   serv_type is the type of the catalog: one of: catalog, archive, 
#   local, imagesvr, etc...
#
#   id is an optional unique id to be associated with a new catalog widget.
#	Not used at all?
#
#   classname is the name of the vmAstroCat subclass to use to create new
#   catalog widgets (defaults to "vmAstroCat").
#
#   $debug is a flag set from the command line arg.
#
#   $w should be the top level window of the caller (optional).
#############################################################
#############################################################

public proc select_catalog {name serv_type id classname {debug 0} tType {w ""} instType} {

    #  Declare global var.s
    # WARNING, have to attached these global variables to a specific
    # window.  Otherwise, when I open another window where slitsize_x
    # is there, and then go back to the original window, where it isn't
    # addXSIZE will be invalid.

    global acDebugLevel
    set acDebugLevel 0

    set addXSIZE "no"
    set addYSIZE "no"
    set addTILT "no"
    set addPRIORITY "no"
    set addSLITTYPE "no"
    set addXOFFSET "no"
    set addYOFFSET "no"
    set itk_option(-prior) "no"
    set itk_option(-xsize) "no"
    set itk_option(-ysize) "no"
    set itk_option(-tilt) "no"
    set itk_option(-stype) "no"
    set itk_option(-xoffset) "no"
    set itk_option(-yoffset) "no"
    set itk_option(-catType) $tType
    set gmos_cat ""
    set col_XSIZE ""
    set col_YSIZE ""
    set col_PRIORITY ""
    set col_SLITTYPE ""
    set col_TILT ""
    set col_XOFFSET ""
    set col_YOFFSET ""
    set raDecCol_s ""
    set xyCol_s ""
    set tmpHeadings ""
    
    #    global rtd_library
    #    $target_image_ cmap file $rtd_library/colormaps/ramp.lasc
    
    #  This type of file should be a FITS file, read it
    #  in and convert to a catalog file.
    if { $tType == 3 || $tType == 4 } {
	set index1 [string last . $name ]
	if { $index1 != -1 } {
	    set index1 [expr $index1 - 1 ]
	    set inName [string range $name 0 $index1 ]
	} else {
	    set inName $name
	}
	#debugStatement "name=$name index1=$index1 inName = $inName"
	#debugStatement "#About to call readInFitsFile"
	set x -1
	if {[catch {set x [ readInFitsFile $inName $tType]} msg]} {
	    #debugStatement "After exec3."
	    error_dialog $msg
	    return
	} else {
	    if { $x == -1 } {
		#debugStatement "select_catalog: fits file exit status: $x. Failed."
		return
	    } else {
		#debugStatement "select_catalog: fits conversion exit status: $x. Passed."
		set name [append inName ".cat"]
		#debugStatement "#new name=$name"
		# @@cba: get the image info in the cat file!
		symbol $name
	    }
	}
    } else {
	#
	#  ELSE this should be a catalog file, grep for "QueryResult"
	#  should be there for a catalog file.
	set catCheck ""
	catch {set catCheck [ exec grep QueryResult $name ] }
	if { $catCheck == "" } {
	    error_dialog "Error! This does not appear to be a valid OT/ODF ASCII catalog file."
	    return 
	}
    }
    
    # Set the pixel scale.
    set pixelScale [get_pixelscale_image]
    if {$pixelScale == 0.0 || $pixelScale == "NOT FOUND"} {
	error_dialog "ERROR: Invalid preimage. Could not identify pixel scale!"
	return
    }
    
    # Set instrument type from image (and catalog if unsuccessful) if it is not already set.
    # Executed if OT (ODF) catalog is loaded for the first time
    if {$instType == "NONE"} {
	set instType [getInstType $name]
	
	if {$instType == "NOT FOUND"} {
	    error_dialog "ERROR: Could not identify which instrument you want to create a mask for!"
	    return
	}
    }
    
    # Also, rewrite file making sure the catalog data is tab-separated. 
    # Get the column headings and save them.

    # Search for the line with all the "----------"
    set buffer [exec grep -n --regexp "^--" $name]
    set headlen [exec echo $buffer | cut -d: -f1]
    set ncol [exec echo $buffer | wc -w]

    set cat [open $name r ]
    set cattmp [open $name.tmp w ]
    
    # Copy all the header lines from the catalog into the new catalog.
    for {set n 1} {$n <= $headlen} {incr n 1} {
	gets $cat line
	puts $cattmp $line
    }
    
    # We do no want space separated lists, we want tab separated lists.
    while {[gets $cat line ] >= 0} {
	set line [concat $line ]
	set line [join $line "\t" ]
	puts $cattmp $line
    }
    
    ::close $cat
    ::close $cattmp

    file rename -force $name.tmp $name

    #  Grep for x_ccd, y_ccd, ID, slitsize_x, slitsize_y, priority, slittilt, slitpos_x, slitpos_y and 
    #  slittype in the catalog.  First 3 are mandatory.
    catch {set raDecCol_s   [exec grep -i RA $name | grep -i DEC | grep -i MAG ]}
    catch {set xyCol_s      [exec grep -i x_ccd $name | grep -i y_ccd | grep -i ID ]}
    catch {set col_XSIZE    [exec grep -i slitsize_x $name ]}
    catch {set col_YSIZE    [exec grep -i slitsize_y $name ]}
    catch {set col_XOFFSET  [exec grep -i slitpos_x $name ]}
    catch {set col_YOFFSET  [exec grep -i slitpos_y $name ]}
    catch {set col_TILT     [exec grep -i slittilt $name ]}
    catch {set col_PRIORITY [exec grep -i priority $name ]}
    catch {set col_SLITTYPE [exec grep -i slittype $name ]}
    catch {set tmpHeadings  [exec grep -i ID $name | grep -i RA | grep -i DEC | grep -i x_ccd | grep -i y_ccd ]}

    #  If we found the minimum table headings that we need then cont.
    if {$xyCol_s == ""} {
	error_dialog "ERROR: ID/x_ccd/y_ccd missing in $name"
	return
    } elseif {$raDecCol_s == ""} {
	error_dialog "ERROR: RA/DEC missing in $name"
	return
    } else {
	# when reading in the ASCII catalog, check for missing columns
	if { $tType != 2 || $tType != 4} {
	    #  If we didn't find the extra columns we need then
	    #  set the flag to indicate we need to add them.
	    if {$col_XSIZE == ""} {
		# I don't know why we need the following four lines
		symbol $name
		$astrocat_ entry remove $name
		cat::CatalogInfo::save {} $w 0
		$astrocat_ reload
		set addXSIZE "yes"
		set itk_option(-xsize) "yes"
	    }
	    if {$col_YSIZE == ""} {
		symbol $name
		$astrocat_ entry remove $name
		cat::CatalogInfo::save {} $w 0
		$astrocat_ reload
		set addYSIZE "yes"
		set itk_option(-ysize) "yes"
	    }
	    if {$col_TILT == ""} {
		symbol $name
		$astrocat_ entry remove $name
		cat::CatalogInfo::save {} $w 0
		$astrocat_ reload
		set addTILT "yes"
		set itk_option(-tilt) "yes"
	    }
	    if {$col_PRIORITY == ""} {
		symbol $name
		$astrocat_ entry remove $name
		cat::CatalogInfo::save {} $w 0
		$astrocat_ reload
		set addPRIORITY "yes"
		set itk_option(-prior) "yes"
	    }
	    if {$col_SLITTYPE == "" } {
		symbol $name
		$astrocat_ entry remove $name
		cat::CatalogInfo::save {} $w 0
		$astrocat_ reload
		set addSLITTYPE "yes"
		set itk_option(-stype) "yes"
	    }
	    if {$col_XOFFSET == ""} {
		symbol $name
		$astrocat_ entry remove $name
		cat::CatalogInfo::save {} $w 0
		$astrocat_ reload
		set addXOFFSET "yes"
		set itk_option(-xoffset) "yes"
	    }
	    if {$col_YOFFSET == ""} {
		symbol $name
		$astrocat_ entry remove $name
		cat::CatalogInfo::save {} $w 0
		$astrocat_ reload
		set addYOFFSET "yes"
		set itk_option(-yoffset) "yes"
	    }
	}
    }


    #  Complicated way to set the column positions of id, ra, dec.
    #  This gets used later to set the headings for the gmmps_sel call.
    set col_numb 0
    set ids ""
    set ras ""
    set decs ""
    
    foreach i $tmpHeadings {
	if { $i == "ID" } { 
	    set ids "id_col $col_numb"
	} elseif {$i == "RA" } { 
	    set ras "ra_col $col_numb"
	} elseif { $i == "DEC" } { 
	    set decs "dec_col $col_numb"
	}
	incr col_numb
    }
    set itk_option(-uphead) [list $ids $ras $decs]
    
    #  Open the catalog, and create an instance of it.  
    new_catalog $name $id $classname $debug "catalog" $tType \
	$addPRIORITY $addSLITTYPE $addTILT $addYSIZE $addXSIZE $addXOFFSET $addYOFFSET \
	$w $instType $pixelScale
}


#############################################################
#  Name: set_state
#
#  Description:
#   Set/reset widget states while busy 
#############################################################
#############################################################

public method set_state {state} {
    set state_ $state
    if {"$state" == "normal"} {
	catch {blt::busy release $w_.options}
	catch {focus -lastfor $w_.options}
    } else {
	catch {focus .}
	catch {blt::busy hold $w_.options}
    }
    update idletasks
}


#############################################################
#  Name: query_done
#
#  Description:
#   This method is called when the background query is done.
#
#   errmsg - If this is not empty, it contains an error message and
#   the following args should be ignored. If there were no
#   errors, this arg is an empty string.
#
#   headings  - are the column headings 
#
#   info - is a list of rows (result of query)
#
#   more - is a flag set to 1 if there were more rows available that were
#   not returned due to the maxrows limit set.
#
#############################################################
#############################################################

public method query_done {errmsg headings info more} {
    #debugStatement "vmAC:query_done $headings"

    if {"$errmsg" != ""} {
	# check if we need user/passwd info. errmsg should have the format:
	# "Authorization Required for <realm> at <host>"
	if {[regsub {Authorization Required} $errmsg {Enter username} msg]} {
	    lassign [passwd_dialog $msg] username passwd
	    if {"$username" != "" && "$passwd" != ""} {
		lassign $errmsg {} {} {} realm {} host
		$w_.cat authorize $username $passwd $realm $host
		$searchopts_ search
	    }
	} else {
	    set_state normal
	    catch {$results_ config -title "Search Results"}
	    # error messages starting with "***" are only 
	    # displayed in progress win
	    if {"[string range $errmsg 0 2]" != "***"} {
		error_dialog $errmsg $w_
	    }
	    # after 0 [list $w_.progress config -text $errmsg]
	}
    } else {
	if {! $iscat_} {
	    # for image servers, the info is the image file name
	    set filename $info
	    # load the image and remove the temp file
	    display_image_file $filename
	    catch {file delete $filename}
	} else {
	    busy {
		#	$results_ save_yview
		#	$results_ save_selection
		set prev_headings $itk_option(-headingx)

		set itk_option(-headingx) $headings
		set headings_ $headings

		set info_ $info
		# debugStatement "query_done: headings=$headings..."

		# update table
		$results_ config -headings $itk_option(-headingx)
		if {$reset_columns_} {
		    set reset_columns_ 0
		    reset_table
		}

		if {"$prev_headings" != "$itk_option(-headingx)"} {
		    $results_ update_options
		    set_menu_states
		}

		$results_ config -info $info
		
		# need to know the equinox of the results, if using world coords
		if {[$w_.cat iswcs]} {
		    lassign [$w_.searchopts get_pos_radius] {} {} equinox
		    $results_ config -equinox $equinox
		} 

		if {$more} {
		    set more "+"
		} else {
		    set more ""
		}
		$results_ config -title "Search Results ([$results_ total_rows]$more)"

		# note column indexes in array (use upper case to simplify search)
		catch {unset col_}
		set n -1
		foreach i $itk_option(-headingx) {
		    set col_($i) [incr n]
		}

		# plot stars
		#puts "plot time: [expr [lindex [time plot] 0]/1000000.]"
		#debugStatement "Beginning symbol plots?"				
		
		plot

		#debugStatement "Done plotting?"

		$results_ restore_yview
		$results_ restore_selection
	    }
	}
	set_state normal
    }
}


#############################################################
#  Name: search
#
#  Description:
#   Start the catalog search based on the current search options
#   and display the results in the table.
#############################################################
#############################################################

public method search {args} {

    set addPRIORITY $itk_option(-prior)
    set addSLITTYPE $itk_option(-stype)
    set addXSIZE    $itk_option(-xsize)
    set addYSIZE    $itk_option(-ysize)
    set addXOFFSET  $itk_option(-xoffset)
    set addYOFFSET  $itk_option(-yoffset)
    set addTILT     $itk_option(-tilt)
    set instType    $itk_option(-instType)
    set catname     $itk_option(-catalog)

    # start the query in the background
    catch {
	$results_ config -title "Searching ..."
	$results_ clear
    }
    set_state disabled
    set servtype [$w_.cat servtype]

    #  Add additional columns required

    # buffer:  Grep for dashes in the catalog  
    # headlen: Line number where the header ends
    # ncol:    Get the number of sets of unlines in the catalog, or number
    #          of column's in the catalog.
    # tmp:     Get the number of lines in the catalog
    # datalen: Get number of data lines (tmp-headlen)
    set buffer [exec grep -n --regexp "^--" $catname]
    set headlen [exec echo $buffer | cut -d: -f1]
    set ncol [exec echo $buffer | wc -w]
    set tmp [exec wc -l $catname]
    set datalen [expr [lindex $tmp 0] - $headlen]

    # In the following we create a temporary catalog which contains
    # all the missing columns. This catalog is then merged with the original one.

    set headerline1 ""
    set headerline2 ""
    set objectline  ""

    set dispDirection [getDispDirection]
    if {$dispDirection == "horizontal"} {
	set dimx "1.0"
	set dimy "5.0"
    } else {
	set dimx "5.0"
	set dimy "1.0"
    }

    # Add slitsize_x
    if {$addXSIZE == "yes" } {
	if {$addSLITTYPE == "no" && $addPRIORITY == "no"} {
	    error_dialog "ERROR: Can't add slitsize_x column!"
	    return 0
	} else {
	    set headerline1 [append headerline1 "slitsize_x\t"]
	    set headerline2 [append headerline2 "----------\t"]
	    set objectline  [append objectline $dimx "\t"]
	    set addXSIZE "no"
	    set itk_option(-xsize) "no"
	}
    }

    # Add slitsize_y
    if {$addYSIZE == "yes" } {
	if {$addSLITTYPE == "no" && $addPRIORITY == "no"} {
	    error_dialog "ERROR: Can't add slitsize_y column!"
	    return 0
	} else {
	    set headerline1 [append headerline1 "slitsize_y\t"]
	    set headerline2 [append headerline2 "----------\t"]
	    set objectline  [append objectline $dimy "\t"]
	    set addYSIZE "no"
	    set itk_option(-ysize) "no"
	}
    }

    # Add slitpos_x
    if {$addXOFFSET == "yes" } {
	if {$addSLITTYPE == "no" && $addPRIORITY == "no"} {
	    error_dialog "ERROR: Can't add slitpos_x column!"
	    return 0
	} else {
	    set headerline1 [append headerline1 "slitpos_x\t"]
	    set headerline2 [append headerline2 "---------\t"]
	    set objectline  [append objectline 0.0 "\t"]
	    set addXOFFSET "no"
	    set itk_option(-xoffset) "no"
	}
    }

    # Add slitpos_y
    if {$addYOFFSET == "yes" } {
	if {$addSLITTYPE == "no" && $addPRIORITY == "no"} {
	    error_dialog "ERROR: Can't add slitpos_y column!"
	    return 0
	} else {
	    set headerline1 [append headerline1 "slitpos_y\t"]
	    set headerline2 [append headerline2 "---------\t"]
	    set objectline  [append objectline 0.0 "\t"]
	    set addYOFFSET "no"
	    set itk_option(-yoffset) "no"
	}
    }

    # Add slittilt
    if {$addTILT == "yes" } {
	if {$addSLITTYPE == "no" && $addPRIORITY == "no"} {
	    error_dialog "ERROR: Can't add slittilt column!"
	    return 0
	} else {
	    set headerline1 [append headerline1 "slittilt\t"]
	    set headerline2 [append headerline2 "--------\t"]
	    set objectline  [append objectline "0.0\t"]
	    set addTILT "no"
	    set itk_option(-tilt) "no"
	}
    }

    # Add priority
    if {$addPRIORITY == "yes"} {
	set headerline1 [append headerline1 "priority\t"]
	set headerline2 [append headerline2 "--------\t"]
	set objectline  [append objectline  "2\t"]
	set addPRIORITY "no"
	set itk_option(-prior) "no"
    }

    # Add slittype
    if {$addSLITTYPE == "yes"} {
	set headerline1 [append headerline1 "slittype\t"]
	set headerline2 [append headerline2 "--------\t"]
	set objectline  [append objectline  "R\t"]
	set addSLITTYPE "no"
	set itk_option(-stype) "no"
    }

    # We must trim the trailing "tab" as it would otherwise lead to an error when parsing the
    # input catalog
    set headerline1 [string trim $headerline1 " \t"]
    set headerline2 [string trim $headerline2 " \t"]

    # Only do the following if columns have to be added!
    # If not, you'll end up in a lot of mess
    if {$headerline1 != "" && $headerline2 != ""} {
	#  Open a file for writing called name.col
	#  Write in there empty lines, up to where the data starts.
	set col [open $catname.col w]
	for {set n 3} {$n <= $headlen} {incr n 1} {
	    puts $col ""
	}
	
	# Put in name.col the header lines
	puts $col $headerline1
	puts $col $headerline2
	
	# Put in name.col the missing object lines
	for {set n 1} {$n <= $datalen} {incr n 1} {
	    set objectline [string trim $objectline " \t"]
	    puts $col $objectline
	}
	::close $col
	
	#  Mush the 2 files together to get the new column(s) in there.
	#  Then rename the name.tmp to name
	# we have to sleep for 1 second as otherwise skycat moves on before the process is done
	# and then we get a "missing columns" errors (or something like that) when loading the OT 
	# catalog. Has to be at least 1s, I got errors for less than that.
	set mx [sleep 1]
	set mx [exec paste $catname $catname.col > $catname.tmp]
	catch {file delete $catname.col}
	file rename -force $catname.tmp $catname
    }


    $searchopts_ search $itk_option(-catalog) $itk_option(-uphead)
    #debugStatement "search: exit... "
}


#############################################################
#  Name: check_local_catalog
#
#  Description:
#   If the given name is the name of a local catalog, check 
#   that the file exists, and if not, ask the user for a new 
#   path name or remove it from the menu.
#############################################################
#############################################################

public proc check_local_catalog {name {id ""} {classname vmAstroCat} 
				 {debug 0} {type "catalog"} tType addPRIORITY addSLITTYPE addTILT \
				     addYSIZE addXSIZE addXOFFSET addYOFFSET {w ""}} {

    if {"[$astrocat_ servtype $name]" != "local"} {
	return 0
    }
    set file [$astrocat_ url $name]
    if {[file isfile $file]} {
	return 0
    }
    
    set msg "The catalog file [file tail $file] does not exist. \
     Do you want to specify a new name?"
    switch [choice_dialog $msg {{Specify a new name} Cancel {Remove from menu}} {Specify a new name} $w] {
	{Specify a new name} {
	    set file [filename_dialog]
	    if {[file isfile $file]} {
		$astrocat_ entry remove $name
		new_catalog $file $id $classname $debug $type $tType \
		    $addPRIORITY $addSLITTYPE $addTILT $addYSIZE $addXSIZE $addXOFFSET $addYOFFSET $w
		if {[winfo exists .catinf]} {
		    .catinf reinit_tree
		}
	    }
	}
	Cancel {
	    return 1
	}
	{Remove from menu} {
	    $astrocat_ entry remove $name
	    if {[winfo exists .catinf]} {
		.catinf reinit_tree
	    }
	}
    }

    # update config file and menus
    cat::CatalogInfo::save "" $w 0
    update_catalog_menus
    return 1
}


#############################################################
#  Name: set_pos_radius
#
#  Description:
# set the values for the position and radius entries from the given
# list, which should be in the format {ra dec equinox radius} if
# we are using wcs, otherwise {x y radius}.
#############################################################
#############################################################

public method set_pos_radius {list} {
    $searchopts_ set_pos_radius $list
}


#############################################################
#  Name: set_pos_width_height
#
#  Description:
# set the values for the position, width and height entries from 
# the given list, which should be in the format 
# {ra dec equinox width height (in arcmin)} for wcs, or {x y width height},
# for pixel coordinates. 
#############################################################
#############################################################

public method set_pos_width_height {list} {
    $searchopts_ set_pos_width_height $list
}


#############################################################
#  Name: new_catalog
#
#  Description:
#   This proc is used to open a window for the named catalog, 
#   or reuse the existing one for the catalog, if it is already open.
#   Specifically:
#     check that type = "local"
#     save this instance
#     open the catalog
#     put the window under top level window
#
#  Args:
#   name is the long name of catalog from config file
#
#   id is an optional unique id to be associated with a new catalog widget.
#
#   classname is the name of the vmAstroCat subclass to use to create new
#   catalog widgets (defaults to "vmAstroCat").
#
#   debug is a flag: if true, run queries in foreground
#
#   type is the catalog serv_type (here it is enough to specify "catalog" or
#   "imagesvr".
#
#   w should be the top level window of the caller, if specified.
#############################################################
#############################################################

public proc new_catalog {name {id ""} {classname vmAstroCat} {debug 0} 
			 {type "catalog"} tType addPRIORITY addSLITTYPE addTILT addYSIZE addXSIZE \
			     addXOFFSET addYOFFSET {w ""} iType {pixelScale ""}} {
    
    # if it is a local catalog, 
    # make sure it exists still, or get a new name
    if {[check_local_catalog $name $id $classname $debug $type $tType \
	     $addPRIORITY $addSLITTYPE $addTILT $addYSIZE $addXSIZE $addXOFFSET $addYOFFSET $w] != 0} {
	return
    }
    
    # Send last loaded image and catalog info to config file.
    set loadedImage [$target_image_ cget -file ]
    gmmps_config [list "catalog" "image" "instrument" "catalog_type"] [list $name $loadedImage $iType $tType] 
    
    #  Save this as an instance name, id
    set i "$name,$id"
    if {[info exists instances_($i)] && [winfo exists $instances_($i)]} {
	utilRaiseWindow $instances_($i)
	if {"[$instances_($i).cat servtype]" == "local"} {
	    # for local catalogs, search automatically when opened
	    $instances_($i) search
	}
	return
    }
    
    #  Open the catalog
    if {[catch {$astrocat_ open $name} msg]} {
	error_dialog $msg
	return
    }
    
    # If $w was specified, put the window under that top level window
    # so we get the window numbers right (for cloning, see TopLevelWidget).
    if {[winfo exists $w]} {
	set instname $w.vmac[incr n_instances_]
    } else {
	set instname .vmac[incr n_instances_]
    }

    set instances_($i) \
	[$classname $instname \
	     -id $id \
	     -debug $debug \
	     -catalog $name \
	     -catalogtype $type \
	     -transient 0 \
	     -center 0 \
	     -catType $tType \
	     -prior $addPRIORITY \
	     -stype $addSLITTYPE \
	     -xsize $addXSIZE \
	     -ysize $addYSIZE \
	     -xoffset $addXOFFSET \
	     -yoffset $addYOFFSET \
	     -tilt $addTILT \
	     -pixs $pixelScale \
	     -instType $iType]
}


public method gmmps_config_ {keys {data ""}} {
    #debugStatement "gmmps_config_ $keys $data"
    
    return [gmmps_config $keys $data ]
}


#############################################################
#  Name: gmmps_config
#
#  Description:
#	Read or write configuration data to $config_file_
#	keys is a list of keys
#	data is a lits of associated data
#
#	If no data argument is provided we are assumed to be in 
#		read mode and the val corresponding to the keys are 
#		returned in a list, in order they were given. 
#	If a data argument is provided then that data is written
#		to the config file, based on its corresponding keys.
#		NOTE: all keys must have valid data for the method to complete.
#############################################################
#############################################################
public proc gmmps_config {keys {data ""}} {
    
    #debugStatement "gmmps_config entry"
    
    # Make sure keys and data are the same length if we are in write mode.
    if {$data != "" && ([llength $keys ] != [llength $data ])} {
	return ""
    }
    
    # If config file does not exist, create a new one. 
    if {![file exists $config_file_ ]} {
	set newConfig [open $config_file_ w ]
	::close $newConfig
    }

    # Open config file for reading. 
    set gmmpsConfig [open $config_file_ r ]
    
    set fileKeys [list ]
    set fileData [list ]
    set blanksAndComments [list ]
    set cnt 0
    
    # Populate fileKeys and fileData lists from config file.
    while {[gets $gmmpsConfig pair] >= 0} {	
	# Ignore comments and empty lines.
	if {$pair == "" || [string index [string trim $pair ] 0 ] == "#"} { 
	    # Store positions and contents of any blank lines and comments,
	    # so they can be restored later.
	    set blanksAndComments [lappend blanksAndComments [list $cnt $pair ]]
	    incr cnt
	    continue
	}
	
	set pair [split [string trim $pair ] "=" ]
	set fileKeys [lappend fileKeys [string trim [lindex $pair 0 ]]]
	set fileData [lappend fileData [string trim [lindex $pair 1 ]]]
	incr cnt
    }
    
    # Done reading, close file. Will reopen if a write is needed. 
    ::close $gmmpsConfig
    
    if {$data == ""} {
	set targetData [list ]
    } else {
	set targetKeys $fileKeys
	set targetData $fileData
    }
    
    # Gather relevant data in targetKeys and targetData. 
    set cnt 0
    foreach key $keys {
	# Search for key in keys loaded form file.
	set idx [lsearch -exact $fileKeys [string trim $key ]]
	
	set newDat [lindex $data $cnt ]
	set oldDat [lindex $fileData $idx ]
	
	if {$idx != -1} {
	    # Key present in config file.
	    
	    if {$data == ""} {
		# In read mode append the data to output list.	
		set targetData [lappend targetData $oldDat ]
	    } else {
		# In write mode edit data in targetData.
		set targetData [lreplace $targetData $idx $idx $newDat ]
	    }
	} else {
	    # Key not present in config file.
	    
	    if {$data == ""} {
		# In read mode all keys must be present.
		return ""
	    } else {
		# In write mode we want to add new data.
		set targetKeys [lappend targetKeys $key ]
		set targetData [lappend targetData $newDat ]
	    }
	}
	
	incr cnt
    }
    
    if {$data == ""} {
	# Read mode return data values.
	#debugStatement "gmmps_config: returning $targetData"
	return $targetData
    } else {
	# Write mode edit file.
	
	# Open config file for writing.
	set gmmpsConfig [open $config_file_ w ]
	
	set keyCnt 0
	set lineCnt 0
	
	# Write all keys back into config file.
	foreach key $targetKeys {
	    
	    # Write blank lines and comments back into place. 
	    if {[llength $blanksAndComments ] != 0 } {
		set tempBAC $blanksAndComments
		foreach blankLine $tempBAC {
		    if {[lindex $blankLine 0 ] == $lineCnt} {
			puts $gmmpsConfig [format "%s" [lindex $blankLine 1 ]]

			# Remove first element from blank and comment list. 
			if {[llength $blanksAndComments ] > 1} {
			    set blanksAndComments [lrange $blanksAndComments 1 end ] 
			} else {
			    set blanksAndComments ""
			}
			
			incr lineCnt
		    } else {
			break
		    }
		}
	    }
	    
	    set curData [lindex $targetData $keyCnt ]
	    set line "$key=$curData"
	    
	    puts $gmmpsConfig $line
	    
	    incr lineCnt
	    incr keyCnt
	}

	# Write blank lines and comments back into place. 
	# (for after config values are done.)
	if {[llength $blanksAndComments ] != 0 } {
	    set tempBAC $blanksAndComments
	    foreach blankLine $tempBAC {
		if {[lindex $blankLine 0 ] == $lineCnt} {
		    puts $gmmpsConfig [format "%s" [lindex $blankLine 1 ]]

		    # Remove first element from blank and comment list. 
		    if {[llength $blanksAndComments ] > 1} {
			set blanksAndComments [lrange $blanksAndComments 1 end ] 
		    } else {
			set blanksAndComments ""
		    }
		    incr lineCnt
		} else {
		    break
		}
	    }
	}
	
	::close $gmmpsConfig
	
	# Return success.
	return 0
    }
}

#############################################################
#  Name: instances
#
#  Description:
#   Return a Tcl list of instances of this class. By default
#   (and for backward compatibility) only catalog windows are
#   included - not image servers. If "choice" is "all", then
#   all instances are returned. If choice is imagesvr, only 
#   image server windows are returned.
#############################################################
#############################################################

public proc instances {{choice catalogs}} {
    set list {}
    if {[info exists instances_]} {
	foreach i [array names instances_] {
	    if {[winfo exists $instances_($i)]} {
		if {[$instances_($i) iscat]} {
		    # instance of a catalog window
		    if {"$choice" == "catalogs" || "$choice" == "all"} {
			lappend list $instances_($i)
		    }
		} else {
		    # instance of an image server window
		    if {"$choice" == "imagesvr"} {
			lappend list $instances_($i) 
		    }
		}
	    }
	}
    }
    return $list
}


#############################################################
#  Name: get_instance
#
#  Description:
#   Return the widget instance for the given catalog, if found,
#   otherwise an empty string
#############################################################
#############################################################

public proc get_instance {catalog} {
    foreach w [cat::vmAstroCat::instances all] {
	if {"[$w cget -catalog]" == "$catalog"} {
	    return $w
	}
    }
}


#############################################################
#  Name: iscat
#
#  Description:
#   Return 1 if this window is for a searchable catalog, 
#   and 0 otherwise (in which case it must be an image server window).
#############################################################
#############################################################

public method iscat {} {
    return $iscat_
}


# ---------------------------------------------------------------
# The following methods deal with images and plotting of objects
# in an image and are meant to be defined in a derived class.
# See the skycat/skycat/interp/library/SkySearch.tcl for an example.
# ---------------------------------------------------------------


# see plot method in vmSkySearch.tcl -df 

public method plot {} {
}


# display the given image file (To be defined in a subclass)

public method display_image_file {filename} {
}


#  return the equinox

public method get_equinox {} {
    return [component searchopts get_equinox]
}


#  return the name of the TableList
public method get_table {} {
    if {[info exists itk_component(results)]} {
	return $itk_component(results)
    }
    error "This catalog is an image server"
}


#############################################################
# Fetch pixelscale from fits image. 
# This is a recursive function!
#############################################################

public proc get_pixelscale_image {{HDU ""}} {
    
    set pixelScale "NOT FOUND"

    if {$HDU == ""} {
	# Search the primary HDU
	set fits [$target_image_ hdu fits ]
    } else {
	# Search the first extension HDU
	set fits [$target_image_ hdu fits 1]
    }

    # Get the (average) pixelscale from the CD matrix, ignoring any distortion
    set CD11 "NOT FOUND"
    set CD12 "NOT FOUND"
    set CD21 "NOT FOUND"
    set CD22 "NOT FOUND"

    # IMPORTANT: Note that I search a keyword including a trailing blank,
    # because I have seen pre-images which had e.g. CRPIX1A and CD1_1A in addition to CRPIX1 and CD1_1

    foreach header [split $fits "\n" ] {
	if {[string range $header 0 5 ] == "CD1_1 "} {
	    set CD11 [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
	    break
	}
    }
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 5 ] == "CD1_2 "} {
	    set CD12 [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
	    break
	}
    }
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 5 ] == "CD2_1 "} {
	    set CD21 [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
	    break
	}
    }
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 5 ] == "CD2_2 "} {
	    set CD22 [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
	    break
	}
    }
    
    if {$CD11 != "NOT FOUND" && $CD12 != "NOT FOUND" && $CD21 != "NOT FOUND" && $CD22 != "NOT FOUND"} {
	set ps1 [expr sqrt($CD11*$CD11+$CD12*$CD12) * 3600. ]
	set ps2 [expr sqrt($CD22*$CD22+$CD21*$CD21) * 3600. ]
	set pixelScale [expr ($ps1+$ps2) / 2. ]
	return $pixelScale
    }

    # No? Then try and get the pixelscale from the CDELT keywords
    if { $pixelScale == "NOT FOUND" } {
	set CDELT1 "NOT FOUND"
	set CDELT2 "NOT FOUND"
	foreach header [split $fits "\n" ] {
	    if {[string range $header 0 6 ] == "CDELT1 "} {
		set CDELT1 [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
		break
	    }
	}
	foreach header [split $fits "\n" ] {
	    if {[string range $header 0 6 ] == "CDELT2 "} {
		set CDELT2 [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
		break
	    }
	}
	if {$CDELT1 != "NOT FOUND" && $CDELT2 != "NOT FOUND"} {
	    set pixelScale [expr sqrt($CDELT1*$CDELT1+$CDELT2*$CDELT2) * 3600. ]
	    return $pixelScale
	}
    }

    # Still nothing? Here is the recursive part (this function calling itself) 
    # (will run only once, because HDU="")
    if {$pixelScale == "NOT FOUND" && $HDU == ""} {
	set pixelScale [get_pixelscale_image 1]

    }

    # Getting desperate!
    if {$pixelScale == "NOT FOUND"} {
	tk_messageBox -message "WARNING: No valid WCS information (CD matrix nor CDELT) in the FITS header!!! Looking for PIXSCALE keyword..." -parent $w_ -title "Missing WCS information"

	# Try and find the PIXSCALE keyword in the primary HDY
	set fits [$target_image_ hdu fits ]
	foreach header [split $fits "\n" ] {
	    if {[string range $header 0 7 ] == "PIXSCALE"} {
		set pixelScale [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
		break
	    }
	}
	if {$pixelScale == "NOT FOUND"} {
	    # Try and find the PIXSCALE keyword in the first extension HDU
	    set fits [$target_image_ hdu fits 1]
	    foreach header [split $fits "\n" ] {
		if {[string range $header 0 7 ] == "PIXSCALE"} {
		    set pixelScale [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
		    break
		}
	    }
	}
	# Return whatever we have ("NOT FOUND" will trigger an error in the calling function) 
	return [string trim $pixelScale ]
    }
}


#############################################################
#  Name: getDispDirection
#
#  Description:
#   Gets the dispersion direction from config/Detectors.dat
#
#############################################################
#############################################################

public method getDispDirection {} {
    set home $::env(GMMPS)
    set instType $itk_option(-instType)
    
    # Open detector config file for reading. 
    set detFile [open "$home/config/Detectors.dat" r ]
    
    set dispDirection ""

    while {[gets $detFile line] >= 0} {
	
	if {$line == ""} {
	    continue
	} elseif {[string index $line 0 ] == "#" } {
	    continue
	}
	
	# remove all leading and trailing white space
	# condense internal white space to single blanks
	# split the string into a list
	set line [string trim $line ]
	set line [regsub -all "\[ \t]+" $line { }]
	set line [split $line " " ]

	# Extract instrument type and detector id from detector property list. 
	set inst [lindex $line 1 ]
	
	if {$inst == $instType} {
	    set dispDirection [lindex $line 4 ]
	}
    }
    
    ::close $detFile

    if {$dispDirection == ""} {
	puts "ERROR: Dispersion direction not found for $instType in $home/config/Detectors.dat!"
    }
    
    return $dispDirection
}


#############################################################
#  Name: getInstsType
#
#  Description:
#   Finds the instrument type from the image, and alternatively the catalog.
#  
#############################################################
#############################################################

protected proc getInstType {name} {
    
    set instType "NOT FOUND"

    # Search in the image
    set fits [$target_image_ hdu fits 1 ]    
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 7 ] == "INSTRUME"} {
	    set instType [lindex [split $header "'" ] 1 ]
	    break
	}
    }

    # Grep from the catalog if unsuccessful
    if {$instType == "NOT FOUND"} {
	# The second 'cut' is to trim any comments
	# (which break the string comparison to determine instType)
	# Must use catch as otherwise a 'main: child process exited abnormally' is thrown even though
	# the Linux command itself just returns an empty string. Don't understand why.
	catch {set instType [exec grep "#fits INSTRUME" $name | cut -f2 -d= | cut -f1 -d/]}
    }
    
    if {$instType == "NOT FOUND"} {
	# Will return error in parent
	return $instType;
    }

    # Capitalize the instrument name, remove blanks, tabs and quotes
    set instType [string toupper $instType]
    set instType [regsub -all "\[' \t]+" $instType {}]
    
    # Check for AO configuration with F2
    if {$instType == "F2"} {
	set aofold ""
	catch { set aofold [$target_image_ fits get AOFOLD] }
	if {$aofold == "in"} {
	    set instType "F2-AO"
	}
    }
    
    # If your instrument had different INSTRUME keywords in the past, then here is the place
    # to translate them to the latest valid string
    
    return $instType
}


#############################################################
#  Name: setDetInfo
#
#  Description:
#   Search for detector information in the preimage file and detector
#	configuration file.
#############################################################
#############################################################	

protected method setDetInfo {instType} {
    
    # Find DET_SPEC_ based on instType and info in detector config file.
    set detFile [open "$home_/config/Detectors.dat" r ]
    
    while {[gets $detFile line ] >= 0} {	
	if {$line == ""} {
	    continue
	} elseif {[string index $line 0 ] == "#" } {
	    continue
	}
	
	# remove all leading and trailing white space
	# condense internal white space to single blanks
	# split the string into a list
	set line [string trim $line ]
	set line [regsub -all "\[ \t]+" $line { }]
	set line [split $line " " ]
	
	set lineInst [lindex $line 1 ]
	set installed [lindex $line 0 ]
	
	if {$lineInst == $instType && $installed == 1} {
	    # Found the installed detector for our instrument! 
	    set DET_SPEC_ [lindex $line 2 ]
	    break
	}
    }
    ::close $detFile
    
    # Find DET_IMG from preimage det_id.
    set DET_IMG_ [get_detid ]
    if {$DET_IMG_ == "NOT FOUND" || $DET_IMG_ == ""} {
	set DET_IMG_ "UNKNOWN"
    }
    
    return
}



#############################################################
#  Name: setRaDecInfo
#
#  Description:
#  Search for RA/DEC in the preimage
#############################################################
#############################################################	

protected method setRaDecInfo {} {
    
    set RA_IMG_  [get_ra_image ]
    set DEC_IMG_ [get_dec_image ]
    
    return
}


#############################################################
#  Name: is_sorted
#
#  Description: Inputs a list of column rows and outputs 1 or 0, depending
#	if the list is in sorted ascending order in the ID column, or not. 
#   
#############################################################

protected method is_sorted {rows} {
    set id_col 0
    set last -1
    
    foreach row $rows {
	set cur [lindex $row $id_col ]
	if {$cur > $last} {
	    set last $cur
	} else {
	    return 0
	}
    }
    return 1
}


#############################################################
#  Name: sort_rowlist
#
#  Description: Sorts a given rowlist by id, and returns the sorted list 
#	(increasing order).
#############################################################

protected method sort_rowlist {rows} {
    set id_col 0
    # Sort this data. 
    set sorted_rows [lsort -integer -index $id_col $rows ]
    return $sorted_rows
}



#############################################################
# Fetch detector id from the fits image. 
#############################################################

protected proc get_detid {} {
    # Find target ID from fits image.		
    
    set targetDetId "NOT FOUND"
    set fits [$target_image_ hdu fits 1 ]
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 4 ] == "DETID"} {
	    set targetDetId [lindex [split $header "'" ] 1 ]
	    set targetDetId [string trim $targetDetId ]
	    # replace blanks within the string by underscores (for GMOS-N)
	    set targetDetId [regsub -all { +} $targetDetId {_}]
	    break
	}
    }
    
    return $targetDetId
}

#############################################################
# Fetch RA from the fits image
#############################################################

protected proc get_ra_image {} {

    set ra_img "UNKNOWN"
    set fits [$target_image_ hdu fits 1 ]

    foreach header [split $fits "\n" ] {
	if {[string range $header 0 7 ] == "RA      "} {
	    # replace multiple empty spaces by single spaces, then extract keyword value
	    regsub -all { +} $header { } header 
	    set ra_img [lindex [split $header " " ] 2 ]
	    break
	}
	if {[string range $header 0 7 ] == "CRVAL1  "} {
	    # replace multiple empty spaces by single spaces, then extract keyword value
	    regsub -all { +} $header { } header 
	    set ra_img [lindex [split $header " " ] 2 ]
	    break
	}
    }
    
    if { $ra_img == "UNKNOWN"} { 
	error_dialog "Error! Did not find RA nor CRVAL1 keyword in $target_image_" 
	return 0.0
    }

    return [string trim $ra_img]
}

######################################################################
# Fetch the reference pixel from the image
# (needed to plot the FoV and chip gaps); refpix = CRPIX1 or CRPIX2
######################################################################
protected proc get_crpix_image {refpix} {

    set crpix "UNKNOWN"
    set naxis 0
    if {$refpix == "CRPIX1"} {
	set refaxiskey "NAXIS1"
    } else {
	set refaxiskey "NAXIS2"
    }	
    set pseudoimage "no"

    set fits [$target_image_ hdu fits ]

    # Search the OBJECT keyword to check for pseudoimage;
    # if yes, then the values returned for CRPIX1/2 will be half NAXIS1/2
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 5 ] == "OBJECT"} {
	    # replace multiple empty spaces by single spaces, then extract keyword value
	    regsub -all { +} $header { } header
	    set objectkey [lindex [split $header " " ] 2 ]
	    set objectkey [string toupper $objectkey]
	    if {[string match *PSEUDO* $objectkey] == 1} {
		set pseudoimage "yes"
	    }
	}
    }


    # Search the primary HDU
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 5 ] == $refpix} {
	    # replace multiple empty spaces by single spaces, then extract keyword value
	    regsub -all { +} $header { } header 
	    set crpix [lindex [split $header " " ] 2 ]
	}
    }


    # Search the first extension HDU
    if {$crpix == "UNKNOWN" } {
	set fits [$target_image_ hdu fits 1]
	foreach header [split $fits "\n" ] {
	    if {[string range $header 0 5 ] == $refpix} {
		# replace multiple empty spaces by single spaces, then extract keyword value
		regsub -all { +} $header { } header 
		set crpix [lindex [split $header " " ] 2 ]
	    }
	}
    }


    # Desperate mode (no CRPIX, or pseudoimage)
    if {$crpix == "UNKNOWN" || $pseudoimage == "yes" } {
	set fits [$target_image_ hdu fits]
	foreach header [split $fits "\n" ] {
	    if {[string range $header 0 5 ] == $refaxiskey} {
		# replace multiple empty spaces by single spaces, then extract keyword value
		regsub -all { +} $header { } header
		set naxis [lindex [split $header " " ] 2 ]
	    }
	    if {$naxis != 0 } {
		set crpix [expr $naxis / 2 ]
		if {$crpix == "UNKNOWN"} {
		    tk_messageBox -message "Warning: Did not find $refpix keyword in $target_image_. Choosing half of $refaxiskey instead!" -parent $w_ -title "$refpix not found"}
		return $crpix
	    }
	}
    }


    # All fine, no pseudo image and CRPIX present:
    return $crpix
}


#############################################################
# Fetch DEC from the fits image
#############################################################

protected proc get_dec_image {} {
    # Find DEC from fits image.		

    set dec_img "UNKNOWN"
    set fits [$target_image_ hdu fits 1 ]
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 7 ] == "DEC     "} {
	    # replace multiple empty spaces by single spaces, then extract keyword value
	    regsub -all { +} $header { } header 
	    set dec_img [lindex [split $header " " ] 2 ]
	    break
	}
	if {[string range $header 0 7 ] == "CRVAL2  "} {
	    # replace multiple empty spaces by single spaces, then extract keyword value
	    regsub -all { +} $header { } header 
	    set dec_img [lindex [split $header " " ] 2 ]
	    break
	}
    }
    
    if { $dec_img == "UNKNOWN"} { 
	error_dialog "Error! Did not find DEC nor CRVAL2 keyword in $target_image_" 
	return 0.0
    }

    return [string trim $dec_img]
}


#############################################################
# Fetch ccdsum from fits image extension, not the PHU (Bryan Miller, 2013/03/18)
#############################################################

protected proc get_ccdsum_image {} {
    
    set ccdsum "NOT FOUND"
    set fits [$target_image_ hdu fits 1 ]
    foreach header [split $fits "\n" ] {
	if {[string range $header 0 5 ] == "CCDSUM"} {
	    set ccdsum [lindex [split [lindex [split $header "=" ] 1 ] "/" ] 0 ]
	    break
	}
    }
    
    return [string trim $ccdsum "' "]
}

#############################################################
# Return the name of this skycat instance. 
#############################################################

public method get_skycat {} {
    
    #debugStatement [string range $this 2 8 ]
    
    if {[string range $this 2 8 ] == ".skycat"} {
	return [string range $this 2 9 ]
    } else { 
	return 1
    }
}



#############################################################
# -- options --
#############################################################

# name of catalog
itk_option define -catalog catalog Catalog "" {
    # make sure we use full path name for local catalogs
    set f $itk_option(-catalog)
    if {[file exists $f] && "[string index $f 0]" != "/"} {
	set itk_option(-catalog) [pwd]/$f
    }
}

# type of catalog (catalog or archive)
itk_option define -catalogtype catalogType CatalogType "catalog"

# Optional unique id, used in searching for already existing catalog widgets.
itk_option define -id id Id ""

# Contains the type of file loaded in, Object Tbl, Master OT , or fitsFile 
itk_option define -catType catType CatType "1"

# Options indicating col's that exist
itk_option define -prior prior Prior "no"
itk_option define -stype stype SType "R"
itk_option define -xsize xsize XSize "no"
itk_option define -ysize ysize YSize "no"
itk_option define -tilt tilt Tilt "no"
itk_option define -uphead uphead UpHead {}
itk_option define -headingx headingx Headingx {}
itk_option define -pixs pixs Pixs "0.0"
itk_option define -xoffset xoffset Xoffset "no"
itk_option define -yoffset yoffset Yoffset "no"

# list of catalog column headings (from results of most recent query)
protected variable headings_ {}

# result from most recent query (list of rows)
protected variable info_ {}

# font to use for labels
#itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*
#itk_option define -labelfont labelFont LabelFont "Arial 12 bold"
itk_option define -labelfont labelFont LabelFont "Courier 14 bold"

# font to use for values
itk_option define -valuefont valueFont ValueFont -Adobe-helvetica-medium-r-normal--12*

# font to use for ra,dec labels
itk_option define -wcsfont wcsFont WcsFont -*-symbol-*-*-*-*-14-*-*-*-*-*-*-*

# set the width for displaying labels
itk_option define -labelwidth labelWidth LabelWidth 12

# set the anchor for labels
itk_option define -anchor anchor Anchor e

# flag: if true, run queries in foreground for better debugging
itk_option define -debug debug Debug 0

# set central wavelength
itk_option define -bw bw Bw ""

# vmAstroQuery widget used to manage search options
protected variable searchopts_ {}

# QueryResult widget used to display search results
protected variable results_ {}

# array(uppercase col name) of col index from catalog headings
protected variable col_

# pipe to read feedback during HTTP transfer
#protected variable rfd_

# pipe to write feedback during HTTP transfer
#protected variable wfd_

# current state: normal, disabled (i.e.: waiting)
protected variable state_ {normal}

# log file handle (used to log URLs)
protected variable logfile_name_ 

# flag: set at end of constructor
protected variable initialized_ 0

# index of this object in the instances_ array
protected variable instance_idx_ {}

# name of File menu widget
protected variable file_menu_

# name of Edit menu widget
protected variable edit_menu_

# name of Options menu widget
protected variable options_menu_

# name of Name Server menu widget
#protected variable ns_menu_

# flag: set when the column headings are changed between TCS and normal
protected variable reset_columns_ {0}

# currently selected object (Id field in row)
protected variable object_name_ {}

# count used for filename generation
#protected variable count_ 0

# flag: true if catalog is not an image server
protected variable iscat_ 1

# The detector ID of the image loaded into skycat.
protected variable DET_IMG_ ""

# The detector ID that the mask is intended for.
protected variable DET_SPEC_ ""

# The RA of the preimage
protected variable RA_IMG_ ""

# The DEC of the preimage
protected variable DEC_IMG_ ""

# Formatting variables. 

protected variable ot_text_ "black"
protected variable ot_bg_ "grey86"
protected variable ot_bg2_ "grey86"

protected variable button_text_ot_ "black"
protected variable button_text_active_ot_ "black"
protected variable button_bg_ot_ "grey86"
protected variable button_bg_active_ot_ "white"

protected variable catalog_text_ot_ "black"
protected variable catalog_text_active_ot_ "black"
protected variable catalog_bg_ot_ "white"
protected variable catalog_bg_active_ot_ "yellow"

#protected variable menu_bg_ot_ "firebrick"
protected variable menu_bg_ot_ #b33
protected variable menu_text_ot_ "white"
protected variable menu_bg_active_ot_ "white"
protected variable menu_text_active_ot_ "black"

protected variable odf_text_ "black"
protected variable odf_bg_ "grey86"
protected variable odf_bg2_ "grey86"

protected variable button_text_odf_ "black"
protected variable button_text_active_odf_ "black"
protected variable button_bg_odf_ "grey86"
protected variable button_bg_active_odf_ "white"

protected variable catalog_text_odf_ "black"
protected variable catalog_text_active_odf_ "black"
protected variable catalog_bg_odf_ "white"
protected variable catalog_bg_active_odf_ "yellow"

#protected variable menu_bg_odf_ "DodgerBlue3"
protected variable menu_bg_odf_ #36d
protected variable menu_text_odf_ "white"
protected variable menu_bg_active_odf_ "white"
protected variable menu_text_active_odf_ "black"


# -- common variables (common to all instances of this class) --

# C++ astrocat object used by static member procs
protected common astrocat_ [astrocat ::cat::.vmastrocat]

# A reference to the band_def_UI window. Useful if we want to destroy
# previous windows on new window loads.
protected common bw_ ""

# A reference to the mask design window, if open.
protected common spoc_ ""

# array mapping catalog name to widget/class name
protected common instances_

# instance count
protected common n_instances_ 0

# current instance name
protected common current_instance_ {}

# array(TopLevelWidget instance) of command used to update the
# Data-Servers menu. This is used to make it posible to update
# all instances of this menu in various top level windows.
protected common catalog_menu_info_

# Reference to the skycat image widget. 
protected common target_image_ image2

protected common home_ $::env(GMMPS)

# GMMPS configuration filename
protected common config_file_ [format "%s/.gmmps.cfg" $::env(HOME) ]

# Information about the detector being used. A list. 
# Will be set to 0 on construction.
# GMOS Order: Currently-Installed Instrument-Name Detector-Id Detector-Name Pixel-Scale Bandsize
# For F2:     Curently-Installed Instrument-Name Detector-Id Detector-Name Pixel-Scale
protected common detector_ 0

# flag used when plotting detector field layout
}
